local Actions = include("sim/btree/actions")
local btree = include("sim/btree/btree")
local cdefs = include("client_defs")
local speechdefs = include( "sim/speechdefs" )
local simdefs = include( "sim/simdefs" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )
local simquery = include( "sim/simquery" )
local array = include("modules/array")
local astar = include( "modules/astar" )
local astar_handlers = include( "sim/astar_handlers" )
local mainframe = include("sim/mainframe")
local guarddefs = include("sim/unitdefs/guarddefs")

----Altered functions----

local function doTrackerAlert(sim, unit)
    if unit:getTraits().trackerAlert then
        local tracker, text, x0, y0 = unpack( unit:getTraits().trackerAlert )
		sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat( STRINGS.UI.ALARM_ADD, tracker ),x=x0,y=y0,color={r=255/255,g=10/255,b=10/255,a=1}} )
		sim:trackerAdvance( tracker, text )
        unit:getTraits().trackerAlert = nil
    end
end

Actions.ShootAtTarget = function(sim, unit)
	if not unit:getBrain():getTarget() then
		return simdefs.BSTATE_FAILED
	end
	local target = unit:getBrain():getTarget()	--we should only have the target if we can see them
	local canShoot = unit:canReact() or unit:getBrain():getSenses():hasLostTarget(target) --if they run into cover and then back out, they need to get shot
	if (canShoot and unit:isAiming() and target:isValid() and not unit:getTraits().noAttack and not (unit:getTraits().noReactionShots and sim:getCurrentPlayer() ~= unit:getPlayerOwner())) or unit:getTraits().skipOverwatch then
		local fixKO = false
		if not target:getTraits().canKO and not target:getTraits().isDrone and unit:getTraits().dartgun and not unit:getTraits().hasFailedToKOPedler then
			target:getTraits().canKO = true
			fixKO = true
		end
		
		local abilityDef = unit:ownsAbility( "shootSingle" )
        if abilityDef and abilityDef:canUseAbility( sim, unit, unit, target:getID() ) then
			if (unit:getTraits().removeAttackAfterUse) then
				unit:getTraits().hasSkippedAttack = 2
				unit:getTraits().skipOverwatch = false
				unit:getTraits().noAttack = true
			end
			if fixKO then
				fixKO = false
				target:getTraits().canKO = false
				unit:getTraits().hasFailedToKOPedler = {
					onTrigger = function( listener, sim, evType, evData )
						if evData:isNPC() then
							sim:removeTrigger( simdefs.TRG_START_TURN, listener )
							unit:getTraits().hasFailedToKOPedler = nil
						end
					end
				}
				
				sim:addTrigger( simdefs.TRG_START_TURN, unit:getTraits().hasFailedToKOPedler )
			end
			
		    abilityDef:executeAbility( sim, unit, unit, target:getID() )
		    --This could result in getting overwatch shot in return
		    if not unit:isValid() then
		    	return simdefs.BSTATE_FAILED
		    end
		    if target:isDead() or target:isKO() or not target:isValid() then
				if target:getTraits().mainframe_turret then
					unit:getSim():emitSpeech( unit, speechdefs.COMBAT_TURRETDESTROYED )
				else
					unit:getSim():emitSpeech( unit, speechdefs.COMBAT_TARGETDESTROYED )
				end
				
				return simdefs.BSTATE_COMPLETE
		    end
        end
		
		if fixKO then
			target:getTraits().canKO = false
			unit:getTraits().hasFailedToKOPedler = {
				onTrigger = function( listener, sim, evType, evData )
					if evData:isNPC() then
						sim:removeTrigger( simdefs.TRG_START_TURN, listener )
						unit:getTraits().hasFailedToKOPedler = nil
					end
				end
			}
			
			sim:addTrigger( simdefs.TRG_START_TURN, unit:getTraits().hasFailedToKOPedler )
		end
	end
	
	if not unit:isAiming() then
		local abilityDef = unit:ownsAbility("overwatch")
		if abilityDef and not unit:getTraits().noAttack then
			abilityDef:executeAbility(sim, unit, unit, target:getID())
			doTrackerAlert(sim, unit)
			if sim:getCurrentPlayer() == unit:getPlayerOwner() then
				return simdefs.BSTATE_WAITINGFORPCTURN
			end
		end
	end
	
    return simdefs.BSTATE_WAITINGFORCORPTURN
end

function Actions.MoveBesideTarget:getDestination()
	local target = self.unit:getBrain():getTarget()
	local x, y = target:getLocation()
	
	if simquery.canUnitReach(self.sim, self.unit, x, y) then	--already close enough
		local fromX, fromY = self.unit:getLocation()
		assert(fromX and fromY)
		local facing = simquery.getDirectionFromDelta(x - fromX, y - fromY)
		return {x = fromX, y = fromY, facing=facing, meleeTargetX = x, meleeTargetY = y}
	end
	
	local neighbour = simquery.findNearestEmptyReachableCell(self.sim, x, y, self.unit )
	if neighbour then
		local facing = simquery.getDirectionFromDelta(x - neighbour.x, y - neighbour.y)
		return {x = neighbour.x, y = neighbour.y, facing=facing, meleeTargetX = x, meleeTargetY = y}
	end
end

local oldExecutePath = Actions.MoveTo.executePath

function Actions.MoveTo:executePath(unit)
	local path = unit:getPather():getPath(unit)
	local nodes = path and path.path and path.path:getNodes()
	local goaldir
	
	if guarddefs["ce_dummy_pather"] and nodes and #nodes > 0 then
		local endNode = nodes[#nodes]
		
		if path.goalx ~= endNode.location.x or path.goaly ~= endNode.location.y then
			goaldir = path.goaldir
			path.goaldir = nil
		end
	end
	
	local result = oldExecutePath(self, unit)
	
	if path and goaldir then
		path.goaldir = goaldir
	end
	
	if goaldir and result == simdefs.BSTATE_COMPLETE then
		unit:updateFacing( goaldir )
	end
	
	return result
end

for k, action in pairs(Actions) do
	if type(action) == "table" and action.executePath == oldExecutePath then
		action.executePath = Actions.MoveTo.executePath
	end
end

local oldDoLookAround = Actions.DoLookAround

function Actions.DoLookAround(sim, unit)
	if unit:getTraits().hasLookedAround and guarddefs["ce_dummy_pather"] and not unit:getTraits().recursiveLookAround then
		return simdefs.BSTATE_COMPLETE
	end
	
	unit:getTraits().hasLookedAround = true
	local cell = sim:getCell(unit:getLocation())
	local exitDir = unit:getFacing()
	local exit = cell.exits[exitDir]
	
	if exit and exit.door and exit.closed and not exit.locked then
		sim:modifyExit(cell, exitDir, simdefs.EXITOP_OPEN, unit)		
	end
	
	local x0, y0 = unit:getLocation()
	unit:getTraits().tellBuddyCell = nil
	if unit:isValid() and not unit:isKO() then
		local target = unit:getTraits().ce_buddyToTell
		local brain = unit:getBrain()
		local interest = brain and brain:getInterest()
		
		if target and unit:isAlerted() and sim:getParams().difficultyOptions.ce_tell_buddy and interest and interest.reason == simdefs.REASON_TELL_BUDDY then
			local x1, y1 = target:getLocation()
			
			local ALERT_BUDDY_RANGE = 3*3
			
			if not target:isAlerted() and not target:isDown() and x0 and x1 and mathutil.distSqr2d(x0,y0,x1,y1) <= ALERT_BUDDY_RANGE and not target:getTraits().ce_yelledAt then
				local raycastX, raycastY = sim:getLOS():raycast( x0, y0, x1, y1)
				if (raycastX == x1 and raycastY == y1) or sim:canUnitSeeUnit(unit, target) then
					target:getTraits().ce_yelledAt = true
					
					local facing = simquery.getDirectionFromDelta(x1 - x0, y1 - y0)
					unit:setFacing(facing)
					sim:dispatchEvent( simdefs.EV_UNIT_TURN, { unit = unit, facing = facing } )
					sim:emitSpeech( unit, speechdefs.HUNT_LOSTTARGET )
					sim:dispatchEvent( simdefs.EV_WAIT_DELAY, cdefs.SECONDS*1.8)
					
					for i, alertUnit in pairs(sim:getNPC():getUnits()) do
						local x2, y2 = alertUnit:getLocation()
						if x2 and alertUnit:getBrain() and not alertUnit:isAlerted() and not alertUnit:getTraits().noInterestDistraction and mathutil.distSqr2d(x0, y0, x2, y2 ) <= ALERT_BUDDY_RANGE then
							alertUnit:setAlerted(true)
							alertUnit:useMP( alertUnit:getMP(), sim )
							sim:getNPC():createOrJoinHuntSituation(alertUnit)
						end
					end
				end
			end
		end
		if not unit:getTraits().no_look_around and unit:getTraits().lookaroundRange and unit:getTraits().high_frequency_scanner	then
			if unit:isAlerted() then
				sim:emitSpeech( unit, speechdefs.HUNT_SEARCH )
			else
				sim:emitSpeech( unit, speechdefs.INVESTIGATE_SEARCH )
			end

			unit:setAiming( false )
			unit:getTraits().lookingAround = true

			local cells = simquery.fillCircle( sim, x0, y0, unit:getTraits().lookaroundRange, 0)
			for i, cell in ipairs(cells) do
				if cell.x ~= x0 or cell.y ~= y0 then
					sim:scanCell( unit, cell, true )
				end
			end
			if #cells > 0 then
				sim:dispatchEvent( simdefs.EV_UNIT_LOOKAROUND, { unit = unit, part="scan", cells=cells} )
			end
			sim:processReactions(unit)
			
			unit:getTraits().lookingAround = nil
			if unit:getTraits().interrupted then
				-- ccc: don't want to walk any further if we saw something, so consume all MP.
				--- (but shouldn't there a way to prevent further behaviours just by returning the correct BSTATE_FAILED?)
				if unit:isValid() then
					unit:useMP( unit:getMP(), sim )
				end
				return simdefs.BSTATE_FAILED
			end

			return simdefs.BSTATE_COMPLETE
		else
			return oldDoLookAround(sim, unit)
		end
	end
end

local oldReactToInterest = Actions.ReactToInterest

function Actions.ReactToInterest( sim, unit )
	local interest = unit:getBrain():getInterest()
	if not interest then
		return simdefs.BSTATE_FAILED
	end
	if interest.noticed then
		return simdefs.BSTATE_COMPLETE
	end
	
	if not unit:getTraits().pacifist and not unit:getTraits().ce_calmed and not unit:getTraits().isDrone and not unit:isAlerted() and sim:getParams().difficultyOptions.ce_suspicious and sim:getParams().difficultyOptions.ce_suspicious > 0 then
		local ignoredReason = false
		for i, ignore_reason in pairs(simdefs.NON_SUSPICIOUS_REASONS) do
			if ignore_reason == interest.reason then
				ignoredReason = true
			end
		end
		if not ignoredReason then
			if not unit:getTraits().ce_suspicious then
				unit:getTraits().ce_suspicious = 0
			end
			if unit:getTraits().ce_suspicious == sim:getParams().difficultyOptions.ce_suspicious then
				unit:getTraits().ce_suspicious = nil
				unit:setAlerted(true)
				local x0, y0 = unit:getLocation()
				local x1 = unit:getBrain():getInterest().x
				local y1 = unit:getBrain():getInterest().y
				unit:getBrain():getInterest().reason = simdefs.REASON_PATROLCHANGED
				unit:getBrain():reset()
				unit:getBrain():spawnInterest(x1, y1, simdefs.SENSE_RADIO, simdefs.REASON_WITNESS, unit)
				unit:getSim():emitSpeech( unit, speechdefs.COMBAT_NEWTARGET )
			elseif unit:getTraits().ce_suspicious < sim:getParams().difficultyOptions.ce_suspicious then
				unit:getTraits().ce_suspicious = unit:getTraits().ce_suspicious + 1
			end
		end
	end
	
	if interest.reason == simdefs.REASON_TELL_BUDDY then
		local target = unit:getTraits().ce_buddyToTell
		if target and not target:isAlerted() and not target:isKO() then
			-- Yell at our buddy to warn them about the scawy intwudews!
			sim:dispatchEvent( simdefs.EV_UNIT_ALERTED, { unitID = unit:getID() } )
			sim:emitSpeech( unit, speechdefs.COMBAT_NEWTARGET )
			
			if unit:canReact() then
				unit:turnToFace(interest.x, interest.y)
			end
		
			interest.noticed = true
			sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*1.5)
		
			target:turnToFace( unit:getLocation() )
			sim:processReactions( target )
		end
		
		return simdefs.BSTATE_COMPLETE
	elseif interest.reason == simdefs.REASON_SCANNED_DEVICE and unit:getTraits().gotConsoleInterest then
		--Avoid turning to face the interest immediately after using a console
		sim:dispatchEvent( simdefs.EV_UNIT_ALERTED, { unitID = unit:getID() } )
		unit:getSim():emitSpeech( unit, speechdefs.INVESTIGATE_GENERIC )
		interest.noticed = true
		
		return simdefs.BSTATE_WAITINGFORPCTURN
	elseif interest.reason == simdefs.REASON_YELLED_AT then
		-- Turn to face the yeller
		if interest.delay then
			sim:dispatchEvent( simdefs.EV_WAIT_DELAY, 60*1.25)
		end
		if interest.facing then
			unit:updateFacing( interest.facing )
		end
		
		return oldReactToInterest(sim, unit)
	else
		return oldReactToInterest(sim, unit)
	end
end

----Ghost Unit----

local function canBeInterrupted(self)
	if self:getTraits().interrupted or self:getTraits().noInterrupt then
		return false
	end
	return self:getTraits().movePath or self:getTraits().modifyingExit or self:getTraits().lookingAround or self:getTraits().throwing
end

function Actions.ce_ignoreOtherTargets(sim,unit)
	local x,y = unit:getBrain():getTarget():getLocation()
	unit:getBrain():getSenses():addInterest(x, y, simdefs.SENSE_SIGHT, simdefs.REASON_SHARED, unit:getBrain():getTarget())
	unit:getTraits().noPeripheralDetection = true
	unit.canBeInterrupted = canBeInterrupted
	unit:getTraits().noInterrupt = true
	return simdefs.BSTATE_COMPLETE
end

function Actions.ce_turnInvisible(sim,unit)
	if unit:getTraits().usesCloaks and (unit:getTraits().cloakCooldown or 0) <= 0 then
		unit:getTraits().cloakCooldown = unit:getTraits().cloakCooldownMax
		unit:setInvisible( true, unit:getTraits().cloakDurationMax + (sim:getCurrentPlayer():isPC() and 1 or 0) )
	end
	return simdefs.BSTATE_COMPLETE
end

function Actions.ce_resetTargeting(sim,unit)
	unit:getTraits().noPeripheralDetection = nil
	unit:getTraits().noInterrupt = nil
	return simdefs.BSTATE_COMPLETE
end

----Support Drone----

local setAiming

Actions.ThrowGrenadeAtTarget = function(sim, unit)
	if not setAiming then
		setAiming = unit.setAiming
	end
	unit.setAiming = function(self,aiming)
		setAiming(self,aiming)
		if not aiming then
			self:getTraits().grenadeAiming = nil
		end
	end
	if not unit:getBrain():getTarget() then
		return simdefs.BSTATE_FAILED
	end
	local target = unit:getBrain():getTarget()	--we should only have the target if we can see them
	if unit:canReact() and unit:getTraits().grenadeAiming then
    	local abilityDef, grenadeUnit = unit:ownsAbility("throw")
		local targetCell = sim:getCell( target:getLocation() )
		
        if abilityDef and grenadeUnit and targetCell and abilityDef:canUseAbility( sim, grenadeUnit, unit, {targetCell.x, targetCell.y} ) then
		    sim:dispatchEvent( simdefs.EV_UNIT_START_SHOOTING, { unitID = unit:getID(), ignoreAgentrig = true,targetUnitID = target:getID() } )	
			local mp = unit:getMP()
			abilityDef:executeAbility( sim, grenadeUnit, unit, {targetCell.x, targetCell.y} )
			unit:setAiming(false)
			unit:getTraits().mp = mp
		    --This could result in getting overwatch shot in return
		    if not unit:isValid() then
		    	return simdefs.BSTATE_FAILED
		    end
		    if target:isDead() or target:isKO() or not target:isValid() then
				if target:getTraits().mainframe_turret then
					unit:getSim():emitSpeech( unit, speechdefs.COMBAT_TURRETDESTROYED )
				else
					unit:getSim():emitSpeech( unit, speechdefs.COMBAT_TARGETDESTROYED )
				end
		    end
		    return simdefs.BSTATE_COMPLETE
        end
	else
		if not unit:getTraits().grenadeAiming then
			unit:getTraits().grenadeAiming = true
			doTrackerAlert(sim, unit)
			if sim:getCurrentPlayer() == unit:getPlayerOwner() then
				return simdefs.BSTATE_WAITINGFORPCTURN
			end
		end
	end
    return simdefs.BSTATE_WAITINGFORCORPTURN
end

----Drone Engineer----

Actions.ShouldMonitorDrone = function(sim, unit)
	local interest = unit:getBrain():getInterest()
	if interest and interest.reason == simdefs.REASON_FOUNDDRONE and interest.sourceUnit:isValid() and not interest.sourceUnit:isDead() then
		return true
	end
	return false
end

Actions.MoveBesideInterest = class(Actions.MoveTo, function(self, name)
	btree.BaseAction.init(self, name or "MoveBesideInterest")
end)

function Actions.MoveBesideInterest:getDestination()
	local unit = self.unit
	local interest = unit:getBrain():getInterest()
	local target = interest.sourceUnit
	local x, y = interest.x, interest.y
	local x1, y1 = unit:getLocation()
	
	if simquery.canReach(self.sim, x1, y1, x, y) and (x ~= x1 or y ~= y1) then--already close enough
		return nil
	end
	
	if x1 == x and y1 == y then
		local dx, dy = simquery.getDeltaFromDirection( unit:getFacing() )
		x1 = x1 + dx
		y1 = y1 + dy
	end

	local resultCell
    for _, dir in ipairs( simdefs.DIR_SIDES ) do
        local dx, dy = simquery.getDeltaFromDirection( dir )
	    local testCell = self.sim:getCell( x + dx, y + dy )
	    if testCell and simquery.isOpenExit( testCell.exits[ simquery.getReverseDirection( dir ) ]) and simquery.canPath( self.sim, unit, nil, testCell ) then
			if not resultCell then
				resultCell = testCell
			else
				local cell1Dist = mathutil.dist2d( testCell.x, testCell.y, x1, y1 )
				local cell2Dist = mathutil.dist2d( resultCell.x, resultCell.y, x1, y1 )
				if cell1Dist < cell2Dist then
					resultCell = testCell
				end
			end
		end
	end
	
	if resultCell then
		local facing = simquery.getDirectionFromDelta(x - resultCell.x, y - resultCell.y)
		return {x = resultCell.x, y = resultCell.y, facing=facing}
	end
end

function Actions.RepairDrone(sim, unit)
	local target = unit:getBrain():getInterest().sourceUnit
	if target and (target:isDown() or target:getTraits().takenDrone) then
		local x, y = target:getLocation()
		if x and simquery.canUnitReach(sim, unit, x, y) then
			local x1,y1 = unit:getLocation()
			local facing = simquery.getDirectionFromDelta( x - x1, y - y1 )
			unit:updateFacing(facing)
			sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = unit:getID(), targetID= target:getID(), facing = facing, sound=simdefs.SOUNDPATH_USE_CONSOLE, soundFrame=10 } )
			
			target:getTraits().controlTimer = 0
			sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.REPAIRED,x=x,y=y,color={r=255/255,g=178/255,b=102/255,a=1 }} )
			sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = target, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params={color ={{symbol="wall",r=1,g=1,b=0,a=1}} }} )
			target:loseControl( sim )
			target:setKO( sim, nil )
			
			return simdefs.BSTATE_COMPLETE
		end
	end
	
	return simdefs.BSTATE_FAILED
end

----System Admin----

function Actions.MeleeReboot(sim, unit)
	local target = unit:getBrain():getInterest().sourceUnit
	if target then
		local x, y = target:getLocation()
		if x and simquery.canUnitReach(sim, unit, x, y) then
			local x1,y1 = unit:getLocation()
			local facing = simquery.getDirectionFromDelta( x - x1, y - y1 )
			unit:updateFacing(facing)
			sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = unit:getID(), targetID= target:getID(), facing = facing, sound=simdefs.SOUNDPATH_USE_CONSOLE, soundFrame=10 } )
			--Hack in!
			mainframe.revertIce( sim, target )
			if unit:getTraits().scanUnits then
				array.removeElement( unit:getTraits().scanUnits, target )
			end
			
			return simdefs.BSTATE_COMPLETE
		end
	end
	
	return simdefs.BSTATE_FAILED
end

Actions.ShouldMeleeReboot = function(sim, unit)
	local interest = unit:getBrain():getInterest()
	if interest and interest.reason == simdefs.REASON_HACKEDDEVICE then
		--We detected something supicious! Better check a console later...
		unit:getTraits().shouldCheckConsole = true
		unit:getTraits().scanUnits = {}
		return true
	end
	if interest and interest.reason == simdefs.REASON_SCANNED_DEVICE then
		return true
	end
	
	--We've just checked a device we previously scanned from a console, now check the next (closest) one
	if interest == nil and unit:getTraits().scanUnits and #unit:getTraits().scanUnits > 0 then
		local startcell = sim:getCell( unit:getLocation() )
		local bestUnit, bestCost
		local pather = astar.AStar:new(astar_handlers.aihandler:new(unit) )
	
		for i, scanUnit in pairs(unit:getTraits().scanUnits) do
			local x1, y1 = scanUnit:getLocation()
			
			if x1 then
				for _, dir in ipairs( simdefs.DIR_SIDES ) do
					local dx, dy = simquery.getDeltaFromDirection( dir )
					local endcell = sim:getCell( x1 + dx, y1 + dy )
					if endcell and simquery.isOpenExit( endcell.exits[ simquery.getReverseDirection( dir ) ]) and simquery.canPath( sim, nil, nil, endcell ) then
						local cost = pather:findNodeCost( startcell, endcell, endcell )
						if cost then
							if bestUnit == nil or cost < bestCost then
								bestUnit = scanUnit
								bestCost = cost
							end
						end
					end
				end	
			end
		end
		
		if bestUnit then
			local x1, y1 = bestUnit:getLocation()
			local interest = unit:getBrain():getSenses():addInterest(x1, y1, simdefs.SENSE_RADIO, simdefs.REASON_SCANNED_DEVICE, bestUnit)
			interest.noticed = true
			return true
		else
			--Device is inaccessible, ignore
			unit:getTraits().scanUnits = {}
		end
	end
	
	return false
end

Actions.MoveToConsole = class(Actions.MoveTo, function(self, name)
	btree.BaseAction.init(self, name or "MoveToConsole")
end)

function Actions.MoveToConsole:getDestination()
	local interest = self.unit:getBrain():getInterest()
	return {x = interest.checkCell.x, y = interest.checkCell.y, facing = interest.facing}
end

function Actions.CheckConsole(sim, unit)
	local target = unit:getBrain():getInterest().sourceUnit
	local x0, y0 = unit:getLocation()
	local x1, y1 = target:getLocation()
	if x1 and simquery.canUnitReach(sim, unit, x1, y1) then
		local facing = simquery.getReverseDirection( target:getFacing() )
		unit:updateFacing(facing)
		sim:dispatchEvent( simdefs.EV_UNIT_USECOMP, { unitID = unit:getID(), facing = facing, sound=simdefs.SOUNDPATH_USE_CONSOLE, soundFrame=10 } )
		
		if target:getTraits().mainframe_status ~= "active" then
			--How strange, the console won't boot... did one of those incompetent guards touch the emp packs again?
			if not unit:getTraits().knownConsoleDefects then
				unit:getTraits().knownConsoleDefects = {}
			end
			
			unit:getTraits().knownConsoleDefects[target:getID()] = true
			
			sim:emitSpeech( unit, speechdefs.INVESTIGATE_GENERIC )
			Actions.DoLookAround(sim, unit)
			return simdefs.BSTATE_FAILED
		else
			if sim:canPlayerSee( sim:getPC(), x1, y1 ) then
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound = "SpySociety/HUD/gameplay/node_capture",x=x1,y=y1} )
				sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = target, kanim = "gui/hud_fx", symbol = "wireless_console_takeover", anim = "idle" } )
			end
			
			unit:getTraits().shouldCheckConsole = nil
			
			local startcell = sim:getCell( x0, y0 )
			local scanUnits = {}
			local bestUnit, bestCost
			local pather = astar.AStar:new(astar_handlers.aihandler:new(unit) )
			
			--Scan all devices
			sim:forEachUnit(function(scanUnit)
				if mainframe.canRevertIce( sim, scanUnit ) then
					--Note, we keep track of the device even if we can't access it, in case it becomes available later
					table.insert(scanUnits, scanUnit)
					local x2, y2 = scanUnit:getLocation()
					
					for _, dir in ipairs( simdefs.DIR_SIDES ) do
						local dx, dy = simquery.getDeltaFromDirection( dir )
						local endcell = sim:getCell( x2 + dx, y2 + dy )
						if endcell and simquery.isOpenExit( endcell.exits[ simquery.getReverseDirection( dir ) ]) and simquery.canPath( sim, nil, nil, endcell ) then
							local cost = pather:findNodeCost( startcell, endcell, endcell )
							if cost then
								if bestUnit == nil or cost < bestCost then
									bestUnit = scanUnit
									bestCost = cost
								end
							end
						end
					end
				end
			end)
			
			--Remember the data we got for later
			unit:getTraits().scanUnits = scanUnits
			
			if bestUnit then
				local x2, y2 = bestUnit:getLocation()
				local interest = unit:getBrain():getSenses():addInterest(x2, y2, simdefs.SENSE_RADIO, simdefs.REASON_SCANNED_DEVICE, bestUnit)
				
				--React immediately
				unit:getTraits().gotConsoleInterest = true
				unit:getBrain():reset()
				return simdefs.BSTATE_RUNNING
			end
			return simdefs.BSTATE_COMPLETE
		end
	end
	
	return simdefs.BSTATE_FAILED
end

Actions.ShouldCheckConsole = function(sim, unit)
	local interest = unit:getBrain():getInterest()
	if interest and interest.reason == simdefs.REASON_CHECK_CONSOLE then
		return true
	end
	return false
end

Actions.AddCheckConsoleInterest = function(sim, unit)
	--By this point we've finished investigation and are prepared to return to our patrol, but if any mainframe anomalies were detected, we should check with a console first
	if unit:getTraits().shouldCheckConsole then
		local x0, y0 = unit:getLocation()
		local startcell = sim:getCell(x0, y0)
		local bestCell, bestConsole, bestCost
		local pather = astar.AStar:new(astar_handlers.aihandler:new(unit) )
		
		--Find the nearest console
		sim:forEachUnit(function(targetUnit)
			if targetUnit:getTraits().mainframe_console and not (unit:getTraits().knownConsoleDefects and unit:getTraits().knownConsoleDefects[targetUnit:getID()]) then
				local dir = targetUnit:getFacing()
				local x0, y0 = targetUnit:getLocation()
				local x1, y1 = simquery.getDeltaFromDirection(dir)
				local endcell = sim:getCell( x0 + x1, y0 + y1 )
				--We must be in front of the console
			
				local cost = pather:findNodeCost( startcell, endcell, endcell )
				if cost then
					if bestCell == nil or cost < bestCost then
						bestConsole = targetUnit
						bestCell = endcell
						bestCost = cost
					end
				end
			end
		end)
		
		if bestCell then
			local x, y = bestConsole:getLocation()
			local facing = simquery.getReverseDirection( bestConsole:getFacing() )
			
			local interest = unit:getBrain():getSenses():addInterest(x, y, simdefs.SENSE_RADIO, simdefs.REASON_CHECK_CONSOLE, bestConsole)
			if interest then
				interest.checkCell = bestCell
				interest.noticed = true
				interest.facing = facing
				
				--React immediately
				unit:getBrain():reset()
				return simdefs.BSTATE_RUNNING
			end
		end
	end
	
	return simdefs.BSTATE_FAILED
end