local aiplayer = include("sim/aiplayer")
local util = include( "modules/util" )
local simquery = include( "sim/simquery" )
local unitdefs = include( "sim/unitdefs" )
local simdefs = include( "sim/simdefs" )
local enforcers = {
	{
		{"npc_guard_enforcer_reinforcement", 100},
		{"ce_inspector", 75, max_one = true},
		{"ce_guard_enforcer_reinforcement_drone", 75},
	},
	{
		{"npc_guard_enforcer_reinforcement", 100},
		{"ce_inspector", 75, max_one = true},
		{"ce_guard_enforcer_reinforcement_drone", 75},
	},
	{
		{"npc_guard_enforcer_reinforcement_2", 100},
		{"ce_inspector_2", 100, max_one = true},
		{"ce_guard_enforcer_reinforcement_drone_2", 100},
		{"ce_guard_enforcer_reinforcement_heavy", 100},
	},
	{
		{"npc_guard_enforcer_reinforcement_2", 100},
		{"ce_inspector_2", 100, max_one = true},
		{"ce_guard_enforcer_reinforcement_drone_2", 100},
		{"ce_guard_enforcer_reinforcement_heavy", 100},
	},
	{
		{"npc_guard_enforcer_reinforcement_2", 100},
		{"ce_inspector_2", 100, max_one = true},
		{"ce_guard_enforcer_reinforcement_drone_2", 100},
		{"ce_guard_enforcer_reinforcement_heavy", 100},
	},
	{
		{"ce_guard_enforcer_reinforcement_3", 100},
		{"ce_inspector_3", 100, max_one = true},
		{"ce_guard_enforcer_reinforcement_drone_3", 100},
		{"ce_guard_enforcer_reinforcement_heavy_2", 100},
	}
}

aiplayer.spawnGuards = function(self, sim, unitType, numGuards)
	local worldgen = include( "sim/worldgen" )
	local cells = sim:getCells( "guard_spawn" )
    if cells then
        cells = util.tdupe( cells )
	    local i = 1
	    while i <= #cells do
		    if not simquery.canStaticPath( sim, nil, nil, cells[i]) or simquery.checkDynamicImpass(sim, cells[i]) then
			    table.remove( cells, i )
		    else
			    i = i + 1
		    end
	    end
    end
    
    numGuards = math.min(numGuards, #cells)
 	local units = {}

    if numGuards > 0 then
		local unitData = unitdefs.lookupTemplate( unitType )
		if unitData.traits.enforcer then
			if unitdefs.lookupTemplate("ce_dummy_omni") and sim:hasObjective( "security_hub" ) and sim:hasObjective( "security_hub" ).current == 2 and sim:getParams().missionEvents and sim:getParams().missionEvents.advancedAlarm and not sim:getTags().spawnedHubGuard then
				sim:getTags().spawnedHubGuard = true
				local wt = util.weighted_list( sim._patrolGuard )
				unitType = wt:getChoice( sim:nextRand( 1, wt:getTotalWeight() ))
			elseif unitdefs.lookupTemplate("ce_inspector") then
				local enforcer_list = enforcers[math.min(sim._params.difficulty,#enforcers)]
				local wt = util.weighted_list()
				
				for i, choice in ipairs(enforcer_list) do
					local count = false
					if choice.max_one then
						for i, unit in pairs(self:getUnits()) do
							if unit:getUnitData().id == choice[1] then
								count = true
								break
							end
						end
					end
					
					if not count then
						wt:addChoice( choice[1], choice[2] )
					end
				end
				
				unitType = wt:getChoice( sim:nextRand( 1, wt:getTotalWeight() ))
			end
		end
		for i = 1, numGuards do
			if (sim._patrolGuard[unitType] or (sim._patrolGuard[1] and util.tnext( sim._patrolGuard, function(v) return v[1] == unitType end )())) and (sim._patrolGuard._unique or ((unitdefs.lookupTemplate("ce_dummy_omni") or unitdefs.lookupTemplate("ce_omni_hacker")) and (sim._patrolGuard == worldgen.OMNI_GUARD or sim._patrolGuard == worldgen.OMNI_GUARD_FIX))) then
				local spawnedThreaths = {}
				for i, unit in pairs(self:getUnits()) do
					if unit:getTraits().isGuard and not unit:isDead() then
						spawnedThreaths[unit:getUnitData().id] = (spawnedThreaths[unit:getUnitData().id] or 0) + 1
					end
				end
				local patrolGuard
				while true do
					patrolGuard = util.tdupe(sim._patrolGuard)
					patrolGuard._unique = nil
					for i, v in pairs(patrolGuard) do
						if (spawnedThreaths[type(i) == "string" and i or patrolGuard[i][1]] or 0) > 0 then
							patrolGuard[i] = nil
						end
					end
					local storageType
					if util.tcount(patrolGuard) == 0 then
						for i, v in pairs(spawnedThreaths) do
							spawnedThreaths[i] = v-1
						end
					else
						if type(next(patrolGuard)) == "number" then
							local newPatrolGuard = {}
							for i, v in pairs(patrolGuard) do
								newPatrolGuard[#newPatrolGuard + 1] = v
							end
							patrolGuard = newPatrolGuard
						end
						break
					end
				end
				local wt = util.weighted_list(patrolGuard)
				unitType = wt:getChoice( sim:nextRand( 1, wt:getTotalWeight() ))
			end
			--sim:getNPC():doTrackerSpawn( sim, simdefs.TRACKER_GUARDS[stage], templateName, true )
			table.insert(units, self:createGuard(sim, unitType ))
		end
		
	   for i,unit in ipairs(units)do
			if unit:getTraits().ReinforcementPWROnHand then
				unit:getTraits().PWROnHand = unit:getTraits().ReinforcementPWROnHand
				unit:getTraits().cashOnHand = nil
			end
	   
		    if cells and #cells > 0 then
				if unit:getTraits().mainframe_ice_based_on_level then
					unit:getTraits().mainframe_ice = math.ceil(math.ceil((sim._params.difficulty/2+1)*(unit:getTraits().hardmode and 1.2 or 1)))
					unit:getTraits().mainframe_iceMax = unit:getTraits().mainframe_ice
				end
			    local cell = table.remove( cells, sim:nextRand( 1, #cells ))		   
			    sim:warpUnit(unit, cell)
			    self:returnToIdleSituation(unit)
		    end
		end

	    if #units == 1 then
	 	    sim:dispatchEvent( simdefs.EV_SHOW_DIALOG, { dialog = "threatDialog", dialogParams = { units[1] }} )
	    end

	 	for i,unit in ipairs(units)do
			unit:getTraits().spawnedGuard = true
	  		sim:dispatchEvent( simdefs.EV_UNIT_APPEARED, { unitID = unit:getID() } )		
		 	sim:dispatchEvent( simdefs.EV_TELEPORT, { units = { unit }, warpOut = false } )
		    sim:getPC():glimpseUnit( sim, unit:getID() )
		end
	end
    return units
end

function aiplayer:tickBrain(unit)
    local thought = nil
	if unit:getBrain() and unit:isValid() then

		self:updateSenses(unit)

		if unit:isKO() or unit:isDead() then
			unit:getBrain():reset()
		elseif unit:getBrain():shouldTick() then
			thought = unit:getBrain():think()
		elseif not unit:getBrain().thinking and unit:getPather() and unitdefs.lookupTemplate("ce_dummy_pather") then
			--Fix for paths not updating immediately when an interest is moved
			local path = unit:getPather():getPath(unit)
			local dest = unit:getBrain():getDestination()
			if path and dest then
				if path.goalx ~= dest.x or path.goaly ~= dest.y then
					thought = unit:getBrain():think()
				end
			end
		end

	end
    return thought
end