local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local mathutil = include( "modules/mathutil" )
local abilityutil = include( "sim/abilities/abilityutil" )
local targets = include( "hud/targeting" )
local inventory = include("sim/inventory")
local speechdefs = include("sim/speechdefs")
local serverdefs = include( "modules/serverdefs" )
local melee = include( "sim/abilities/melee" )
local actions = include("sim/btree/actions")

local melee_tooltip = class()

function melee_tooltip:init( hud, ability, abilityOwner, abilityUser, targetUnitID )
	self._hud = hud
	self._ability = ability
	self._abilityOwner = abilityOwner
	self._abilityUser = abilityUser
	self._targetUnitID = targetUnitID
end

function melee_tooltip:setPosition( wx, wy )
	self._panel:setPosition( self._hud._screen:wndToUI( wx, wy ))
end

function melee_tooltip:getScreen()
	return self._hud._screen
end

function melee_tooltip:activate( screen )
	local combat_panel = include( "hud/combat_panel" )
	local sim = self._hud._game.simCore

	self._panel = combat_panel( self._hud, self._hud._screen )
	
	self._panel:refreshCEMelee( self._ability, self._abilityOwner, self._abilityUser, sim:getUnit( self._targetUnitID ))
end

function melee_tooltip:deactivate()
	self._hud._game.boardRig:getUnitRig( self._targetUnitID )._prop:setRenderFilter( nil )
	self._panel:setVisible( false )
end

local ce_melee = {
	name = STRINGS.ABILITIES.MELEE,
	profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-action_ko.png",
	usesAction = true,
	alwaysShow = true,
	getName = function( self, sim, unit )
		return self.name
	end,

	onTooltip = function( self, hud, sim, abilityOwner, abilityUser, targetUnitID )
		return melee_tooltip( hud, self, abilityOwner, abilityUser, targetUnitID )
	end,
		
	acquireTargets = melee.acquireTargets,

	onSpawnAbility = function( self, sim, unit )
		sim:addTrigger( simdefs.TRG_OVERWATCH, self, unit )		
		sim:addTrigger( simdefs.TRG_UNIT_NEWINTEREST, self, unit )
		sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
		sim:addTrigger( "CE_UNIT_STOP_WALKING", self, unit )
	end,

	onDespawnAbility = function( self, sim, unit )
		sim:removeTrigger( simdefs.TRG_OVERWATCH, self )
		sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
		sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
		sim:removeTrigger( "CE_UNIT_STOP_WALKING", self )
	end,
	
	onTrigger = function( self, sim, evType, evData, userUnit )
		if not userUnit or not userUnit:isValid() or userUnit:isDown() or userUnit:getTraits().ce_meleeaiming or userUnit:getTraits().ce_cantmelee then
			return
		end
		
		local userPlayer = userUnit:getPlayerOwner()
		local x0,y0 = userUnit:getLocation()
		
		if userUnit:isPC() then
			--We are mind controlled! Run the behaviour of meleeOverwatch
			if evType == simdefs.TRG_OVERWATCH and userUnit:getTraits().isMeleeAiming then
				local targetUnit = evData

				if targetUnit and targetUnit:isValid() and not targetUnit:isKO() then
					local doOverwatch = false
					if	(userPlayer ~= sim:getCurrentPlayer() and simquery.isEnemyAgent( userPlayer, targetUnit )) or userUnit:getTraits().shootIndiscriminate then
						doOverwatch = true
					end
					
					local canUse, reason1,reason2, floatTxt = self:canUseAbility( sim, userUnit, userUnit, targetUnit:getID() ) 

					if floatTxt then
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=floatTxt,x=x0,y=y0,color={r=1,g=1,b=41/255,a=1}, skipQue=true} )
					end

					if doOverwatch and canUse and sim:canUnitSeeUnit( userUnit, targetUnit ) then
						self:executeAbility( sim, userUnit, userUnit, targetUnit:getID() )
					end
				end
			end
		elseif userUnit:getBrain() then
			local targetUnit = userUnit:getBrain():getTarget()
			
			if targetUnit then
				if simquery.isEnemyTarget( userPlayer, targetUnit ) and not targetUnit:isDown() then
					local x1,y1 = targetUnit:getLocation()
					
					if not userUnit:getTraits().ce_performing_melee and not userUnit:getTraits().isMeleeAiming and not userUnit:getTraits().lookingAround then
						local dir = simquery.getDirectionFromDelta(x1-x0,y1-y0)
						sim:dispatchEvent( simdefs.EV_SCRIPT_EXIT_MAINFRAME )
						
						userUnit:getTraits().ce_meleeaiming = true
						
						userUnit:updateFacing(dir)
						targetUnit:interruptMove( sim, userUnit )
						if targetUnit:getTraits().movePath then
							sim:dispatchEvent( simdefs.EV_UNIT_STOP_WALKING, { unit = targetUnit, unblocked = true } )
						end
						userUnit:getTraits().isMeleeAiming = true
						sim:dispatchEvent( simdefs.EV_UNIT_OVERWATCH_MELEE, { unit = userUnit })
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
						
						userUnit:getTraits().ce_meleeaiming = nil
					end
					
					local canUse, reason = self:canUseAbility( sim, userUnit, userUnit, targetUnit:getID() )
					
					if canUse then
						self:executeAbility( sim, userUnit, userUnit, targetUnit:getID() )
						self.onTrigger( self, sim, evType, evData, userUnit )
						return
					elseif reason == "MOVING" then
						userUnit:getTraits().noInterrupt = false
						userUnit:interruptMove( sim, userUnit )
					end
					
					local destination = userUnit:getBrain():getDestination()
					
					if destination and destination.meleeTargetX and (destination.meleeTargetX ~= x1 or destination.meleeTargetY ~= y1) then
						util.tmerge( destination, actions.MoveBesideTarget.getDestination( { sim = sim, unit = userUnit } ) )
					end
				end
			end
		end
	end,
	
	isValidTarget = function( self, sim, unit, userUnit, targetUnit )
		if targetUnit == nil or targetUnit:isGhost() then
			return false
		end

		if not simquery.isEnemyTarget( userUnit:getPlayerOwner(), targetUnit ) then
			return false
		end
		
		if not targetUnit:getTraits().canKO then
			--return false
		end

		if targetUnit:isKO() then
			return false
		end

		return true
	end,

	canUseAbility = function( self, sim, unit, userUnit, targetID )
		if simquery.getEquippedMelee( userUnit ) == nil then
			return false
		end
		
		if userUnit:getTraits().ce_performing_melee or userUnit:getTraits().refreshingLOS then
			return false
		end
		
		if userUnit:getAP() < 1 then
			return false
		end
		
		if targetID then
			local targetUnit = sim:getUnit(targetID)
			
			if not self:isValidTarget( sim, userUnit, userUnit, targetUnit ) then
				return false
			end
		
			if not simquery.canUnitReach( sim, userUnit, targetUnit:getLocation() ) then
				return false, STRINGS.UI.COMBAT_PANEL_FAIL_KO, STRINGS.UI.COMBAT_PANEL_NO_RANGE
			end
			
			--local dmg = simquery.calculateMeleeDamage(sim,  simquery.getEquippedMelee( userUnit ), targetUnit)
			--if dmg <= 0 and targetUnit:getArmor() > 0 then
			--	return false, STRINGS.UI.COMBAT_PANEL_ARMORED
			--end
		end
		
		if userUnit:getTraits().movePath or userUnit:getTraits().modifyingExit or userUnit:getTraits().lookingAround then
			return false, "MOVING"
		end
		
		return true
	end,
	
	executeMelee = function( self, sim, unit, userUnit, targetUnit )
		sim:dispatchEvent( simdefs.EV_SCRIPT_EXIT_MAINFRAME )

		local x0,y0 = unit:getLocation()
		local x1,y1 = targetUnit:getLocation()
		local distance = mathutil.dist2d( x0, y0, x1, y1 )
		local tazer = simquery.getEquippedMelee(unit)
		if distance > 1 then 
			local cell = simquery.findNearestEmptyCell( sim, x0, y0, targetUnit )
			sim:warpUnit( targetUnit, cell )
			x1,y1 = targetUnit:getLocation()
		end

		local dir = simquery.getDirectionFromDelta(x1-x0,y1-y0)
		unit:updateFacing(dir)

		if not unit:isValid() then
			return
		end
		unit:setInvisible( false )
		unit:setDisguise(false)
		if not unit:isValid() then
			return
		end

		sim:emitSpeech( unit, speechdefs.EVENT_ATTACK_MELEE )		

		local meleeDamage = 0
		local hitDir = nil
		local targetFacing = targetUnit:getFacing()
		local dFac = dir - targetFacing
		
		if (dFac >= -6 and dFac <= -2) or (dFac >= 2 and dFac <= 6) then
			hitDir = "frt"
		end
		
		if targetUnit:getTraits().canKO then
			meleeDamage = simquery.calculateMeleeDamage(sim, simquery.getEquippedMelee( userUnit ), targetUnit)
		end
		
		sim:dispatchEvent( "EV_CE_MELEE", { unit = unit, targetUnit = targetUnit, meleeSuccess = true} )
		self._sim:dispatchEvent( simdefs.EV_UNIT_HIT, {unit = targetUnit, kodamage = meleeDamage, crit = false, dir = hitDir } )
		
		if meleeDamage > 0 then
			if simquery.getEquippedMelee( userUnit ):getTraits().lethalMelee then
				targetUnit:killUnit(sim)
			else
				local koTime = math.max( 0, meleeDamage )
				targetUnit:setKO( sim, koTime )
				targetUnit:getTraits().lastHit = { x=x0, y=y0, unit=unit }
			end
		else
			unit:getTraits().ap = unit:getTraits().ap - 1
		end
		
		sim:dispatchEvent( "EV_CE_MELEE", { finishMelee=true })
		sim:triggerEvent(simdefs.TRG_UNIT_HIT, {targetUnit=targetUnit, sourceUnit=unit, x=x1, y=y1, melee=true})
		unit:getTraits().psiBulletproof = nil
		targetUnit:getTraits().psiBulletproof = nil
		
		if unit:getBrain() then
			unit:getBrain():reset()
		end
		
		sim:refreshUnitLOS( userUnit )
		unit:getTraits().turning = true -- This is just a fun little thing that allows agents like Pedler who are immune to the attack to counter
		sim:processReactions(unit)
		unit:getTraits().turning = nil
	end,

	executeAbility = function( self, sim, unit, userUnit, target )
		local targetUnit = sim:getUnit(target)
		userUnit:getTraits().ce_performing_melee = true
	
		sim:refreshUnitLOS( userUnit )
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
		userUnit:interruptMove( sim, targetUnit )
		targetUnit:interruptMove( sim, userUnit )
	
		userUnit:setInvisible(false)
		userUnit:setDisguise(false,nil,true)

		if targetUnit:getPlayerOwner() then
			targetUnit:getPlayerOwner():glimpseUnit( sim, userUnit:getID() )
		end
		userUnit:resetAllAiming()
		userUnit:getTraits().noPeripheralDetection = nil
		userUnit:getTraits().noInterrupt = nil
		
		self:executeMelee( sim, userUnit, userUnit, targetUnit )
		
		userUnit:getTraits().ce_performing_melee = nil
		sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit  } )
	end
}

return ce_melee