local array = include( "modules/array" )
local util = include( "modules/util" )
local cdefs = include( "client_defs" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local abilityutil = include( "sim/abilities/abilityutil" )

local ce_throw_support = {
	name = STRINGS.ABILITIES.THROW,

	getName = function( self, sim, unit, userUnit )
		return self.name
	end,

	createToolTip = function( self,sim,unit,targetCell)
		return abilityutil.formatToolTip( STRINGS.ABILITIES.THROW,  STRINGS.MOD_CE.UI.DRONE_GRENADE_DESC, 1 )
	end,

	--profile_icon = "gui/icons/action_icons/Action_icon_Small/icon-item_shoot_small.png",
	profile_icon = "gui/actions/aim.png",
	usesAction = true,
	showTargets = true,--Makes it show up in the bottom bar (hopefully)

	acquireTargets = function( self, targets, game, sim, abilityOwner, unit)
		if not self:canUseAbility( sim, abilityOwner, unit ) then
			return nil
		end
		
		local abilityDef, grenadeUnit = unit:ownsAbility("throw")
		if grenadeUnit then
			return targets.throwTarget( game, grenadeUnit:getTraits().range or 0, sim, unit, unit:getTraits().maxThrow, grenadeUnit:getTraits().targeting_ignoreLOS)
		end
	end, 

	canUseAbility = function( self, sim, abilityOwner, unit, targetCell )
		if abilityOwner ~= unit then
			--Not a proxy ability
			return false
		end
	
		if unit:getTraits().PCGrenadesThrown >= unit:getTraits().maxPCGrenades then
            return false, STRINGS.MOD_CE.UI.REASON_DRONE_GRENADE
		end

		if targetCell then
			local targetX,targetY = unpack(targetCell)
			local unitX, unitY = unit:getLocation()
    		local raycastX, raycastY = sim:getLOS():raycast(unitX, unitY, targetX, targetY)
			if raycastX ~= targetX or raycastY ~= targetY then
				return false
			end
		end

		return true
	end,

	executeAbility = function( self, sim, abilityOwner, userUnit, targetCell )
		local abilityDef, grenadeUnit = userUnit:ownsAbility("throw")
		
		abilityDef:executeAbility( sim, grenadeUnit, userUnit, targetCell )
		
		userUnit:getTraits().PCGrenadesThrown = userUnit:getTraits().PCGrenadesThrown + 1
	end
}

return ce_throw_support
