local util = include( "modules/util" )
local simdefs = include("sim/simdefs")
local commondefs = include("sim/unitdefs/commondefs")

commondefs.onGuardTooltip = function( tooltip, unit )
	if unit:getTraits().invisible then
		local sim = unit:getSim()
		
		if not sim:canPlayerSeeUnit(sim:getPC(), unit) then
			tooltip._h = -4
			tooltip._w = 0
			tooltip.activate = function(self,screen)
				self._screen = screen
			end
			return
		end
	end
	commondefs.onAgentTooltip( tooltip, unit )

    local traits = unit:getTraits()

	if traits.heartMonitor == "enabled" then
        tooltip:addAbility( STRINGS.UI.TOOLTIPS.HEART_MONITOR, STRINGS.UI.TOOLTIPS.HEART_MONITOR_DESC, "gui/icons/item_icons/items_icon_small/icon-item_heart_monitor_small.png" )		
    end
	
    if not traits.canKO then
	    tooltip:addAbility( STRINGS.UI.TOOLTIPS.KO_IMMUNE, STRINGS.UI.TOOLTIPS.KO_IMMUNE_DESC,  "gui/icons/arrow_small.png"  )									
    end
	if not traits.hasHearing then
	    tooltip:addAbility( STRINGS.UI.TOOLTIPS.NO_HEARING, STRINGS.UI.TOOLTIPS.NO_HEARING_DESC,  "gui/icons/arrow_small.png"  )				
  	end
	if traits.mainframe_suppress_rangeMax then
		if traits.isDrone then
			tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.NULL_ZONE ), util.sformat( STRINGS.UI.TOOLTIPS.NULL_ZONE_DESC, traits.mainframe_suppress_rangeMax ), "gui/icons/arrow_small.png" )
		else
			tooltip:addAbility( STRINGS.UI.TOOLTIPS.NULL_ZONE, util.sformat( STRINGS.MOD_CE.TOOLTIPS.NULL_ZONE_DESC, traits.mainframe_suppress_rangeMax ), "gui/icons/arrow_small.png" )
		end
    end  
	local armor
	if traits.armor then
		armor = traits.armor or 0
		if unit:getTraits().temporaryArmor then
			armor = armor - math.ceil(traits.temporaryArmor/2)
		end
		if traits.psiShield then
			armor = armor-1
		end
	end
	if traits.psiShield then
		tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.TELEKENETIC_PROTECTION, STRINGS.MOD_CE.TOOLTIPS.TELEKENETIC_PROTECTION_DESC,  "gui/hud3/hud3_armor_tutorial_icon.png"  )
	end
    if traits.weakBarrier then
		tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.WEAK_BARRIER, string.format( STRINGS.MOD_CE.TOOLTIPS.WEAK_BARRIER_DESC, traits.weakBarrier ) , "gui/icons/item_icons/items_icon_small/icon-item_personal_shield_small.png" )
    elseif traits.shieldArmor then
		tooltip:addAbility( STRINGS.UI.TOOLTIPS.BARRIER, STRINGS.UI.TOOLTIPS.BARRIER_DESC, "gui/icons/item_icons/items_icon_small/icon-item_personal_shield_small.png" )
    elseif armor and armor > 0 then
        tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.HEAVY_ARMOR, armor ), STRINGS.UI.TOOLTIPS.HEAVY_ARMOR_DESC,  "gui/hud3/hud3_armor_tutorial_icon.png"  )
    end
	if traits.temporaryArmor and traits.temporaryArmor > 0 then
		tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.TEMPORARY_ARMOR ), string.format( STRINGS.MOD_CE.TOOLTIPS.TEMPORARY_ARMOR_DESC, math.ceil(traits.temporaryArmor/2) ),  "gui/icons/arrow_small.png"  )
	end
	if traits.psiShield then
		if traits.resistKO then
			if traits.resistKO < 1 then
				tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.KO_VULNERABLE, STRINGS.MOD_CE.TOOLTIPS.KO_VULNERABLE_DESC, "gui/icons/arrow_small.png" )
			elseif traits.resistKO > 1 then
				tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.KO_RESISTANT, traits.resistKO-1 ), STRINGS.UI.TOOLTIPS.KO_RESISTANT_DESC,  "gui/icons/arrow_small.png"  )
			end
		end
	else
		if traits.resistKO then
			if traits.resistKO < 0 then
				tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.KO_VULNERABLE, STRINGS.MOD_CE.TOOLTIPS.KO_VULNERABLE_DESC, "gui/icons/arrow_small.png" )
			elseif traits.resistKO > 0 then
				tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.KO_RESISTANT, traits.resistKO ), STRINGS.UI.TOOLTIPS.KO_RESISTANT_DESC,  "gui/icons/arrow_small.png"  )
			end
		end
	end
	
	if traits.hardmode then
		if traits.specialGrenades then
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.OMNI_GRENADE_HARD, STRINGS.MOD_CE.TOOLTIPS.OMNI_GRENADE_HARD_DESC,  "gui/icons/arrow_small.png" )
		elseif traits.isSupportGuard then 
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.GRENADE_GUARD_HARD, STRINGS.MOD_CE.TOOLTIPS.GRENADE_GUARD_HARD_DESC,  "gui/icons/arrow_small.png" )
		elseif traits.throwsScangrenades then
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.SCAN_GRENADE, STRINGS.MOD_CE.TOOLTIPS.SCAN_GRENADE_DESC,  "gui/icons/arrow_small.png" )
		elseif traits.throwsPulseGrenades then
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.PULSE_GRENADE, STRINGS.MOD_CE.TOOLTIPS.PULSE_GRENADE_DESC,  "gui/icons/arrow_small.png" )
		end 
	else
		if traits.specialGrenades then
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.OMNI_GRENADE, STRINGS.MOD_CE.TOOLTIPS.OMNI_GRENADE_DESC,  "gui/icons/arrow_small.png" )
		elseif traits.isSupportGuard then 
			tooltip:addAbility( STRINGS.UI.TOOLTIPS.GRENADE_GUARD, STRINGS.UI.TOOLTIPS.GRENADE_GUARD_DESC,  "gui/icons/arrow_small.png" )
		elseif traits.throwsScangrenades then
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.SCAN_GRENADE, STRINGS.MOD_CE.TOOLTIPS.SCAN_GRENADE_DESC,  "gui/icons/arrow_small.png" )
		elseif traits.throwsPulseGrenades then
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.PULSE_GRENADE, STRINGS.MOD_CE.TOOLTIPS.PULSE_GRENADE_DESC,  "gui/icons/arrow_small.png" )
		end 
	end

	if traits.failsafeAi then
		tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.FAILSAFE, STRINGS.MOD_CE.TOOLTIPS.FAILSAFE_DESC ,  "gui/icons/arrow_small.png"  )
	elseif traits.isDrone then
	    tooltip:addAbility( STRINGS.UI.TOOLTIPS.CONTROLLABLE, STRINGS.UI.TOOLTIPS.CONTROLLABLE_DESC,  "gui/icons/arrow_small.png"  )				
    end
	
	if traits.psiLink and not traits.psiSupport then
		tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.PSI_LINK, STRINGS.MOD_CE.TOOLTIPS.PSI_LINK_DESC, "gui/icons/arrow_small.png" )
	end
	
	if (not unit.getModifiers or not unit:getModifiers():has("LOSarc", "control")) then
		if traits.mainframebooster then
			--Augment sight
			tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.PHERIPHERAL_OPTIMIZER ), string.format( STRINGS.MOD_CE.TOOLTIPS.PHERIPHERAL_OPTIMIZER_DESC), "gui/icons/arrow_small.png" )
		elseif (traits.LOSarc or 0) >= 2 * math.pi and (traits.LOSrange or 0) > 0.5 then
			--360 main sight
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.SIGHT_360, string.format( STRINGS.MOD_CE.TOOLTIPS.SIGHT_360_DESC, string.format(" %.1f", traits.LOSrange):gsub("%.?0+$", "") ),  "gui/icons/arrow_small.png"  )
		elseif (traits.LOSperipheralArc or 0) >= (2 * math.pi) and (traits.LOSperipheralRange or 0) > 0.5 then
			--360 peripheral
			tooltip:addAbility( STRINGS.UI.TOOLTIPS.ADVANCED_PERIPHERAL_BOOST, STRINGS.UI.TOOLTIPS.ADVANCED_PERIPHERAL_BOOST_DESC ,  "gui/icons/arrow_small.png"  )
		elseif (traits.LOSarc or 0) >= math.pi and (traits.LOSrange or 0) > 0.5 then
			--180 main sight
			tooltip:addAbility( STRINGS.UI.TOOLTIPS.PERIPHERAL_BOOST, STRINGS.UI.TOOLTIPS.PERIPHERAL_BOOST_DESC,  "gui/icons/arrow_small.png"  )
		elseif (traits.LOSperipheralArc or 0) >= math.pi and (traits.LOSperipheralRange or 0) > 0.5 then
			--180 peripheral sight
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.PHERIPHERAL_IMPROVEMENT, STRINGS.MOD_CE.TOOLTIPS.PHERIPHERAL_IMPROVEMENT_DESC,  "gui/icons/arrow_small.png"  )
		elseif (traits.LOSperipheralArc or 0) >= 11*math.pi/16 + ((traits.hardmode and math.pi/4) or 0) and (traits.LOSperipheralRange or 0) > 0.5 then
			--123.75 peripheral sight 
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.PHERIPHERAL_INCREASE, STRINGS.MOD_CE.TOOLTIPS.PHERIPHERAL_INCREASE_DESC,  "gui/icons/arrow_small.png"  )
		end
	end
	
    if traits.neutralize_shield then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.PROTECTIVE_SHIELDS, traits.neutralize_shield),  STRINGS.UI.TOOLTIPS.PROTECTIVE_SHIELDS_DESC, "gui/icons/item_icons/items_icon_small/icon-item_personal_shield_small.png",nil,true )
    end
	if traits.empDeath then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.EMP_VULNERABLE ), STRINGS.UI.TOOLTIPS.EMP_VULNERABLE_DESC, "gui/icons/arrow_small.png",nil,true )
    end 
	
	if traits.pacifist then
		tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.PACIFIST ), string.format( STRINGS.MOD_CE.TOOLTIPS.PACIFIST_DESC), "gui/icons/arrow_small.png" )
	end
	
	if traits.dartgun then
		tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.DARTGUN ), string.format( STRINGS.MOD_CE.TOOLTIPS.DARTGUN_DESC), "gui/icons/arrow_small.png" )
	end
	
	if traits.meleeRecapture then
		tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.MELEE_HACK ), util.sformat( STRINGS.MOD_CE.TOOLTIPS.MELEE_HACK_DESC, traits.mainframeRecapture), "gui/icons/arrow_small.png" )
	end

    if traits.mainframeRecapture then
		--if traits.hardmode then
			--tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.COUNTER_HACK ), util.sformat( STRINGS.MOD_CE.TOOLTIPS.COUNTER_HACK_HARD_DESC, traits.mainframeRecapture), "gui/icons/arrow_small.png" )
		--else
			tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.COUNTER_HACK ), util.sformat( STRINGS.UI.TOOLTIPS.COUNTER_HACK_DESC, traits.mainframeRecapture), "gui/icons/arrow_small.png" )
		--end
    end 
    if traits.koDaemon then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.KO_DEAMON ), string.format( STRINGS.UI.TOOLTIPS.KO_DEAMON_DESC), "gui/icons/arrow_small.png" )
    end 
	if traits.investigateHackedDevices then
	    tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.MAINFRAME_SENSORY, STRINGS.MOD_CE.TOOLTIPS.MAINFRAME_SENSORY_DESC,  "gui/icons/arrow_small.png"  )				
    end
	if traits.consoleScan then
		tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.CONSOLE_SCAN ), util.sformat( STRINGS.MOD_CE.TOOLTIPS.CONSOLE_SCAN_DESC, traits.mainframeRecapture), "gui/icons/arrow_small.png" )
	end
    if traits.lookaroundRange and not traits.no_look_around then
		if traits.high_frequency_scanner then
			tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.SUPER_SCAN ), string.format( STRINGS.MOD_CE.TOOLTIPS.SUPER_SCAN_DESC ), "gui/icons/arrow_small.png" )
		elseif traits.lookaroundRange >= 7 + ((traits.hardmode and 2) or 0) then
			tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.BETTER_SCAN ), string.format( STRINGS.MOD_CE.TOOLTIPS.BETTER_SCAN_DESC ), "gui/icons/arrow_small.png" )
		elseif traits.isDrone then
			tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.DRONE_SCAN ), string.format( STRINGS.UI.TOOLTIPS.DRONE_SCAN_DESC), "gui/icons/arrow_small.png" )
		else
			tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.DRONE_SCAN ), string.format( "This guard investigates with a short range scanner that will locate targets even behing cover."), "gui/icons/arrow_small.png" )
		end
    end
	
	if traits.true_drone_repair then
		tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.DRONE_REPAIR, STRINGS.MOD_CE.TOOLTIPS.DRONE_REPAIR_DESC, "gui/icons/arrow_small.png" )
	end
	
	if traits.patrolPath then
		local str = "("
		for i, node in pairs(traits.patrolPath) do
			if i > 1 then
				str = str..", "
			end
			str = str.."( "..tostring(node.x)..", "..tostring(node.y).." )"
		end
		--tooltip:addAbility( string.format( "PATROL" ), str..")" )
	end
	
	if unit:getBrain() and unit:getBrain():getDestination() then
		--tooltip:addAbility( "DESTINATION", tostring(unit:getBrain():getDestination().x)..", "..tostring(unit:getBrain():getDestination().y) )
	end
	
	if traits.selfFirewallBuff then
		tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.SELF_ICE_BUFF ), util.sformat( STRINGS.MOD_CE.TOOLTIPS.SELF_ICE_BUFF_DESC, traits.selfFirewallBuff ), "gui/icons/arrow_small.png" )
	end

	if traits.meleeFighter then
		tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.MELEEFIGHTER ), string.format( STRINGS.MOD_CE.TOOLTIPS.MELEEFIGHTER_DESC ), "gui/icons/arrow_small.png" )
	end
	
	if traits.usesCloaks then
		tooltip:addAbility( string.format( STRINGS.MOD_CE.TOOLTIPS.CLOAKRIG ), string.format( STRINGS.MOD_CE.TOOLTIPS.CLOAKRIG_DESC, traits.cloakDurationMax, traits.cloakDurationMax ~= 1 and "s" or "", traits.cloakCooldownMax ), "gui/icons/arrow_small.png" )
	end
	
    if traits.magnetic_reinforcement then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.MAGNETIC_REINFOREMENTS ), string.format( STRINGS.UI.TOOLTIPS.MAGNETIC_REINFOREMENTS_DESC), "gui/icons/arrow_small.png" )
    end
	
	local abilities = unit:getAbilities()   
	for i, ability in ipairs( abilities ) do  
		if ability.buffAbility then  
			tooltip:addAbility( ability.name or ability.getName(traits), ability.buffDesc or ability.getDesc(traits), ability.icon or "gui/icons/arrow_small.png" ) 
     	end  
    end   


    --NEW TOOLTIPS
    if traits.neural_scanned then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.NEURAL_SCANNED ), string.format( STRINGS.UI.TOOLTIPS.NEURAL_SCANNED_DESC), "gui/icons/arrow_small.png" )
    end
    if traits.pulseScan then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.PULSE_SCANNER ), util.sformat(STRINGS.UI.TOOLTIPS.PULSE_SCANNER_DESC,traits.range ), "gui/icons/arrow_small.png" )
    end  
    if traits.AOEFirewallsBuff then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.AOE_FIREWALL_BUFF ), util.sformat(  STRINGS.UI.TOOLTIPS.AOE_FIREWALL_BUFF_DESC,traits.AOEFirewallsBuffRange,traits.AOEFirewallsBuff ), "gui/icons/arrow_small.png" )
    end      
    if traits.noInterestDistraction then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.FOCUSED_AI ), util.sformat( STRINGS.UI.TOOLTIPS.FOCUSED_AI_DESC ), "gui/icons/arrow_small.png" )
    end  
    if traits.buffArmorOnKO then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.KO_ARMOR_BUFF ), util.sformat( STRINGS.UI.TOOLTIPS.KO_ARMOR_BUFF_DESC,traits.buffArmorOnKO), "gui/icons/arrow_small.png" )
    end  

    if traits.searchedAnarchy5 then
    	tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.SEARCHED_ADVANCED ), util.sformat( STRINGS.UI.TOOLTIPS.SEARCHED_ADVANCED_DESC ), "gui/icons/arrow_small.png" )
    elseif traits.searched then
		tooltip:addAbility( string.format( STRINGS.UI.TOOLTIPS.SEARCHED ), util.sformat( STRINGS.UI.TOOLTIPS.SEARCHED_DESC ), "gui/icons/arrow_small.png" )
    end

	if traits.ce_suspicious and unit:getSim():getParams().difficultyOptions.ce_suspicious and unit:getSim():getParams().difficultyOptions.ce_suspicious > 0 and traits.ce_suspicious == unit:getSim():getParams().difficultyOptions.ce_suspicious and not unit:isAlerted() then
		tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.SUSPICIOUS, STRINGS.MOD_CE.TOOLTIPS.SUSPICIOUS_DESC, "gui/icons/thought_icons/status_hunt.png" )
	elseif traits.ce_calmed and not unit:isAlerted() then
		tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.CALM, STRINGS.MOD_CE.TOOLTIPS.CALM_DESC, "gui/icons/arrow_small.png" )
	end

	for i,tooltipFunction in ipairs(mod_manager:getTooltipDefs().onGuardTooltips)do
		tooltipFunction(tooltip, unit )
	end

    if traits.dlcFooter then
		tooltip:addFooter( traits.dlcFooter[1],traits.dlcFooter[2] )
	end
end

return commondefs