local simengine = include( "sim/engine" )
local simdefs = include( "sim/simdefs" )
local simquery = include( "sim/simquery" )
local inventory = include( "sim/inventory" )
local array = include( "modules/array" )

function simengine:modifyExit( cell, dir, exitOp, unit, stealth) 
	local exit = cell.exits[dir]
	assert( exit )

	local rdir = simquery.getReverseDirection( dir )
	local reverseExit = exit.cell.exits[ rdir ]
	assert( reverseExit )

	if unit then
		unit:getTraits().modifyingExit = exit
	end

	local incAlarm = false 

	local altCell = cell.exits[dir].cell
	
	--[[if exit.temporarykeybits then
		exit.keybits = reverseExit.temporarykeybits
		exit.temporarykeybits = nil
	end
	if reverseExit.temporarykeybits then
		reverseExit.keybits = reverseExit.temporarykeybits
		reverseExit.temporarykeybits = nil
	end]]

	if exitOp == simdefs.EXITOP_OPEN then
		assert( exit.door and reverseExit.door )
		assert( exit.closed and reverseExit.closed )
		assert( (not exit.locked and not reverseExit.locked) or (unit:isNPC() and exit.keybits == simdefs.DOOR_KEYS.SECURITY) )
		exit.closed, reverseExit.closed = nil, nil
		if exit.locked or reverseExit.locked then
			--[[exit.temporarykeybits = exit.keybits
			exit.keybits = simdefs.DOOR_KEYS.GUARD
			reverseExit.temporarykeybits = exit.keybits
			reverseExit.keybits = simdefs.DOOR_KEYS.GUARD]]
			exit.closeEndTurn = true
			exit.temporaryCloseEndTurn = true
			reverseExit.closeEndTurn = true
			reverseExit.temporaryCloseEndTurn = true
		end
		self:getLOS():removeSegments( cell, dir, exit.cell, rdir )
		self:emitSound( exit.openSound, cell.x, cell.y, unit, {{x=altCell.x,y=altCell.y}} )
		if self:getCurrentPlayer() ~= self:getNPC() then		
			self:getNPC().pather:invalidatePathsCrossingCell(cell.x, cell.y)		
			self:getNPC().pather:invalidatePathsCrossingCell(altCell.x, altCell.y)
			self:triggerEvent( simdefs.TRG_OPEN_DOOR, { unit = unit } )
		end

		if self:getTags().doors_alarmOnOpen and unit:isPC() and not exit.door_disarmed then
			incAlarm = true			
		end

	elseif exitOp == simdefs.EXITOP_CLOSE then
		assert( exit.door and reverseExit.door )
		assert( not exit.closed and not reverseExit.closed )
		if exit.temporaryCloseEndTurn or reverseExit.temporaryCloseEndTurn then
			exit.closeEndTurn = false
			exit.temporaryCloseEndTurn = false
			reverseExit.closeEndTurn = false
			reverseExit.temporaryCloseEndTurn = false
		end
		exit.closed, reverseExit.closed = true, true
		self:getLOS():insertSegments( cell, dir, exit.cell, rdir )
		self:emitSound( exit.closeSound, cell.x, cell.y, unit, {{x=altCell.x,y=altCell.y}})		
		if self:getCurrentPlayer() ~= self:getNPC() then		
			self:getNPC().pather:invalidatePathsCrossingCell(cell.x, cell.y)		
			self:getNPC().pather:invalidatePathsCrossingCell(altCell.x, altCell.y)
			self:getNPC().pather:invalidatePathsWithThrows()
		end

	elseif exitOp == simdefs.EXITOP_TOGGLE_DOOR then
		assert( exit.door and reverseExit.door )
		if exit.closed then
			exit.closed, reverseExit.closed = nil, nil
			self:getLOS():removeSegments( cell, dir, exit.cell, rdir )
			self:emitSound( exit.openSound, cell.x, cell.y, unit, {{x=altCell.x,y=altCell.y}})
			self:triggerEvent( simdefs.TRG_OPEN_DOOR, { unit = unit } )	

			if self:getTags().doors_alarmOnOpen and unit:isPC() then
				self:trackerAdvance( 1 )
			end
		else
			exit.closed, reverseExit.closed = true, true
			self:getLOS():insertSegments( cell, dir, exit.cell, rdir )
			self:emitSound( exit.closeSound, cell.x, cell.y, unit, {{x=altCell.x,y=altCell.y}})
		end

	elseif exitOp == simdefs.EXITOP_LOCK then
		assert( exit.door and reverseExit.door )
		assert( exit.keybits and reverseExit.keybits )
		assert( not exit.locked and not reverseExit.locked )
		exit.locked, reverseExit.locked = true, true

	elseif exitOp == simdefs.EXITOP_UNLOCK then
		assert( exit.door and reverseExit.door )
		assert( exit.keybits and reverseExit.keybits )
		assert( exit.locked and reverseExit.locked )
		exit.locked, reverseExit.locked = nil

		if self:isVersion("0.17.5") then
					
			if exit.keybits == simdefs.DOOR_KEYS.VAULT then
				for i,item in ipairs(unit:getChildren()) do
					if item:getTraits().keybits == simdefs.DOOR_KEYS.VAULT then
						self:triggerEvent( simdefs.TRG_UNLOCK_DOOR, { cell = cell, tocell = exit.cell, unit = unit } )
						inventory.trashItem( self, unit, item )
						break
					end
				end
	        elseif exit.keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT then
	        	local key = nil
				for i,item in ipairs(unit:getChildren()) do
					if item:getTraits().keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT then
						key = item
						break
					elseif simquery.isKey( item, simdefs.DOOR_KEYS.SPECIAL_EXIT ) and not simquery.isKey( item, simdefs.DOOR_KEYS.VAULT )  then
						key = item
						-- Keep searching, we prioritize SPECIAL_EXIT keys precisely.
					end
				end
				if key then
					inventory.trashItem( self, unit, key )
				end            
			end	
		else
			---------------------------------------------------------------------------------------------------------------
			--#############################################################################################################			
			--#############################################################################################################			
			-- This is to replicate the old function of the keys for games before version 17.5 as simdefs was changed from :
			--				SPECIAL_EXIT = 32 + 256, to 
			--				SPECIAL_EXIT = 256
			if exit.keybits == simdefs.DOOR_KEYS.VAULT then
				for i,item in ipairs(unit:getChildren()) do
					if item:getTraits().keybits == simdefs.DOOR_KEYS.VAULT then
						self:triggerEvent( simdefs.TRG_UNLOCK_DOOR, { cell = cell, tocell = exit.cell, unit = unit } )
						inventory.trashItem( self, unit, item )
						break
					elseif item:getTraits().keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT then
						self:triggerEvent( simdefs.TRG_UNLOCK_DOOR, { cell = cell, tocell = exit.cell, unit = unit } )
					end
				end
	        elseif exit.keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT then
	        	local key = nil
				for i,item in ipairs(unit:getChildren()) do
					if item:getTraits().keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT or item:getTraits().keybits == simdefs.DOOR_KEYS.VAULT then
						key = item
						break
					elseif simquery.isKey( item, simdefs.DOOR_KEYS.SPECIAL_EXIT ) or simquery.isKey( item, simdefs.DOOR_KEYS.VAULT ) then
						key = item
						-- Keep searching, we prioritize SPECIAL_EXIT keys precisely.
					end
				end
				if key then
					inventory.trashItem( self, unit, key )
				end            
			end			
			--#############################################################################################################
			--#############################################################################################################
			---------------------------------------------------------------------------------------------------------------
		end

		if unit then
			self:emitSound( simdefs.SOUND_DOOR_UNLOCK, cell.x, cell.y, unit)
		end

	elseif exitOp == simdefs.EXITOP_TOGGLE_LOCK then
		assert( exit.door and reverseExit.door )
		assert( exit.keybits and reverseExit.keybits )
		if exit.locked then
			exit.locked, reverseExit.locked = nil, nil
		else
			exit.locked, reverseExit.locked = true, true
		end

	elseif exitOp == simdefs.EXITOP_BREAK_DOOR then
		assert( exit.door and reverseExit.door )
		assert( exit.closed and reverseExit.closed )
		assert( not exit.locked and not reverseExit.locked )

		exit.closed, reverseExit.closed = nil, nil
		exit.locked, reverseExit.locked = true, true		
		self:getLOS():removeSegments( cell, dir, exit.cell, rdir )
		self:emitSound( exit.breakSound, cell.x, cell.y, unit, {{x=altCell.x,y=altCell.y}} )		
		if self:getCurrentPlayer() ~= self:getNPC() then		
			self:getNPC().pather:invalidatePathsCrossingCell(cell.x, cell.y)		
			self:getNPC().pather:invalidatePathsCrossingCell(altCell.x, altCell.y)
		end

	elseif exitOp == simdefs.EXIT_DISARM then	
		assert( exit.door and reverseExit.door )
		exit.door_disarmed = true
		reverseExit.door_disarmed = true
		unit:useMP( 3, self )
		local x1, y1 = unit:getLocation()
		self:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT, {txt="ALARM DISABLED",x=x1,y=y1,color={r=163/255,g=0,b=0,a=1},alwaysShow=true} )		
	end

	self:dispatchEvent( simdefs.EV_EXIT_MODIFIED, { cell = cell, dir = dir, exitOp = exitOp } )

	-- Refresh LOS for any unit whose LOS arc intersects the exit.
	if exitOp == simdefs.EXITOP_TOGGLE_DOOR or exitOp == simdefs.EXITOP_CLOSE or exitOp == simdefs.EXITOP_OPEN or exitOp == simdefs.EXITOP_BREAK_DOOR then
		--use the door before refreshing LOS
		self:triggerEvent( simdefs.TRG_UNIT_USEDOOR_PRE, { exitOp = exitOp, cell = cell, tocell = exit.cell, unit = unit } )

        local seers = {}
        for unitID, seerUnit in pairs(self:getAllUnits()) do
        	local x0, y0 = seerUnit:getLocation()
            if seerUnit:getTraits().hasSight and x0 then
                local nx, ny = exit.cell.y - cell.y, exit.cell.x - cell.x  -- normal of exit.cell -> exit.cell
                local cx, cy = (cell.x + exit.cell.x) / 2, (cell.y + exit.cell.y) / 2
                if (x0 == cell.x and y0 == cell.y) or (x0 == exit.cell.x and y0 == exit.cell.y) or
                	self:getLOS():withinLOS( seerUnit, cx + nx/2, cy + ny/2, cx - nx/2, cy - ny/2 ) then
                    table.insert( seers, unitID )
                end
            end
        end

		for i, seerID in ipairs(seers) do
			local seerUnit
			if seerID >= simdefs.SEERID_PERIPHERAL then
				seerUnit = self:getUnit( seerID - simdefs.SEERID_PERIPHERAL )
			else
				seerUnit = self:getUnit( seerID )
			end

			if seerUnit then
				if seerUnit:hasTrait("hasSight") then
					self:refreshUnitLOS( seerUnit )
				end

				-- Make sure to also refresh the exit knowledge on either side of the exit.
				if seerUnit:getPlayerOwner() then
					seerUnit:getPlayerOwner():glimpseExit( cell.x, cell.y, dir )
				end
			end
		end

		self:triggerEvent( simdefs.TRG_UNIT_USEDOOR, { exitOp = exitOp, cell = cell, tocell = exit.cell, unit = unit } )
	end

	if exitOp == simdefs.EXITOP_BREAK_DOOR then
		--knock out any suckers on the other side of the door
		for i,targetUnit in ipairs(altCell.units) do
			if simquery.isAgent(targetUnit) and not targetUnit:isKO() and not targetUnit:isDead() and targetUnit:getArmor() == 0 then
				self:dispatchEvent(simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.UI.FLY_TXT.DOOR_SLAMMED,x=altCell.x,y=altCell.y,color={r=163/255,g=0,b=0,a=1},alwaysShow=true} )
				local koCount = 1
				if self:getCurrentPlayer() == targetUnit:getPlayerOwner() then
					--If a guard is knocked out on their own turn, they should still go down for a turn
					koCount = 2
				end
				if targetUnit:getTraits().canKO then
					targetUnit:setKO(self, koCount)
				elseif targetUnit:getTraits().isDrone and targetUnit.deactivate then
					targetUnit:deactivate(self)
				end
			end
		end
	end

	if unit and unit:isValid() and not unit:getTraits().interrupted then
		self:processReactions( unit )
	end

	if unit then
		unit:getTraits().modifyingExit = nil
	end

	if incAlarm then
		self:trackerAdvance( 1 )
	end
end

local oldDispatchEvent = simengine.dispatchEvent

function simengine:dispatchEvent( evType, evData, ... )
	if evType == simdefs.EV_UNIT_STOP_WALKING and not evData.unblocked then
		self:triggerEvent( "CE_UNIT_STOP_WALKING", evData )
	end
	
	local ret = oldDispatchEvent( self, evType, evData, ... )
	
	return ret
end