local util = include( "modules/util" )
local simdefs = include( "sim/simdefs" )
local commondefs = include( "sim/unitdefs/commondefs" )
local itemdefs = include( "sim/unitdefs/itemdefs" )
local speechdefs = include( "sim/speechdefs" )
local guarddefs = include( "sim/unitdefs/guarddefs" )

---------------------------------------------------------------------------------------------------------
-- NPC templates

local SOUNDS = commondefs.SOUNDS

local EG_SOUNDS =   
{   
    speech="SpySociety/Agents/dialogue_player",     
 	step = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_SOFT,    
 	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_FEMALE_HARDWOOD_SOFT,    
    
 	wallcover = "SpySociety/Movement/foley_trench/wallcover",   
 	crouchcover = "SpySociety/Movement/foley_trench/crouchcover",   
 	fall = "SpySociety/Movement/foley_trench/fall",   
 	land = "SpySociety/Movement/deathfall_agent_hardwood",   
 	land_frame = 16,						   
 	getup = "SpySociety/Movement/foley_trench/getup",	   
 	grab = "SpySociety/Movement/foley_trench/grab_guard",   
 	pin = "SpySociety/Movement/foley_trench/pin_guard",   
 	pinned = "SpySociety/Movement/foley_trench/pinned",   
 	peek_fwd = "SpySociety/Movement/foley_trench/peek_forward",	   
 	peek_bwd = "SpySociety/Movement/foley_trench/peek_back",	   
 	move = "SpySociety/Movement/foley_trench/move",   
 	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",   
} 

local SILENT_SOUNDS = {
	appeared="SpySociety/HUD/gameplay/peek_negative", 
	rustle = "SpySociety/Movement/foley/gear_light",
	alert ="SpySociety/Actions/guard/guard_alerted", 
	speech="SpySociety/Agents/dialogue_KO", 
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_SOFT, 
	--stealthStep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,
	step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,

	getup = "SpySociety/Movement/foley_guard/getup",
	fall = "SpySociety/Movement/foley_guard/fall",
	fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
	fall_kneeframe = 9,
	fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
	fall_handframe = 20,
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 34,					
	grabbed = "SpySociety/Movement/foley_guard/grabbed",
	pin = "SpySociety/Movement/foley_guard/pin_guard",
	pinned = "SpySociety/Movement/foley_guard/pinned",
	move ="SpySociety/Movement/foley_guard/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",	
						
	die = "die",
	hurt_small = "SpySociety/Agents/<voice>/hurt_small",	
	hurt_large = "SpySociety/Agents/<voice>/hurt_large",	
}

local SILENT_HEAVY_SOUNDS = {
	appeared="SpySociety/HUD/gameplay/peek_negative",
	rustle = "SpySociety/Movement/foley/gear_light",
	alert ="SpySociety/Actions/guard/guard_alerted", 
	speech="SpySociety/Agents/dialogue_KO", 
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_MALE_HARDWOOD_SOFT, 
	--stealthStep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,
	step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,
	wakeup = "SpySociety/Agents/<voice>/bark_wakeup",

	getup = "SpySociety/Movement/foley_heavy/getup",
	fall = "SpySociety/Movement/foley_heavy/fall",
	fallLandFrame = 30,
	fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
	fall_kneeframe = 9,
	fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
	fall_handframe = 20,
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 34,					
	grabbed = "SpySociety/Movement/foley_heavy/grabbed",
	pin = "SpySociety/Movement/foley_heavy/pin_guard",
	pinned = "SpySociety/Movement/foley_heavy/pinned",
	move ="SpySociety/Movement/foley_heavy/move",
	hit = "SpySociety/HitResponse/hitby_ballistic_flesh",	
						
	die = "die",
	hurt_small = "SpySociety/Agents/<voice>/hurt_small",	
	hurt_large = "SpySociety/Agents/<voice>/hurt_large",	
}

local AUGMENTED_SOUNDS =
{
	fall = "SpySociety/Movement/foley_cyborg/fall",		
	getup = "SpySociety/Movement/foley_cyborg/getup",
	grab = "SpySociety/Movement/foley_cyborg/grab_guard",
	pin = "SpySociety/Movement/foley_cyborg/pin_guard",
	pinned = "SpySociety/Movement/foley_cyborg/pinned",
	--move = "SpySociety/Movement/foley_cyborg/move",
	move = "SpySociety/Movement/foley_cyborg/crouchcover",
	hit = "SpySociety/HitResponse/hitby_ballistic_cyborg",
	
	appeared="SpySociety/HUD/gameplay/peek_negative", 
	alert ="SpySociety/Actions/guard/guard_alerted", 
	speech="SpySociety/Agents/dialogue_KO", 
	stealthStep = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL, 
	step = simdefs.SOUNDPATH_FOOTSTEP_GUARD_HARDWOOD_NORMAL,

	fall_knee = "SpySociety/Movement/bodyfall_agent_knee_hardwood",
	fall_kneeframe = 9,
	fall_hand = "SpySociety/Movement/bodyfall_agent_hand_hardwood",
	fall_handframe = 20,
	land = "SpySociety/Movement/deathfall_agent_hardwood",
	land_frame = 34,					
					
	die = "die",
	hurt_small = "SpySociety/Agents/<voice>/hurt_small",	
	hurt_large = "SpySociety/Agents/<voice>/hurt_large",
}




local DEFAULT_IDLES = commondefs.DEFAULT_IDLES

local DEFAULT_ABILITIES = commondefs.DEFAULT_ABILITIES

local DEFAULT_DRONE = commondefs.DEFAULT_DRONE

local onGuardTooltip = commondefs.onGuardTooltip

local npc_templates =
{
	ce_crazy_drone = util.extend( DEFAULT_DRONE )
	{
		onWorldTooltip = onGuardTooltip,
		type = "simcameradrone",
		name = STRINGS.MOD_CE.GUARDS.DRONE_CRAZY,
		kanim = "kanim_ce_drone_wacky",
		brain = "PacifistBrain",
		sounds = SOUNDS.DRONE_HOVER, 
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/ce_crazy_drone",
		profile_image = "ce_search_drone.png",	
		children = { _OVERRIDE = true },
		traits = util.extend( DEFAULT_DRONE.traits )
		{
			camera_drone = true, 
			controlTimerMax = 2, 
			dashSoundRange = 0,
            dynamicImpass = false,
            empDeath = true,

			recap_icon = "search_drone",
            scanSweeps = false,
            pacifist = true,
            PWROnHand = 1,
			no_look_around = true,
			
			LOSrange = 1.5,
			LOSarc = math.pi * 2,
		},
		dropTable = 
		{
			{nil, 67}
		},
		abilities = { "ce_search_tools" },

	    CAMERA_DRONE = 100,
	},
	
	--FTM ADMIN
	ce_ftm_sysadmin =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SYS_ADMIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ftm_sysadmin_build",	
		profile_image = "ftm_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ftm_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 125, 
			vip=true,
            pacifist = true,
			mainframeRecapture = 2,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},			
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "WimpBrain",
		FTM_ADMIN = 100,
	},
	ce_ftm_sysadmin_new =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SYS_ADMIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ftm_sysadmin_build",	
		profile_image = "ftm_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ftm_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 125, 
			vip=true,
            pacifist = true,
			meleeRecapture = true,
			consoleScan = true,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},			
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "ceGuardbrainSysadmin",
		FTM_ADMIN_NEW = 100,
	},
	
	--K&O ADMIN
	ce_ko_sysadmin =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SYS_ADMIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_sysadmin_build",	
		profile_image = "ko_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ko_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 125, 
			vip=true,
            pacifist = true,
			mainframeRecapture = 2,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},	
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "WimpBrain",
		KO_ADMIN = 100,
	},
	ce_ko_sysadmin_new =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SYS_ADMIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_sysadmin_build",	
		profile_image = "ko_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ko_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 125, 
			vip=true,
            pacifist = true,
			consoleScan = true,
			meleeRecapture = true,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},	
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "ceGuardbrainSysadmin",
		KO_ADMIN_NEW = 100,
	},
	
	--PLASTECH ADMIN
	ce_plastek_sysadmin =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SYS_ADMIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/plas_sysadmin_build",	
		profile_image = "plas_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_plas_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 125, 
			vip=true,
            pacifist = true,
			mainframeRecapture = 2,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},	
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "WimpBrain",
		PLASTECH_ADMIN = 100,
	},
	ce_plastek_sysadmin_new =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SYS_ADMIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/plas_sysadmin_build",	
		profile_image = "plas_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_plas_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 125, 
			vip=true,
            pacifist = true,
			consoleScan = true,
			meleeRecapture = true,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},	
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = { },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "ceGuardbrainSysadmin",
		PLASTECH_ADMIN_NEW = 100,
	},
	
	--SANKAKU ADMIN
	ce_sankaku_sysadmin =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SYS_ADMIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/sk_sysadmin_build",	
		profile_image = "sk_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_sk_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 135, 
			vip=true,
            pacifist = true,
			mainframeRecapture = 2,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},	
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = {},
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "WimpBrain",
		SANKAKU_ADMIN = 100,
	},
	ce_sankaku_sysadmin_new =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SYS_ADMIN,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/sk_sysadmin_build",	
		profile_image = "sk_sys.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_sk_sysadmin",
		
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			cashOnHand = 135, 
			vip=true,
            pacifist = true,
			consoleScan = true,
			meleeRecapture = true,
			investigateHackedDevices = true,
			recap_icon = "executive",
		},
		dropTable =
		{
			{ "item_adrenaline" ,5},
			{ "item_laptop",5},
			{ "item_icebreaker",5},
		    { "item_portabledrive",5},
			{nil,150}
		},
		anarchyDropTable =
		{
			{ "item_laptop_2",5},
			{ "item_icebreaker_2",5},
		    { "item_portabledrive_2",5},
			{ "item_stim",3 },
			{ "item_tazer", 2 },
			{ "item_adrenaline" ,10},
			{ "item_paralyzer" ,10},
			{nil,130}
		},	
		speech = speechdefs.NPC,
		voices = {"Executive"},
		skills = {},
		abilities = { "ce_panicbutton" },
		children = {},
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "ceGuardbrainSysadmin",
		SANKAKU_ADMIN_NEW = 100,
	},
	
	--ALTERNATE TIER 1 GUARDS
	ce_ftm_guard_weak = util.extend( guarddefs.ftm_guard )
	{
		onWorldTooltip = onGuardTooltip,
		traits = util.extend( guarddefs.ftm_guard.traits )
		{
			resistKO = -1,
			LOSperipheralArc = 11*math.pi/16,
			lookaroundArc = 11*math.pi/16,
			mpMax= 9,
			mp= 9,
			recap_icon = "guard",
		},
		FTM_GUARD = 50,
	},
	
	ce_sankaku_guard_weak = util.extend( guarddefs.sankaku_guard )
	{
		onWorldTooltip = onGuardTooltip,
		traits = util.extend( guarddefs.sankaku_guard.traits )
		{
			resistKO = -1,
			LOSperipheralArc = 11*math.pi/16,
			lookaroundArc = 11*math.pi/16,
			mpMax= 9,
			mp= 9,
			recap_icon = "guard",
		},
		SANKAKU_GUARD = 50,
	    SANKAKU_HUMAN_GUARD = 50,
	},
	
	ce_ko_guard_weak = util.extend( guarddefs.ko_guard )
	{
		onWorldTooltip = onGuardTooltip,
		traits = util.extend( guarddefs.ko_guard.traits )
		{
			resistKO = -1,
			LOSperipheralArc = 11*math.pi/16,
			lookaroundArc = 11*math.pi/16,
			mpMax= 9,
			mp= 9,
			recap_icon = "guard",
		},
		KO_GUARD = 50,
	},
	
	ce_plastek_guard_weak = util.extend( guarddefs.plastek_guard )
	{
		onWorldTooltip = onGuardTooltip,
		traits = util.extend( guarddefs.plastek_guard.traits )
		{
			resistKO = -1,
			LOSperipheralArc = 11*math.pi/16,
			lookaroundArc = 11*math.pi/16,
			mpMax= 9,
			mp= 9,
			recap_icon = "guard",
		},
		PLASTECH_GUARD = 50,
	},
	
	--PROTOTYPE GUARDS
	ce_ko_guard_prototype = util.extend( guarddefs.ko_guard )
	{
		name = STRINGS.MOD_CE.GUARDS.KO_PROTOTYPE,
		profile_build = "portraits/ce_ko_sprinter_build",	
		onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_prototype_ko",
		traits = util.extend( guarddefs.ko_guard.traits )
		{
			LOSrange = 4,
			LOSperipheralRange = 5,
			recap_icon = "guard",
			always_patrol = true,
			sneaking = true,
		},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_sprint_pads_silent_passive"},
		sounds = SILENT_SOUNDS,
		KO_GUARD = 50,
	},
	
	ce_ftm_guard_prototype = util.extend( guarddefs.ftm_guard )
	{
		onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_prototype_ftm",
		traits = util.extend( guarddefs.ftm_guard.traits )
		{
			mpMax= 6,
			mp= 6,
			recap_icon = "guard",
			throwsScangrenades = true,
		},
		children = { itemdefs.item_npc_pistol, itemdefs.item_npc_scangrenade },
		FTM_GUARD = 50,
	},
	
	ce_plastek_guard_prototype= {
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.PLASTECH_PROTOTYPE,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_plastech_aug_1_build",			
		profile_image = "ce_plas_aug_1.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_aug_plastech_1",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			
			mainframe_item = true,
			mainframe_ice = 1,
			mainframe_iceMax = 1,	
			mainframe_status = "active",	
			noTakeover = true,
			noRaiseIfIceZero = true,
			mainframe_ice_set_in_def = true,
			mainframe_no_daemon_spawn = true,
			
			mp = 7,
			mpMax = 7,
			
			iceMp = 2,
			LOSarc = math.pi/4 + math.pi/18,
			iceLOS = 3*math.pi/18,
			
			max_bonus_ice = 2,
			
			LOSperipheralArc = math.pi/4,
			mainframebooster = true,
			recap_icon = "psi",
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_icebreaker", 5 },
			{nil,135}
		},
		anarchyDropTable =
		{
			{ "item_portabledrive",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_mainframe_booster"},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = AUGMENTED_SOUNDS,
		brain = "GuardBrain",
		PLASTECH_GUARD = 50,
	},
	
	--FTM ELITE
	ce_scanner_guard =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.GUARD_SCANNER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ftm_scanner_1_build",	
		profile_image = "ce_ftm_scanner_1.png", 

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_scanner_guard_ftm",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			detect_cloak = true,
			always_patrol = true,
			throwsScangrenades = true,
			
			recap_icon = "searcher",
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_stickycam", 10 },
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,20},		    
			{ "item_adrenaline" ,15},
			{ "item_wireless_scanner_1", 10 },
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_ultraviolet_spectrometer_passive", "ce_long_range_sensors_passive" },
		children = { itemdefs.item_npc_pistol, itemdefs.item_npc_scangrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
		FTM_THREAT = 50,	
	},
	
	
	ce_electrical_guard =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.INTERFERER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ftm_interferer_build",	
		profile_image = "ce_ftm_interfere.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ftm_interferer",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			electricalInterference = 1,
			electricalInterferenceRange = 6,
			recap_icon = "searcher",
			koBomb = 2,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_emp_pack", 10 },
			{ "item_flash_pack", 10 },
			{nil,95}
		},
		anarchyDropTable =
		{
		    { "item_stim",5},
			{ "item_clip" ,20},		
			{ "item_flash_pack", 8 },
			{ "item_adrenaline" ,5},
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 5 },
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_electricalInterference", "ce_koBomb" },
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",	
		FTM_THREAT = 50,
	},
	
	ce_ftm_defense_matrix_guard =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SHIELDER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ftm_interferer_build",	
		profile_image = "ce_ftm_interfere.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ftm_interferer",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="disabled",
			enforcer = false,
			dashSoundRange = 8,
			recap_icon = "searcher",
			throwsPulseGrenades = true,

			aoe_shield = 1,
			aoe_shield_range = 4,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_emp_pack", 15 },
			{ "item_flash_pack", 5 },
			{nil,95}
		},
		anarchyDropTable =
		{
		    { "item_stim",5},
			{ "item_clip" ,25},		
			{ "item_flash_pack", 5 },
			{ "item_adrenaline" ,5},
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 5 },
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_aoe_shield", "ce_daemon_core" },
		children = { itemdefs.item_npc_pistol, itemdefs.item_ce_pulsegrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",	
		FTM_THREAT = 50,
	},
	
	--PLASTECH ELITE
	ce_plastek_firewall_guard =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.X,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_plastech_firewall_build",	
		profile_image = "ce_plas_fw.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_firewall_plastech",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			firewallSupportCount = 1,
			firewallSupportStrength = 1,
			firewallSupportRange = 6,
			investigateHackedDevices = true,
			cashOnHand = 90, 
			recap_icon = "psi",			
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_portabledrive" ,5},
			{nil,85}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
			{ "item_laptop_2",5},
			{ "item_portabledrive_3",5 },
		    { "item_stim_3",5},
			{ "item_clip" ,15},		    
			{ "item_adrenaline" ,15},
			{ "item_stickycam",5},
			{nil,125}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_firewallSupport" },
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
		PLASTECH_THREAT = 50,
	},
	
	ce_plastek_melee_guard =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.GHOST,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_plastech_psi_build",	
		profile_image = "ce_plas_psi.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_psi_plastech",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			recap_icon = "psi",	
			meleeFighter = true,
			usesCloaks = true,
			cloakCooldown = 0,
			cloakCooldownMax = 3,
			cloakDuration = 0,
			cloakDurationMax = 1,
			--detect_cloak = true,
		},
		dropTable =
		{
			{ "item_cloakingrig_1" ,10},
			{ "item_power_tazer_1" ,5},
			{ "item_stim",5},
			{nil,80},
		},
		anarchyDropTable =
		{
			{ "item_power_tazer_1",5},
		    { "item_stim",15},
			{ "item_cloakingrig_1" ,20},
			{ "item_cloakingrig_2" ,5},		
			{ "item_smokegrenade",10},    
			{nil,145}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "breakDoor", "ce_melee", "overwatchMelee", "ce_patrol_generator" },
		children = { itemdefs.item_ce_tazer_npc },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "ceGuardbrainMelee",
		PLASTECH_THREAT = 50,		
	},
	
	ce_plastek_psi_guard =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.PSI,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_plastech_psi_build",	
		profile_image = "ce_plas_psi.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_psi_plastech",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			psiSupport = true,
			mp = 7,
			mpMax = 7,
			recap_icon = "psi",	
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_tazer",5},
			{ "item_adrenaline" ,5},
			{ "item_smokegrenade" ,5},
			{ "item_stim",5},
			{nil,100}
		},
		anarchyDropTable =
		{
			{ "item_smokegrenade",5},
			{ "item_defiblance",5},
		    { "item_stim_2",5},
			{ "item_clip" ,15},		    
			{ "item_adrenaline" ,15},
			{ "item_stickycam",5},
			{nil,120}
		},		
		speech = speechdefs.NPC,
		voices = {"KO_Heavy", },
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_psi_support", "ce_neurostim_passive" },
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
		PLASTECH_THREAT = 50,		
	},
	
	ce_plastek_guard_augmented =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.AUGMENTED,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_plastech_aug_1_build",			
		profile_image = "ce_plas_aug_1.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_aug_plastech_1",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			
			mainframe_item = true,
			mainframe_ice = 3,		
			mainframe_iceMax = 3,		
			mainframe_status = "active",	
			noTakeover = true,
			noRaiseIfIceZero = true,
			remove_noRaiseIfIceZero_inHard = true,
			mainframe_ice_set_in_def = true,
			mainframe_no_daemon_spawn = true,
			
			mp = 5,
			mpMax = 5,
			
			iceMp = 2,
			LOSarc = math.pi/4,
			iceLOS = 3*math.pi/18,
			
			max_bonus_ice = 5,
			
			LOSperipheralArc = math.pi/4,
			mainframebooster = true,
			resistKO = 1,
			recap_icon = "psi",
		},
		dropTable =
		{
			{ "item_clip" , 10},
			{ "item_adrenaline", 3 },
			{ "item_tazer" ,5},
			{ "item_tazer_2", 2 },
			{nil,80}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,20},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 5 },
			{ "item_portabledrive_2", 5 },
			{ "item_icebreaker", 5 },
			{nil,140}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_mainframe_booster"},
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = AUGMENTED_SOUNDS,
		brain = "GuardBrain",
		PLASTECH_THREAT = 50,
	},
	
	--K&O ELITE
	ce_ko_thor =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.THOR,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_heavy_build",
		profile_image = "KO_heavy.png",					
    	onWorldTooltip = onGuardTooltip,
		--kanim = "kanim_ce_ko_thor",
		kanim = "kanim_guard_male_ko_heavy",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			mainframe_item = true,
			mainframe_ice = 3,
			mainframe_iceMax = 3,
			mainframe_status = "active",	
			noTakeover = true,
			noRaiseIfIceZero = true,
			remove_noRaiseIfIceZero_inHard = true,
			mainframe_ice_set_in_def = true,
			mainframe_no_daemon_spawn = true,
			mpMax=8,
			mp=8,		
			throwsPulseGrenades = true,
			cashOnHand = 90,
			refracture_from = 0.5,
			refracture_to = 0.5,
			refracture_to_range = 5,
			refracture_from_range = 5,
			refracturedIce = 0,
			recap_icon = "heavy",
		},
		dropTable =
		{
			{ "item_clip" , 6},
			{ "item_defiblance", 5 },
			{ "item_shocktrap", 5 },
			{ "item_stim", 4},
			{ "item_emp_pack", 3 },
			{ "item_hologrenade_17_9", 2 },
			{ "item_stickycam", 1 },
			{ nil, 74 },
		},
		anarchyDropTable =
		{
			{ "item_power_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,5},		    
			{ "item_defiblance", 5 },
			{ "item_shocktrap_2", 5 },
			{ "item_emp_pack_2", 5 },
			{ "item_hologrenade_17_9", 5 },
			{ "item_flash_pack", 1 },
			{ "item_stickycam", 3 },
			{ "item_adrenaline" ,12},
			{nil,135}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "mainframe_attunement_passive", "ce_mainframe_refraction", "ce_magnetic_rifle" },
		children = { itemdefs.item_npc_smg, itemdefs.item_ce_pulsegrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.HEAVY,
		brain = "GuardBrain",
		--KO_THREAT = 50,
	},
	
	ce_ko_sniper =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SNIPER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ko_sniper_build",
		profile_image = "ce_ko_sniper.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_sniper_ko",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 6,	
			mp = 4,
			mpMax = 4,
            --LOSperipheralArc = math.pi/4,
			LOSrange = 30,
			--lookaroundArc = math.pi/4,
			lockdown_color = {r=0,g=1,b=55/255,a=1 },
			recap_icon = "assault",	
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_stim", 6 },
			{ "item_paralyzer", 5 },
			{ "item_adrenaline", 10 },
			{ "item_cloakingrig_1", 5 },
			{ "item_hologrenade_17_9", 5 },
			{nil, 74},
		},
		anarchyDropTable =
		{
			{ "item_light_pistol_KO",15},
			{ "item_tazer",5},
		    { "item_hologrenade_17_9",5},
			{ "item_clip" ,15},		    
			{ "item_adrenaline" ,15},
			{nil,145}
		},	
		speech = speechdefs.NPC,
		voices = {"KO_SpecOps",},
		skills = {},
		abilities = {"overwatch", "breakDoor", "ce_piercing_scope_passive_new", "ce_holographic_pinpointing"},
		--abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_piercing_scope_passive"},
		children = { itemdefs.item_ce_sniper_rifle },
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
		KO_THREAT = 50,
	},
	
	ce_ko_assault =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.ASSAULT,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ko_assault_build",	
		profile_image = "KO_med_2.png",			
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ko_assault",
		traits = util.extend( commondefs.basic_guard_traits )    
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 6,	
			recap_icon = "assault",
			resistKO = -1,
			LOSrange = 4,
			LOSperipheralRange = 5,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_stim_2", 6},
			{ "item_stim_3", 5 },
			{ "item_adrenaline", 10 },
			{nil, 74},
		},
		anarchyDropTable =
		{
			{ "item_light_pistol_KO",5},
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,135}
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_overtuned_reflexes_passive", "ce_regenerative_nanocells_passive" },
		children = { itemdefs.item_npc_smg },
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
		KO_THREAT = 50,
	},
	
	--ENFORCERS
	ce_guard_enforcer_reinforcement_3 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SUPER_ENFORCER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_enforcer_elite_2_build",
		profile_image = "ce_enforcer_elite_2.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_enforcer_elite_2",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 0,
			sneaking = true,
			armor = 2,
			mp = 8,
			mpMax = 8,
			resistKO = 1,
			throwsScangrenades = true,
			recap_icon = "enforcer_armour",
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_sprint_pads_silent_passive" },
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_scangrenade },
		brain = "GuardBrain",
		sounds = SILENT_HEAVY_SOUNDS,
	},	
	ce_guard_enforcer_reinforcement_heavy =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.HEAVY_ENFORCER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_enforcer_heavy_1_build",
		profile_image = "ce_enforcer_heavy_1.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_enforcer_heavy_1",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
			armor = 2,
			mp = 5,
			mpMax = 5,
			resistKO = 1,
			isSupportGuard = true,
			recap_icon = "enforcer_armour",
			LOSrange = 6,
			LOSperipheralRange = 8,
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_regenerative_nanocells_passive" },
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_flashgrenade },
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},
	
	ce_guard_enforcer_reinforcement_heavy_2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.HEAVY_ENFORCER_2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_enforcer_heavy_2_build",
		profile_image = "ce_enforcer_heavy_2.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_enforcer_heavy_2",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
			armor = 3,
			mp = 6,
			mpMax = 6,
			resistKO = 2,
			buffArmorOnKO = 1,
			isSupportGuard = true,
			recap_icon = "enforcer_armour",
			LOSrange = 6,
			LOSperipheralRange = 8,
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_regenerative_nanocells_passive" },
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_flashgrenade },
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},
	
	ce_guard_reckon_enforcer =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.RECON_ENFORCER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_recon_enforcer_build",
		profile_image = "ce_recon_enforcer.png",
    		onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_female_enforcer",
		traits = util.extend( commondefs.basic_guard_traits )
		{
			enforcer = true,
			hidesInCover = true,
			canHide = true,
			sneaking = true,
			silencer = true,
			LOSrange = 8,
			LOSarc = math.pi / 4,
			LOSperipheralRange = 10,
			LOSperipheralArc = math.pi / 2,
         		lookaroundArc = math.pi / 2 + math.pi / 4,
     			lookaroundOffset = math.pi / 4 - math.pi / 8,
			calcAutoFacing = true,
			quick_scan_grenades = "true",
			recap_icon = "enforcer_armour",
		},
		speech = speechdefs.stealth_1,
		voices = {},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_sprint_pads_silent_passive", "consciousness_monitor_passive", "ce_regenerative_nanocells_passive", },
		children = { itemdefs.item_ce_precise_smg, itemdefs.item_ce_scangrenade_eg1 },
		brain = "GuardBrain",
		sounds = EG_SOUNDS,
	},
	
	ce_guard_reckon_enforcer_2 = 
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.RECON_ENFORCER_2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_recon_enforcer_build",
		profile_image = "ce_recon_enforcer.png",	
    		onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_female_enforcer",
		traits = util.extend( commondefs.basic_guard_traits )
		{
			enforcer = true,
			hidesInCover = true,
			canHide = true,
			sneaking = true,
			silencer = true,
			LOSrange = 8,
			LOSarc = math.pi / 4,
			LOSperipheralRange = 10,
			LOSperipheralArc = math.pi / 2,
         		lookaroundArc = math.pi / 2 + math.pi / 4,
     			lookaroundOffset = math.pi / 4 - math.pi / 8,
			calcAutoFacing = true,
			quick_scan_grenades = "true",
			recap_icon = "enforcer_armour",
		},
		speech = speechdefs.stealth_1,
		voices = {},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){ "shootOverwatch", "overwatch", "breakDoor", "ce_sprint_pads_silent_passive_2", "consciousness_monitor_passive", "ce_regenerative_nanocells_passive", },
		children = { itemdefs.item_ce_precise_smg, itemdefs.item_ce_scangrenade_eg1  },
		brain = "GuardBrain",
		sounds = EG_SOUNDS,
	},
	
	ce_inspector =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.INSPECTOR,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_enforcer_elite_2_build",
		profile_image = "ce_enforcer_elite_2.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_enforcer_assault_1",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
			resistKO = -1,
			dartgun = true,
			
			firewallSupportCount = 1,
			firewallSupportStrength = 1,
			firewallSupportRange = 3,
			mainframeRecapture = 3,
			investigateHackedDevices = true,
			recap_icon = "enforcer",
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities ={ "shootOverwatch", "overwatch", "breakDoor", "ce_long_range_sensors_passive" },
		children = { itemdefs.item_ce_dartgun_npc, itemdefs.item_npc_scangrenade },
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},
	
	ce_inspector_2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.INSPECTOR_2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_enforcer_elite_2_build",
		profile_image = "ce_enforcer_elite_2.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_enforcer_assault_1",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
			
			firewallSupportCount = 1,
			firewallSupportStrength = 1,
			firewallSupportRange = 2,
			mainframeRecapture = 4,
			investigateHackedDevices = true,
			recap_icon = "enforcer_armour",
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities ={ "shootOverwatch", "overwatch", "breakDoor", "ce_long_range_sensors_passive", "ce_firewallSupport" },
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_scangrenade },
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},
	
	ce_inspector_3 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.INSPECTOR_3,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_enforcer_elite_2_build",
		profile_image = "ce_enforcer_elite_2.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_enforcer_assault_2",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
			
			LOSperipheralArc = 11*math.pi/16,
			firewallSupportCount = 1,
			firewallSupportStrength = 1,
			firewallSupportRange = 4,
			mainframeRecapture = 5,
			lookaroundArc = 11*math.pi/16,
			investigateHackedDevices = true,
			recap_icon = "enforcer_armour",
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities ={ "shootOverwatch", "overwatch", "breakDoor", "ce_ultraviolet_spectrometer_passive", "ce_long_range_sensors_passive", "ce_firewallSupport" },
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_scangrenade },
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},
	
		ce_guard_enforcer_reinforcement_assault =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.ASSAULT_ENFORCER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_enforcer_elite_2_build",
		profile_image = "ce_enforcer_elite_2.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_enforcer_assault_1",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
			resistKO = -1,
			recap_icon = "enforcer_armour",
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities ={ "shootOverwatch", "overwatch", "breakDoor", "ce_regenerative_nanocells_passive", "ce_overtuned_reflexes_passive" },
		children = { itemdefs.item_npc_smg },
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},
	
	ce_guard_enforcer_reinforcement_assault_2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.ASSAULT_ENFORCER_2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_enforcer_elite_2_build",
		profile_image = "ce_enforcer_elite_2.png",
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_enforcer_assault_2",
		traits = util.extend( commondefs.basic_guard_traits ) 
		{
			heartMonitor="enabled",
			enforcer = true,
			dashSoundRange = 8,
			armor = 1,
			resistKO = -1,
			throwsScangrenades = true,
			recap_icon = "enforcer_armour",
		},
		speech = speechdefs.NPC,
		voices = {"KO_Heavy",},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_regenerative_nanocells_passive", "ce_overtuned_reflexes_passive" },
		children = { itemdefs.item_npc_smg, itemdefs.item_npc_scangrenade },
		brain = "GuardBrain",
		sounds = SOUNDS.ARMORED,
	},
	
	ce_guard_enforcer_reinforcement_drone_3 = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_ENFORCER_3,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/ce_enforcer_drone_3_build",
		profile_image = "ce_enforcer_drone_3.png",					
		kanim = "kanim_ce_drone_enforcer_superheavy",
		sounds = SOUNDS.DRONE_WALK,
    	onWorldTooltip = onGuardTooltip,
		traits = util.extend( DEFAULT_DRONE.traits )   
		{
			enforcer = true,
			mainframe_ice = 5,	
			mainframe_iceMax = 5,
			mainframe_ice_based_on_level = true,
			armor = 2,
			mp = 7,
			mpMax = 7,
			lookaroundRange = 5,
			failsafeAi = true,
			recap_icon = "enforcer_armour",
			ReinforcementPWROnHand = 2,
			
			pulseScan = true,
			range = 2,
			pulse_sound = "SpySociety/Actions/Engineer/wireless_emitter",
		},
		children = {itemdefs.item_drone_turret},	
	},
	ce_guard_enforcer_reinforcement_drone_2 = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_ENFORCER_2,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/ce_enforcer_drone_2_build",
		profile_image = "ce_enforcer_drone_2.png",				
		kanim = "kanim_ce_drone_enforcer_heavy",
		sounds = SOUNDS.DRONE_WALK,
    	onWorldTooltip = onGuardTooltip,
		traits = util.extend( DEFAULT_DRONE.traits )   
		{
			enforcer = true,
			mainframe_ice = 4,	
			mainframe_iceMax = 4,
			mainframe_ice_based_on_level = true,
			armor = 1,
			ReinforcementPWROnHand = 2,
			failsafeAi = true,
			recap_icon = "enforcer_armour",
		},
		children = {itemdefs.item_drone_turret},	
	},
	ce_guard_enforcer_reinforcement_drone = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_ENFORCER,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/ce_enforcer_drone_1_build",
		profile_image = "ce_enforcer_drone_1.png",
		kanim = "kanim_ce_drone_enforcer",
		sounds = SOUNDS.DRONE_WALK,
		onWorldTooltip = onGuardTooltip,
		traits = util.extend( DEFAULT_DRONE.traits )   
		{
			enforcer = true,
			mainframe_ice = 3,	
			mainframe_iceMax = 3,
			mainframe_ice_based_on_level = true,
			ReinforcementPWROnHand = 2,
			failsafeAi = true,
			recap_icon = "enforcer_armour",
		},
		children = {itemdefs.item_drone_turret},	
	},
	
	--SANKAKU ELITE
	ce_sankaku_engineer =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.DRONE_ENGINEER,
		profile_anim = "portraits/portrait_animation_template",
 		profile_build = "portraits/ce_sk_engineer_build",	 
 		profile_image = "ce_sk_engi.png", 
     	onWorldTooltip = onGuardTooltip, 
 		kanim = "kanim_ce_sk_engineer", 
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			cashOnHand = 110,
			
			droneMonitor = 4,
			true_drone_repair = true,
			drone_repair = 2,
			drone_boost = 2,
			drone_boost_range = 6,
			pettier = 0.75,
			drone_upgrade_table = {
				[1] = {min=1,max=2,chance=0.5},
			},
			
			recap_icon = "drone_engineer",
		},
		dropTable =
		{
			{ "item_emp_pack", 10 },
			{ "item_item_icebreaker_2", 5 },
			{ "item_portabledrive_2", 5 },
			{ "item_scanchip", 15 },
			{nil, 70}
		},
		anarchyDropTable =
		{
			{ "item_clip" ,20},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack_2", 15 },
			{ "item_wireless_scanner_1", 5 },
			{ "item_portabledrive_3", 5 },
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_droneMonitor", "ce_drone_boost", "ce_spawnDrone" },
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "ceGuardbrainEngineer",
		SANKAKU_THREAT = 25,
		SANKAKU_HUMAN_THREAT = 25,
	},
	
	ce_sk_grenade_drone = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_SUPPORT,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/ce_drone_grenade",
		profile_image = "drone_grenade.png",				
		kanim = "kanim_ce_drone_grenade",
		sounds = SOUNDS.DRONE_WALK,
    	onWorldTooltip = onGuardTooltip,
		abilities = { "ce_throw_support" },
		traits = util.extend( DEFAULT_DRONE.traits )   
		{
			pacifist = true,
			isSupportGuard = true,
			maxThrow = 15,
			recap_icon = "drone_support",
			
			PCGrenadesThrown = 0,
			maxPCGrenades = 1,
		},
		dropTable = 
		{
			{ "item_portabledrive", 5 },
			{ "item_portabledrive_2", 2 },
			{ "item_icebreaker", 5 },
			{ "item_icebreaker_2", 2 },
			{ "item_scanchip", 2 },
			{ "item_flashgrenade", 4 },
			{nil, 130}
		},
		brain = "ceSupportBrain",
		children = {itemdefs.item_npc_flashgrenade},
		SANKAKU_GUARD = 50,
	},
	
	ce_tech_drone = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_TECH,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/ce_drone_tech_build",
		profile_image = "drone_tech.png",					
		kanim = "kanim_ce_drone_tech",
		sounds = SOUNDS.DRONE_WALK,
    	onWorldTooltip = onGuardTooltip,
		abilities = { "shootOverwatch", "overwatch", "ce_alarm_sync" },
		traits = util.extend( DEFAULT_DRONE.traits )   
		{
			mainframe_ice = 2,	
			mainframe_iceMax = 2,
			failsafeAi = true,
			lookaroundRange = 7,
			empKO = 2,
			mainframe_no_daemon_spawn = false,
			PWROnHand = 3,
			--selfFirewallBuff = 1,
			recap_icon = "drone_tech",
		},
		dropTable = 
		{
			{ "item_hologrenade_17_9", 10 },
			{ "item_scanchip", 10 },
			{ "item_stickycam", 5 },
			{ "item_portabledrive", 15 },
			{ "item_portabledrive_2", 5 },
			{ "item_portabledrive_3", 3 },
			{ "item_icebreaker", 20 },
			{ "item_icebreaker_2", 5 },
			{ "item_icebreaker_3", 3 },
			{ "item_laptop_2", 4 },
			{ "item_laptop", 5 },
			{nil, 315}
		},
		anarchyDropTable =
		{
			{ "item_hologrenade_17_9", 10 },
			{ "item_scanchip", 10 },
			{ "item_stickycam", 5 },
			{ "item_portabledrive", 15 },
			{ "item_portabledrive_2", 5 },
			{ "item_portabledrive_3", 3 },
			{ "item_icebreaker", 20 },
			{ "item_icebreaker_2", 5 },
			{ "item_icebreaker_3", 3 },
			{ "item_laptop_2", 4 },
			{ "item_laptop", 5 },
			{nil, 315}
		},
		children = {itemdefs.item_drone_turret},
		SANKAKU_THREAT = 25,
	},
	
	ce_sk_drone_barrier = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_BARRIER,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/sankaku_drone_face_new",
		profile_image = "sankaku_drone.png",		
		kanim = "kanim_drone_SA",
		traits = util.extend( DEFAULT_DRONE.traits )
		{
			mainframe_ice = 3,		
			mainframe_iceMax = 3,

			weakBarrier = 1,
			shieldArmor = true,
			firewallArmor = true,
			firewallShield = true,
			shields = 1,
			armor = 0,
			recap_icon = "drone_null",
		},
		sounds = SOUNDS.DRONE_WALK,
		--SANKAKU_THREAT = 50,
		children = {itemdefs.item_drone_turret},	
	},
	
	ce_sk_drone_protocol = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_PROTOCOL,
		kanim = "kanim_drone_null_SA", 
		sounds = SOUNDS.DRONE_HOVER,
		profile_anim = "portraits/sankaku_drone_null_new",
		profile_image = "sankaku_drone_null.png",		
		brain = "PacifistBrain",
		abilities = { _OVERRIDE = true, "ce_mp_pulse", "ce_drone_communications" --[["ce_patrol_generator",]]--[[ "ce_reverseIonField",]] --[["ce_keibi_grid_uplink"]] },
		children = { _OVERRIDE = true },
		traits = util.extend( DEFAULT_DRONE.traits )
		{
			mainframe_ice = 2,
			mainframe_iceMax = 2,
			
			daemon_shuffle = 3,
			keibi_bonus = 1,

			mp_siphon = 2,
			mp_pulse = 5,
			
			selfFirewallBuff = 1,
			--lookaroundRange = 3,
			--reverseIonField = 4,
			high_frequency_scanner = true,
            dynamicImpass = false,
			scanSweeps = true,
            pacifist = true,
			recap_icon = "drone_tech",
		},
		dropTable = 
		{
			{ "item_hologrenade_17_9", 20 },
			{ "item_stickycam", 10 },
			{ "item_portabledrive", 15 },
			{ "item_portabledrive_2", 5 },
			{ "item_portabledrive_3", 3 },
			{ "item_icebreaker", 5 },
			
			{nil, 110}
		},
		anarchyDropTable =
		{
			{ "item_portabledrive_3", 3 },
			{ "item_hologrenade_17_9", 20 },
			{ "item_stickycam", 10 },
			{ "item_laptop_3", 5 },
			{ "item_laptop_2", 5 },
			{ "item_laptop", 5 },
			{ "steelcrow_item_router_icemelter", 20 },
			{ nil, 110 },
		},
		--SANKAKU_THREAT = 50,
	},
	
	--OMNI
	ce_omni_hacker = 
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.OMNI_HACKER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_omni_hacker_build",
		profile_image = "ce_omni_hacker.png",								
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_omni_hacker",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
            omni = true,
			investigateHackedDevices = true,	
			mainframeRecapture = 6,	
			koDaemon = true,
			mainframe_suppress_rangeMax = 1,
			mainframe_suppress_range = 1,
			--firewallSupport = 1,
			firewallSupportCount = 1,
			firewallSupportStrength = 1,
			firewallSupportRange = 6,
			recap_icon = "omni_hunter",
		},
		dropTable = 
		{
			{nil, 67}
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_firewallSupport" },
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
		OMNI_GUARD = 75,
		KO_LV2 = 60,
		FTM_LV2 = 60,
		PLASTECH_LV2 = 25,
		SANKAKU_LV2 = 60,
		NEPTUNE_LV2 = 75,
	},
	
	ce_omni_drone = util.extend( DEFAULT_DRONE )
	{
		type = "simcameradrone",
		name = STRINGS.MOD_CE.GUARDS.OMNI_DRONE,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/ce_drone_omni_build",
		profile_image = "ce_omni_drone.png",
		kanim = "kanim_ce_drone_omni",
		sounds = SOUNDS.DRONE_WALK,
		onWorldTooltip = onGuardTooltip,
		brain = "PacifistBrain",
		
		children = { _OVERRIDE = true },
		abilities = { _OVERRIDE = true, "ce_alarm_sync", --[["ce_patrol_generator", "ce_electricalInterference"]] },
		traits = util.extend( commondefs.basic_robot_traits )
		{
			camera_drone = true, 
			--controlTimerMax = 2,
			mainframe_ice = 3,	
			mainframe_iceMax = 3,
			recap_icon = "drone_camera",
			selfFirewallBuff = 1,
			
			magnetic_reinforcement = true,
			mainframe_always_daemon_spawn = true,
			lookaroundRange = 5,
			empKO = 3,
			mainframe_no_daemon_spawn = false,
			omni = true,
			
            scanSweeps = true,
            pacifist = true,
            PWROnHand = 2,
		},
		dropTable = 
		{
			{nil, 67}
		},
		CE_OMNI_DRONE = 100,
	},
	
	ce_omni_drone_LV2 = 
	{
		type = "simdrone",
		name = STRINGS.MOD_CE.GUARDS.OMNI_DRONE_2,
		profile_anim = "portraits/sankaku_drone_tank_face_new",
		profile_build = "portraits/ce_destroyer_build",
		profile_image = "ce_destroyer.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_destroyer",
		rig = "dronerig",
		traits = util.extend( commondefs.basic_robot_traits )   
		{
			walk=true,
			enforcer = false,
			dashSoundRange = 8,		
			mpMax=5,
			mp=5,			
			
			mainframe_no_daemon_spawn = false,
			mainframe_always_daemon_spawn = true,
            mainframe_ice = 4,
            mainframe_iceMax = 4,
			selfFirewallBuff = 1,
            empKO = 2,
			omni = true,

			LOSarc = 11*math.pi/16,
			failsafeAi = true,
			high_frequency_scanner = true,
			magnetic_reinforcement = true,
			
			--[[
			failsafeAiDuration = 4,
			ko_weak = true,
			refracture_from = 0.3,
			refracture_to = 0.4,
			refracture_to_range = 5,
			refracture_from_range = 5,
			refracturedIce = 0,
			
			mainframe_suppress_rangeMax = 2,
			mainframe_suppress_range = 2,]]
			
			aoe_shield = 1,
			aoe_shield_range = 3,

			armor = 1,
			PWROnHand = 4,
            lookaroundPeripheralArc = math.pi,

            lookaroundRange = 4,
			recap_icon = "omni_drone",
		},

		voices = {"Drone"},
		speech = speechdefs.NPC,
		skills = {},
		dropTable = 
		{
			--[[{ "item_icebreaker", 12 },
			{ "item_icebreaker_2", 3 },
			{ "item_icebreaker_3", 2 },
			{ "item_portabledrive_3", 1 },
			{ "item_portabledrive_2", 2 },
			{ "item_portabledrive", 12 },
			{ "item_icebreaker_3" , 3 },]]
			{nil, 65}
		},
		abilities =  { "shootOverwatch", "overwatch", "breakDoor", "ce_aoe_shield" --[[, "ce_alarm_sync", "ce_ko_weak", "ce_mainframe_refraction", "ce_patrol_generator"]]  },
		children = {itemdefs.item_npc_smg_drone},
		sounds = SOUNDS.DRONE_HOVER_HEAVY,
		brain = "DroneBrain",

		OMNI_GUARD = 50,
		KO_LV2 = 60,
		FTM_LV2 = 60,
		PLASTECH_LV2 = 60,
		SANKAKU_LV2 = 20,
		NEPTUNE_LV2 = 75,
	},	
	
	ce_omni_hunter = 
	{
		type = "simunit",
		name = STRINGS.GUARDS.OMNI_HUNTER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/omni_hunter_face",
		profile_image = "sankaku_med_2.png",								
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_omni_hunter",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			armor = 1,	
            omni = true,
			detect_cloak = true,
			always_patrol = true,
			throwsScangrenades = true,
			drone_boost_range = 8,
			drone_boost = 2,
			sneaking = true,
			dashSoundRange = 0,
			
			recap_icon = "omni_hunter",
		},
		dropTable = 
		{
			{nil, 67}
		},
		beginnerTraits = 
		{
			armor = 0, 
		},
		speech = speechdefs.NPC,
		voices = {"KO_SpecOps",},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_sprint_pads_silent_passive", "ce_ultraviolet_spectrometer_passive", "ce_long_range_sensors_passive", "ce_drone_boost" },
		children = { itemdefs.item_npc_rifle, itemdefs.item_npc_scangrenade },
		idles = DEFAULT_IDLES,
		OMNI_GUARD = 60,
		KO_LV2 = 60,
		FTM_LV2 = 25,
		PLASTECH_LV2 = 60,
		SANKAKU_LV2 = 60,
		NEPTUNE_LV2 = 75,
		sounds = SILENT_SOUNDS,
		brain = "GuardBrain",
	},
	
	ce_omni_killer = 
	{
		type = "simunit",
		name = STRINGS.GUARDS.OMNI_KILLER,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_omni_killer_build",
		profile_image = "ce_omni_killer.png",								
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_omni_killer",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
            omni = true,
			--specialGrenades = true,
			isSupportGuard = true,
			recap_icon = "omni_killer",
		},
		dropTable = 
		{
			--{ "item_flashgrenade",1 },
			{nil, 67}
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_overtuned_reflexes_passive", "ce_regenerative_nanocells_passive" },
		children = { itemdefs.item_npc_smg_omni, itemdefs.item_npc_flashgrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "GuardBrain",
		OMNI_GUARD = 30,
		KO_LV2 = 20,
		FTM_LV2 = 50,
		PLASTECH_LV2 = 50,
		SANKAKU_LV2 = 50,
		NEPTUNE_LV2 = 60,
	},
	
	--FTM ELITE LEVEL 2
	ce_scanner_guard_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.GUARD_SCANNER_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ftm_scanner_2_build",	 
 		profile_image = "ce_ftm_scanner_2.png", 
     	onWorldTooltip = onGuardTooltip, 
 		kanim = "kanim_ce_scanner_guard_ftm_2", 

		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			detect_cloak = true,
			always_patrol = true,
			armor = 1,
			recap_icon = "searcher2",
			throwsScangrenades = true,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_stickycam", 10 },
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,20},		    
			{ "item_adrenaline" ,15},
			{ "item_wireless_scanner_1", 10 },
			{nil,150}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_ultraviolet_spectrometer_passive", "ce_long_range_sensors_passive", "ce_ultrasonic_echolocation_passive" },
		children = { itemdefs.item_npc_pistol, itemdefs.item_npc_scangrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
		FTM_LV2 = 400,	
	},
	
	ce_electrical_guard_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.INTERFERER_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ftm_interferer_2_build",	
		profile_image = "ce_ftm_interfere.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ftm_interferer_2",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			armor = 1,
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			electricalInterference = 2,
			electricalInterferenceRange = 8,
			koBomb = 3,
			recap_icon = "searcher",
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_emp_pack", 10 },
			{ "item_flash_pack", 10 },
			{nil,95}
		},
		anarchyDropTable =
		{
		    { "item_stim",5},
			{ "item_clip" ,20},		
			{ "item_flash_pack", 8 },
			{ "item_adrenaline" ,5},
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 5 },
			{nil,150}
		},	
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_electricalInterference", "ce_koBomb" },
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",	
		FTM_LV2 = 400,
	},
	
	ce_ftm_defense_matrix_guard_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SHIELDER_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ftm_interferer_build",	
		profile_image = "ce_ftm_interfere.png",

    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ftm_interferer",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="disabled",
			enforcer = false,
			dashSoundRange = 8,
			recap_icon = "searcher",
			throwsPulseGrenades = true,

			armor = 1,
			aoe_shield = 2,
			aoe_shield_range = 6,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_emp_pack", 15 },
			{ "item_flash_pack", 5 },
			{nil,95}
		},
		anarchyDropTable =
		{
		    { "item_stim",5},
			{ "item_clip" ,25},		
			{ "item_flash_pack", 5 },
			{ "item_adrenaline" ,5},
			{ "item_emp_pack", 15 },
			{ "item_emp_pack_2", 5 },
			{nil,150}
		},			
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_aoe_shield", "ce_daemon_core" },
		children = { itemdefs.item_npc_pistol, itemdefs.item_ce_pulsegrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",	
		FTM_LV2 = 400,
	},
	
	--PLASTECH ELITE LEVEL 2
	ce_plastek_guard_augmented_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.AUGMENTED_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_plastech_aug_2_build",			
		profile_image = "ce_plas_aug_2.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_aug_plastech_2",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			
			mainframe_item = true,
			mainframe_ice = 5,		
			mainframe_iceMax = 5,	
			armor = 1,
			magnetic_reinforcement = true,
			mainframe_status = "active",	
			noTakeover = true,
			noRaiseIfIceZero = true,
			remove_noRaiseIfIceZero_inHard = true,
			mainframe_ice_set_in_def = true,
			mainframe_no_daemon_spawn = true,
			
			mp = 5,
			mpMax = 5,
			
			iceMp = 2,
			LOSarc = math.pi/4,
			iceLOS = 3*math.pi/18,
			
			max_bonus_ice = 6,
			
			LOSperipheralArc = math.pi/4,
			mainframebooster = true,
			resistKO = 1,
			recap_icon = "psi2",
		},
		dropTable =
		{
			{ "item_clip" , 10},
			{ "item_adrenaline", 3 },
			{ "item_tazer" ,5},
			{ "item_tazer_2", 2 },
			{nil,80}
		},
		anarchyDropTable =
		{
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,20},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack", 5 },
			{ "item_portabledrive_2", 5 },
			{ "item_icebreaker", 5 },
			{nil,140}
		},	
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_mainframe_booster"},
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = AUGMENTED_SOUNDS,
		brain = "GuardBrain",
		PLASTECH_LV2 = 400,
	},
	
	ce_plastek_psi_guard_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.PSI_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_plastech_psi_build",	
		profile_image = "ce_plas_psi.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_psi_plastech",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			psiSupport = true,
			recap_icon = "psi2",
			armor = 1,
			neurostim = 2,

			firewallSupportCount = 1,
			firewallSupportStrength = 1,
			firewallSupportRange = 6,
			mp = 7,
			mpMax = 7,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_tazer",5},
			{ "item_adrenaline" ,5},
			{ "item_smokegrenade" ,5},
			{ "item_stim",5},
			{nil,100}
		},
		anarchyDropTable =
		{
			{ "item_smokegrenade",5},
			{ "item_defiblance",5},
		    { "item_stim_2",5},
			{ "item_clip" ,15},		    
			{ "item_adrenaline" ,15},
			{ "item_stickycam",5},
			{nil,120}
		},		
		speech = speechdefs.NPC,
		voices = {"KO_Heavy", },
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_psi_support", "ce_firewallSupport", "ce_neurostim_passive" },
		children = { itemdefs.item_npc_pistol },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
		PLASTECH_LV2 = 400,		
	},
	
	ce_plastek_melee_guard_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.GHOST_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_plastech_psi_build",	
		profile_image = "ce_plas_psi.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_psi_plastech",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			recap_icon = "psi2",	
			meleeFighter = true,
			usesCloaks = true,
			cloakCooldown = 0,
			cloakCooldownMax = 2,
			cloakDurationMax = 2,
			cloakDuration = 0,
			armor = 1,
			always_patrol = true,
			sneaking = true,
			--detect_cloak = true,
		},
		dropTable =
		{
			{ "item_cloakingrig_1" ,10},
			{ "item_power_tazer_1" ,5},
			{ "item_stim",5},
			{nil,80},
		},
		anarchyDropTable =
		{
			{ "item_power_tazer_1",5},
		    { "item_stim",15},
			{ "item_cloakingrig_1" ,20},
			{ "item_cloakingrig_2" ,5},		
			{ "item_smokegrenade",10},    
			{nil,145}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "breakDoor", "ce_melee", "overwatchMelee", "ce_patrol_generator", "ce_sprint_pads_silent_passive" },
		children = { itemdefs.item_ce_tazer_npc_heavy },
		idles = DEFAULT_IDLES,
		sounds = SILENT_SOUNDS,
		brain = "ceGuardbrainMelee",
		PLASTECH_LV2 = 400,		
	},
	
	--SANKAKU ELITE LEVEL 2
	ce_tech_drone_LV2 = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_TECH_LV2,
		profile_anim = "portraits/sankaku_drone_heavy_face_new",
		profile_build = "portraits/ce_drone_tech_heavy_build",
		profile_image = "drone_tech_heavy.png",					
		kanim = "kanim_ce_drone_tech_LV2",
		sounds = SOUNDS.DRONE_WALK,
    	onWorldTooltip = onGuardTooltip,
		abilities = { "shootOverwatch", "overwatch", "ce_alarm_sync", "ce_ultraviolet_spectrometer_passive" },
		traits = util.extend( DEFAULT_DRONE.traits )   
		{
			mainframe_ice = 6,	
			mainframe_iceMax = 6,
			mainframe_always_daemon_spawn = true,
			failsafeAi = true,
			lookaroundRange = 8,
			empKO = 2,
			mainframe_no_daemon_spawn = false,
			magnetic_reinforcement = true,
			PWROnHand = 3,
			--selfFirewallBuff = 2,
			pulseScan = true,
			range = 3,
			always_patrol = true,
			pulse_sound = "SpySociety/Actions/Engineer/wireless_emitter",
			recap_icon = "drone_tech2",
		},
		dropTable = 
		{
			{ "item_hologrenade_17_9", 10 },
			{ "item_scanchip", 10 },
			{ "item_stickycam", 5 },
			{ "item_portabledrive", 15 },
			{ "item_portabledrive_2", 5 },
			{ "item_portabledrive_3", 3 },
			{ "item_icebreaker", 20 },
			{ "item_icebreaker_2", 5 },
			{ "item_icebreaker_3", 3 },
			{ "item_laptop_2", 4 },
			{ "item_laptop", 5 },
			{nil, 265}
		},
		anarchyDropTable =
		{
			{ "item_hologrenade_17_9", 10 },
			{ "item_scanchip", 10 },
			{ "item_stickycam", 5 },
			{ "item_portabledrive", 15 },
			{ "item_portabledrive_2", 5 },
			{ "item_portabledrive_3", 3 },
			{ "item_icebreaker", 20 },
			{ "item_icebreaker_2", 5 },
			{ "item_icebreaker_3", 3 },
			{ "item_laptop_2", 4 },
			{ "item_laptop", 5 },
			{nil, 265}
		},
		children = {itemdefs.item_drone_turret},
		SANKAKU_LV2 = 400,	
	},
	
	ce_sk_drone_protocol_LV2 = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_PROTOCOL_2,
		kanim = "kanim_drone_null_SA", 
		sounds = SOUNDS.DRONE_HOVER,
		profile_anim = "portraits/sankaku_drone_null_new",
		profile_image = "sankaku_drone_null.png",		
		brain = "PacifistBrain",
		abilities = { _OVERRIDE = true, "ce_patrol_generator",--[[ "ce_reverseIonField",]] "ce_keibi_grid_uplink" },
		children = { _OVERRIDE = true },
		traits = util.extend( DEFAULT_DRONE.traits )
		{
			mainframe_ice = 4,
			mainframe_iceMax = 4,
			
			daemon_shuffle = 3,
			keibi_bonus = 2,

			armor = 1,
			selfFirewallBuff = 2,
			lookaroundRange = 5,
			--reverseIonField = 6,
			high_frequency_scanner = true,
            dynamicImpass = false,
			scanSweeps = true,
            pacifist = true,
			recap_icon = "drone_tech",
		},
		dropTable = 
		{
			{ "item_hologrenade_17_9", 20 },
			{ "item_stickycam", 10 },
			{ "item_portabledrive", 15 },
			{ "item_portabledrive_2", 5 },
			{ "item_portabledrive_3", 3 },
			{ "item_icebreaker", 5 },
			
			{nil, 110}
		},
		anarchyDropTable =
		{
			{ "item_portabledrive_3", 3 },
			{ "item_hologrenade_17_9", 20 },
			{ "item_stickycam", 10 },
			{ "item_laptop_3", 5 },
			{ "item_laptop_2", 5 },
			{ "item_laptop", 5 },
			{ "steelcrow_item_router_icemelter", 20 },
			{ nil, 110 },
		},
		--SANKAKU_LV2 = 400,
	},
	
	ce_sk_drone_barrier_LV2 = util.extend( DEFAULT_DRONE )
	{
		name = STRINGS.MOD_CE.GUARDS.DRONE_BARRIER_LV2,
		profile_anim = "portraits/sankaku_drone_face_new",
		profile_build = "portraits/sankaku_drone_face_new",
		profile_image = "sankaku_drone.png",		
		kanim = "kanim_drone_SA",
		traits = util.extend( DEFAULT_DRONE.traits )
		{
			mainframe_ice = 6,		
			mainframe_iceMax = 6,
			mainframe_no_daemon_spawn = false,
			mainframe_always_daemon_spawn = true,
			magnetic_reinforcement = true,
			high_frequency_scanner = true,
			recap_icon = "sankaku_drone_null2",

			shieldArmor = true,
			firewallArmor = true,
			firewallShield = true,
			shields = 1,
			armor = 0,
		},
		sounds = SOUNDS.DRONE_WALK,
		--SANKAKU_LV2 = 400,
		children = {itemdefs.item_drone_turret},	
	},
	
	ce_sankaku_engineer_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.DRONE_ENGINEER_LV2,
		profile_anim = "portraits/portrait_animation_template",
 		profile_build = "portraits/ce_sk_engineer_build",	 
 		profile_image = "ce_sk_engi.png", 
     	onWorldTooltip = onGuardTooltip, 
 		kanim = "kanim_ce_sk_engineer_2", 
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,	
			cashOnHand = 110,
			
			true_drone_repair = true,
			droneMonitor = 6,
			drone_repair = 4,
			drone_boost = 4,
			drone_boost_range = 8,
			pettier = 1.5,
			armor = 1,
			
			omni_upgrade = true,
			
			drone_upgrade_table = {
				[1] = {min=1,max=2,chance=0.75},
				[2] = {min=1,max=2,chance=0.5},
			},
			
			recap_icon = "drone_engineer",
		},
		dropTable =
		{
			{ "item_emp_pack", 10 },
			{ "item_item_icebreaker_2", 5 },
			{ "item_portabledrive_2", 5 },
			{ "item_scanchip", 15 },
			{nil, 70}
		},
		anarchyDropTable =
		{
			{ "item_clip" ,20},		    
			{ "item_adrenaline" ,15},
			{ "item_emp_pack_3", 15 },
			{ "item_wireless_scanner_1", 5 },
			{ "item_portabledrive_3", 5 },
			{nil,150}
		},	
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_droneMonitor", "ce_drone_boost", "ce_spawnDrone" },
		children = { itemdefs.item_npc_pistol2 },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.ARMORED,
		brain = "ceGuardbrainEngineer",
		SANKAKU_LV2 = 400,
	},
	
	--K&O ELITE LEVEL 2
	ce_ko_assault_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.ASSAULT_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ko_assault_build",
		profile_image = "KO_med_2.png",			
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_ko_assault",
		traits = util.extend( commondefs.basic_guard_traits )    
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 6,	
			recap_icon = "assault2",	
			resistKO = 1,
			armor = 1,
			always_patrol = true,
			sneaking = true,
			LOSrange = 4,
			LOSperipheralRange = 6,
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_stim_2", 6},
			{ "item_stim_3", 5 },
			{ "item_adrenaline", 10 },
			{nil, 74},
		},
		anarchyDropTable =
		{
			{ "item_light_pistol_KO",5},
			{ "item_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,25},		    
			{ "item_adrenaline" ,15},
			{nil,135}
		},
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "ce_overtuned_reflexes_passive", "ce_regenerative_nanocells_passive", "ce_sprint_pads_silent_passive" },
		children = { itemdefs.item_npc_smg },
		sounds = SILENT_SOUNDS,
		brain = "GuardBrain",
		KO_LV2 = 400,
	},
	ce_ko_grenade_guard_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SUPPORT_GUARD_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_grenadier_build",
		profile_image = "KO_grenadier.png",	
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_grenade_ko",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			specialGrenades = true,
			armor = 1,
			recap_icon = "KO_heavy2",
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_adrenaline" ,5},
			{ "item_flashgrenade" ,50},
			{nil,90}
		},
		anarchyDropTable =
		{
			{ "item_tazer_2",5},
		    { "item_stim",5},
			{ "item_flashgrenade" ,50},	    
			{ "item_adrenaline" ,15},
			{nil,150}
		},			
					
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = util.extend(DEFAULT_ABILITIES){},
		children = { itemdefs.item_npc_pistol, itemdefs.item_ce_omnigrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.GUARD,
		--KO_LV2 = 400,
		brain = "GuardBrain",		
		blurb = "<ttheader>EMPLOYED</>\nIt is corporate policy to invest in life insurance for each an every one of it's valued employees. In event of accident causing injury or permanent system failure, dependents will receive insurance benefits in an order as noted by form CSF_312.\n<ttrule>Read all fine Print</>\n",
	},
	ce_ko_sniper_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.SNIPER_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ce_ko_sniper_2_build",
		profile_image = "ce_ko_sniper_2.png",		
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_ce_sniper_ko_2",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 6,	
			mp = 5,
			mpMax = 5,
			armor = 1,
            --LOSperipheralArc = math.pi/4,
			LOSrange = 30,
			--lookaroundArc = math.pi/4,
			lockdown_color = {r=1,g=120/255,b=0,a=1 },
			recap_icon = "assault2",	
		},
		dropTable =
		{
			{ "item_clip" ,5},
			{ "item_stim", 6 },
			{ "item_paralyzer", 5 },
			{ "item_adrenaline", 10 },
			{ "item_cloakingrig_1", 5 },
			{ "item_hologrenade_17_9", 5 },
			{nil, 74},
		},
		anarchyDropTable =
		{
			{ "item_light_pistol_KO",15},
			{ "item_tazer",5},
		    { "item_hologrenade_17_9",5},
			{ "item_clip" ,15},		    
			{ "item_adrenaline" ,15},
			{nil,145}
		},	
		speech = speechdefs.NPC,
		voices = {"KO_SpecOps",},
		skills = {},
		abilities = {"overwatch", "breakDoor", "ce_piercing_scope_passive_new", "ce_holographic_pinpointing","ce_ultrasonic_echolocation_passive"},
		--abilities = {"shootOverwatch", "overwatch", "breakDoor", "ce_piercing_scope_passive"},
		children = { itemdefs.item_ce_sniper_rifle },
		sounds = SOUNDS.GUARD,
		brain = "GuardBrain",
		KO_LV2 = 400,
	},
	ce_ko_thor_LV2 =
	{
		type = "simunit",
		name = STRINGS.MOD_CE.GUARDS.THOR_LV2,
		profile_anim = "portraits/portrait_animation_template",
		profile_build = "portraits/ko_heavy_build",
		profile_image = "KO_heavy.png",					
    	onWorldTooltip = onGuardTooltip,
		kanim = "kanim_guard_male_ko_heavy",
		traits = util.extend( commondefs.basic_guard_traits )   
		{
			walk=true,
			heartMonitor="enabled",
			enforcer = false,
			dashSoundRange = 8,
			mainframe_item = true,
			mainframe_ice = 5,
			mainframe_iceMax = 5,
			mainframe_status = "active",	
			noTakeover = true,
			noRaiseIfIceZero = true,
			remove_noRaiseIfIceZero_inHard = true,
			mainframe_ice_set_in_def = true,
			mainframe_no_daemon_spawn = true,
			mpMax=8,
			mp=8,
			armor = 1,
			buffArmorOnKO = 1,
			buffArmorOnKO_sfx = "SpySociety_DLC001/Actions/flackguard_armourincrease",	
			throwsPulseGrenades = true,
			cashOnHand = 90,
			refracture_from = 0.8,
			refracture_to = 0.8,
			refracture_to_range = 8,
			refracture_from_range = 8,
			refracturedIce = 0.2,
			recap_icon = "KO_heavy2"
		},
		dropTable =
		{
			{ "item_clip" , 6},
			{ "item_defiblance", 5 },
			{ "item_shocktrap", 5 },
			{ "item_stim", 4},
			{ "item_emp_pack", 3 },
			{ "item_hologrenade_17_9", 2 },
			{ "item_stickycam", 1 },
			{ nil, 74 },
		},
		anarchyDropTable =
		{
			{ "item_power_tazer",5},
		    { "item_stim",5},
			{ "item_clip" ,5},		    
			{ "item_defiblance", 5 },
			{ "item_shocktrap_2", 5 },
			{ "item_emp_pack_2", 5 },
			{ "item_hologrenade_17_9", 5 },
			{ "item_flash_pack", 1 },
			{ "item_stickycam", 3 },
			{ "item_adrenaline" ,12},
			{nil,135}
		},		
		speech = speechdefs.NPC,
		voices = {"Guard_1", "Guard_2", "Guard_3"},
		skills = {},
		abilities = { "shootOverwatch", "overwatch", "breakDoor", "mainframe_attunement_passive", "ce_mainframe_refraction", "ce_magnetic_rifle" },
		children = { itemdefs.item_npc_smg, itemdefs.item_ce_pulsegrenade },
		idles = DEFAULT_IDLES,
		sounds = SOUNDS.HEAVY,
		brain = "GuardBrain",
		--KO_LV2 = 400,
	},
}

return npc_templates
