local itemdefs = include( "sim/unitdefs/itemdefs" )
local util = include( "modules/util" )
local commondefs = include( "sim/unitdefs/commondefs" )

local item_ce_pulsegrenade = util.extend( commondefs.npc_grenade_template ) {
	type = "pulse_grenade",
	name = "PULSE GRENADE",
	desc = "You probably shouldn't see this tooltip",
	--icon = "itemrigs/FloorProp_Bandages.png",
	profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_scanner_small.png",	
	profile_icon_100 = "gui/icons/item_icons/icon-item_scanner.png",		
	kanim = "kanim_scangrenade",		
	sounds = { explode="SpySociety/Actions/EMP_explo", bounce="SpySociety/Grenades/bounce" },
	traits = { range=1, aimRange = 4, explodes = 0, throwUnit="item_ce_pulsegrenade_thrown", keepPathing=false },
	value = 600,
	floorWeight = 2, 
}

local defs = {
	item_ce_sniper_rifle = util.extend( itemdefs.item_npc_rifle )
	{
		id = "item_ce_sniper_rifle",
		traits = { weaponType="rifle", baseDamage = 2, armorPiercing = 2 },
		weapon_anim = "kanim_ce_sniper_rifle",
	},

	item_ce_dartgun_npc = util.extend( commondefs.npc_weapon_template )
	{
		name =  STRINGS.ITEMS.DART_GUN_AMMO,
		desc =  STRINGS.ITEMS.DART_GUN_AMMO_TOOLTIP,
		flavor = STRINGS.ITEMS.DART_GUN_AMMO_FLAVOR,
		icon = "itemrigs/FloorProp_Precision_Pistol.png",	
		--profile_icon = "gui/items/icon-item_gun_pistol.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_gun_dart_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_gun_dart.png",				
		equipped_icon = "gui/items/equipped_pistol.png",
		traits = { weaponType="pistol", baseDamage = 2, canSleep = true },
		sounds = {shoot="SpySociety/Weapons/Precise/shoot_dart", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup",shell="SpySociety/Weapons/Shells/shell_dartgun_wood"},
		weapon_anim = "kanim_precise_revolver_fix",
		agent_anim = "anims_1h",
		value = 900,
	},
		
	item_ce_omnigrenade = util.extend( itemdefs.item_npc_flashgrenade )
	{
		name="omnigrenade",
		id = "item_ce_omnigrenade",
		traits = { baseDamage = 2, canSleep = true, explodes = 0, range=3, throwUnit="item_ce_omnigrenade", keepPathing=true },
	},

	item_ce_tazer_npc = {
		name = "enemytazer",
		traits = { slot = "melee", melee = true, damage = 2, armorPiercing = 1 },
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_tazer_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_tazer.png",	
		type = "simunit",
		abilities = { "equippable" },
		value = 1,
		createUpgradeParams = function( self, unit )
			return { traits = { autoEquip = (unit:getTraits().equipped == true) } }
		end,
	},

	item_ce_tazer_npc_heavy = {
		name = "enemytazer_2",
		traits = { slot = "melee", melee = true, damage = 3, armorPiercing = 2 },
		icon = "itemrigs/FloorProp_AmmoClip.png",
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_tazer_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_tazer.png",	
		type = "simunit",
		abilities = { "equippable" },
		value = 1,
		createUpgradeParams = function( self, unit )
			return { traits = { autoEquip = (unit:getTraits().equipped == true) } }
		end,
	},

	item_ce_omnigrenade_hard = util.extend( itemdefs.item_npc_flashgrenade )
	{
		name="hardgrenade",
		id = "item_ce_omnigrenade_hard",
		traits = { baseDamage = 3, canSleep = true, explodes = 0, range=0, throwUnit="item_ce_killgrenade", keepPathing=true },
	},

	item_ce_killgrenade = util.extend( itemdefs.item_npc_flashgrenade )
	{
		name="killgrenade",
		id = "item_ce_killgrenade",
		traits = { baseDamage = 3, canSleep = true, explodes = 0, range=4, throwUnit="item_ce_killgrenade", keepPathing=true },
	},

	item_ce_scangrenade_hard_thrown = util.extend( itemdefs.item_npc_scangrenade )
	{
		name="hardscangrenade_thrown",
		traits = { range=5, aimRange = 4, scan=true, explodes = 0, throwUnit="item_ce_scangrenade_hard_thrown", keepPathing=true },
	},

	item_ce_scangrenade_hard = util.extend( itemdefs.item_npc_scangrenade )
	{
		name="hardscangrenade",
		traits = { range=1, aimRange = 4, scan=true, explodes = 0, throwUnit="item_ce_scangrenade_hard_thrown", keepPathing=true },
	},
	
	item_ce_pulsegrenade = item_ce_pulsegrenade,
	
	item_ce_pulsegrenade_thrown = util.extend( item_ce_pulsegrenade )
	{
		name = "PULSE GRENADE THROWN",
		traits = { range=5, aimRange = 4, explodes = 0, throwUnit="item_ce_pulsegrenade_thrown", keepPathing=false },
	},

	item_ce_pulsegrenade_hard_thrown = util.extend( item_ce_pulsegrenade )
	{
		name = "PULSE GRENADE HARD THROWN",
		traits = { range=6, raise_cooldowns = 2, aimRange = 5, explodes = 0, throwUnit="item_ce_pulsegrenade_hard_thrown", keepPathing=true },
	},
	item_ce_pulsegrenade_hard = util.extend( item_ce_pulsegrenade )
	{
		name = "PULSE GRENADE HARD",
		traits = { range=0, raise_cooldowns = 2, aimRange = 5, explodes = 0, throwUnit="item_ce_pulsegrenade_hard_thrown", keepPathing=true },
	},
	
	item_ce_calm_pistol = util.extend( commondefs.weapon_template )
	{
		name =  STRINGS.MOD_CE.ITEMS.CALMER_GUN,
		desc =  STRINGS.MOD_CE.ITEMS.CALMER_GUN_TIP,
		flavor = STRINGS.MOD_CE.ITEMS.CALMER_GUN_FLAVOR,
		icon = "itemrigs/FloorProp_Pistol.png",	
		profile_icon = "gui/icons/item_icons/items_icon_small/icon-item_gun_microchip_small.png",
		profile_icon_100 = "gui/icons/item_icons/icon-item_gun_microchip.png",				
		equipped_icon = "gui/items/equipped_pistol.png",
		traits = { weaponType="pistol", baseDamage = 0, pwrCost= 1, calmingGun= true, noTargetAlert=true, ignoreArmor=true, armorPiercing = math.huge, nopwr_guards = {}, doNotInterruptMove = true},
		abilities = util.tmerge( { "ce_gun_calm" }, commondefs.weapon_template.abilities ) ,
		sounds = {shoot="SpySociety/Weapons/Precise/shoot_dart", reload="SpySociety/Weapons/LowBore/reload_handgun", use="SpySociety/Actions/item_pickup"},
		weapon_anim = "kanim_precise_revolver",
		agent_anim = "anims_1h",
		value = 300,
	},
}

return defs