local array = include( "modules/array" )
local util = include( "modules/util" )
local version = include( "modules/version" )

local guarddefsDefaultTraits = {}
local guarddefsDefaultChildren = {}

-- init will be called once
local function init( modApi )
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	KLEIResourceMgr.MountPackage( dataPath .. "/gui.kwad", "data" )
	KLEIResourceMgr.MountPackage( dataPath .. "/anims.kwad", "data" )
	KLEIResourceMgr.MountPackage( dataPath .. "/anims2.kwad", "data" )
	modApi:addGenerationOption("ce_sysadmins", STRINGS.MOD_CE.OPTIONS.ENABLE_SYSADMINS , STRINGS.MOD_CE.OPTIONS.ENABLE_SYSADMINS_TIP, {noUpdate = true, value = 0, values = {0,1,2,3}, strings = STRINGS.MOD_CE.OPTIONS.ENABLE_SYSADMINS_STRINGS } )
	modApi:addGenerationOption("ce_sysadmin_response", STRINGS.MOD_CE.OPTIONS.SYSADMIN_RESPONSE , STRINGS.MOD_CE.OPTIONS.SYSADMIN_RESPONSE_TIP, {value = 5, values = {0,1,2,3,4,5,6}, strings = STRINGS.MOD_CE.OPTIONS.SYSADMIN_RESPONSE_STRINGS } )
	modApi:addGenerationOption("ce_guards", STRINGS.MOD_CE.OPTIONS.ENABLE_ENEMIES , STRINGS.MOD_CE.OPTIONS.ENABLE_ENEMIES_TIP, {noUpdate = true} )
	modApi:addGenerationOption("ce_enforcers", STRINGS.MOD_CE.OPTIONS.ENABLE_ENFORCERS , STRINGS.MOD_CE.OPTIONS.ENABLE_ENFORCERS_TIP, {noUpdate = true} )
	modApi:addGenerationOption("ce_omni", STRINGS.MOD_CE.OPTIONS.ENABLE_OMNI , STRINGS.MOD_CE.OPTIONS.ENABLE_OMNI_TIP, {noUpdate = true} )
	modApi:addGenerationOption("ce_patrols", STRINGS.MOD_CE.OPTIONS.ENABLE_PATROLS , STRINGS.MOD_CE.OPTIONS.ENABLE_PATROLS_TIP, {noUpdate = true} )
	modApi:addGenerationOption("ce_keycards", STRINGS.MOD_CE.OPTIONS.ENABLE_KEYCARDS , STRINGS.MOD_CE.OPTIONS.ENABLE_KEYCARDS_TIP, {noUpdate = true} )
	modApi:addGenerationOption("ce_validate", STRINGS.MOD_CE.OPTIONS.ENABLE_VALIDATE , STRINGS.MOD_CE.OPTIONS.ENABLE_VALIDATE_TIP , {noUpdate = true})
	modApi:addGenerationOption("ce_suspicious", STRINGS.MOD_CE.OPTIONS.ENABLE_SUSPICIOUS , STRINGS.MOD_CE.OPTIONS.ENABLE_SUSPICIOUS_TIP , {noUpdate = true, values = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15}, value=0, strings = STRINGS.MOD_CE.OPTIONS.ENABLE_SUSPICIOUS_VALUES})
	modApi:addGenerationOption("ce_tell_buddy", STRINGS.MOD_CE.OPTIONS.ENABLE_TELL_BUDDY , STRINGS.MOD_CE.OPTIONS.ENABLE_TELL_BUDDY_TIP , {noUpdate = true})
	modApi:addGenerationOption("ce_tips", STRINGS.MOD_CE.OPTIONS.ENABLE_TIPS , STRINGS.MOD_CE.OPTIONS.ENABLE_TIPS_TIP , {noUpdate = true})
	
	if modApi.mod_manager.lib_sequentialModLoader and version.isVersionOrHigher( modApi.mod_manager.lib_sequentialModLoader.version, "2.0.0" ) then
		modApi:addGenerationOption("ce_grenades", STRINGS.MOD_CE.OPTIONS.ENABLE_HARDGRENADES , STRINGS.MOD_CE.OPTIONS.ENABLE_HARDGRENADES_TIP,{enabled=false, noUpdate = true}  ) 
		modApi:addGenerationOption("ce_hardmode", STRINGS.MOD_CE.OPTIONS.ENABLE_HARDMODE , STRINGS.MOD_CE.OPTIONS.ENABLE_HARDMODE_TIP,{enabled=false, noUpdate = true}  ) 
	else
		modApi:addGenerationOption("ce_hardmode_fail", STRINGS.MOD_CE.OPTIONS.ENABLE_HARDMODE_FAIL , STRINGS.MOD_CE.OPTIONS.ENABLE_HARDMODE_FAIL_TIP, {noUpdate = true} )
	end

	util.weighted_list.addChoice = function( self, item, weight )
		if type(weight) == "number" then
			table.insert( self, weight )
			table.insert( self, item )
			self._totalWeight = self._totalWeight + weight
		end
	end
    local logs = include( scriptPath .. "/logs" )
    for i,log in ipairs(logs) do      
        modApi:addLog(log)
    end
	
	include( scriptPath .. "/commondefs" )
	include( scriptPath .. "/simquery" )
	include( scriptPath .. "/actions" )
	include( scriptPath .. "/worldgen" )
	include( scriptPath .. "/simdefs" )
	include( scriptPath .. "/aiplayer" )
	include( scriptPath .. "/idle" )
	include( scriptPath .. "/astar_handlers" )
	include( scriptPath .. "/inventory" )
	include( scriptPath .. "/pather" )
	include( scriptPath .. "/unitrig" )
	include( scriptPath .. "/senses" )
	include( scriptPath .. "/boardrig" )
	include( scriptPath .. "/agentrig" )
	include( scriptPath .. "/cellrig" )
	include( scriptPath .. "/ceGuardbrainMelee" )
	include( scriptPath .. "/ceGuardbrainEngineer" )
	include( scriptPath .. "/ceGuardbrainSysadmin" )
	include( scriptPath .. "/ceSupportBrain" )
	include( scriptPath .. "/simgrenade" )
	include( scriptPath .. "/engine" )
	include( scriptPath .. "/break_ice" )
	--include( scriptPath .. "/hud" )
	modApi:addAbilityDef("ce_melee", scriptPath .. "/ce_melee" )
	modApi:addAbilityDef("ce_throw_support", scriptPath .. "/ce_throw_support" )
end

local function lateInit( modApi )
    local scriptPath = modApi:getScriptPath()

	include( scriptPath .. "/simunit" )
	include( scriptPath .. "/line_of_sight" )
	include( scriptPath .. "/combat_panel" )
	include( scriptPath .. "/mainframe" )
end

local function earlyInit( modApi )
    local scriptPath = modApi:getScriptPath()

	include( scriptPath .. "/shootOverwatch" )
end

local function unload( modApi )
	local worldgen = include( "sim/worldgen" )
	worldgen.CE_OMNI_DRONE = {}
	worldgen.FTM_ADMIN = {}
	worldgen.KO_ADMIN = {}
	worldgen.PLASTECH_ADMIN = {}
	worldgen.SANKAKU_ADMIN = {}
	worldgen.FTM_ADMIN_NEW = {}
	worldgen.KO_ADMIN_NEW = {}
	worldgen.PLASTECH_ADMIN_NEW = {}
	worldgen.SANKAKU_ADMIN_NEW = {}
	
	for i, tip in ipairs( STRINGS.MOD_CE.LOADING_TIPS ) do
		array.removeElement( STRINGS.LOADING_TIPS, tip )
	end
end

-- load may be called multiple times with different options enabled
local function load( modApi, options, params )
    local scriptPath = modApi:getScriptPath()
	local dataPath = modApi:getDataPath()
	modApi:addTooltipDef( {onAgentTooltip = function( tooltip, unit )
		if unit:getTraits().holoTarget and unit:getTraits().holoTarget > 0 then
			tooltip:addAbility( STRINGS.MOD_CE.TOOLTIPS.HOLOTARGET, STRINGS.MOD_CE.TOOLTIPS.HOLOTARGET_DESC,"gui/icons/arrow_small.png" )
		end
	end} )
	
	local animdefs = include( scriptPath .. "/animdefs" )
    for name, animDef in pairs(animdefs) do
        modApi:addAnimDef( name, animDef )
    end
	
    local itemdefs = include( scriptPath .. "/itemdefs" )
    for name, itemDef in pairs(itemdefs) do
        modApi:addItemDef( name, itemDef )
    end
	unload( modApi )
	--Remember to include guarddefs *after* adding itemdefs
	local guarddefs = include( scriptPath .. "/guarddefs" )
	local worldgen = include( "sim/worldgen" )
	if options["ce_guards"].enabled then
		for name, guarddef in pairs(guarddefs) do
			if not guarddef.traits.enforcer and not guarddef.traits.vip then
				modApi:addGuardDef( name, guarddef )
				if guarddef.CE_OMNI_DRONE then
					worldgen.CE_OMNI_DRONE[name] = guarddef.CE_OMNI_DRONE
				end
				if worldgen.NEPTUNE then
					if guarddef.NEPTUNE_GUARD then
						table.insert(worldgen.NEPTUNE.GUARD,{name,guarddef.NEPTUNE_GUARD})
					end
					if guarddef.NEPTUNE_THREAT then
						table.insert(worldgen.NEPTUNE.THREAT,{name,guarddef.NEPTUNE_THREAT})
					end
					if guarddef.NEPTUNE_LV2 then
						table.insert(worldgen.NEPTUNE.LVL2,{name,guarddef.NEPTUNE_LV2})
					end
				end
			end
		end  
	end	
	if (options["ce_guards"] and not options["ce_sysadmins"]) or (options["ce_sysadmins"] and options["ce_sysadmins"].enabled) then
		for name, guarddef in pairs(guarddefs) do
			if guarddef.traits.vip then
				modApi:addGuardDef( name, guarddef )
				if guarddef.FTM_ADMIN then
					worldgen.FTM_ADMIN[name] = guarddef.FTM_ADMIN
				end
				if guarddef.KO_ADMIN then
					worldgen.KO_ADMIN[name] = guarddef.KO_ADMIN
				end
				if guarddef.PLASTECH_ADMIN then
					worldgen.PLASTECH_ADMIN[name] = guarddef.PLASTECH_ADMIN
				end
				if guarddef.SANKAKU_ADMIN then
					worldgen.SANKAKU_ADMIN[name] = guarddef.SANKAKU_ADMIN
				end
				if guarddef.FTM_ADMIN_NEW then
					worldgen.FTM_ADMIN_NEW[name] = guarddef.FTM_ADMIN_NEW
				end
				if guarddef.KO_ADMIN_NEW then
					worldgen.KO_ADMIN_NEW[name] = guarddef.KO_ADMIN_NEW
				end
				if guarddef.PLASTECH_ADMIN_NEW then
					worldgen.PLASTECH_ADMIN_NEW[name] = guarddef.PLASTECH_ADMIN_NEW
				end
				if guarddef.SANKAKU_ADMIN_NEW then
					worldgen.SANKAKU_ADMIN_NEW[name] = guarddef.SANKAKU_ADMIN_NEW
				end
			end
		end
	end
	if options["ce_enforcers"] and options["ce_enforcers"].enabled then
		for name, guarddef in pairs(guarddefs) do
			if guarddef.traits.enforcer then
				modApi:addGuardDef( name, guarddef )
			end
		end
	end
	if options["ce_patrols"] and options["ce_patrols"].enabled then
		modApi:addGuardDef( "ce_dummy_patrol", {name="Dummy",traits={}} )
	end
	if options["ce_omni"] and options["ce_omni"].enabled then
		modApi:addGuardDef( "ce_dummy_omni", {name="Dummy",traits={}} )
	end
	if options["ce_keycards"] and options["ce_keycards"].enabled then
		modApi:addGuardDef( "ce_dummy_doorkeys", {name="Dummy",traits={}} )
	end
	if options["ce_validate"] and options["ce_validate"].enabled then
		for name, daemonAbility in pairs(include(scriptPath.."/npc_abilities")) do
			modApi:addDaemonAbility( name, daemonAbility )
		end
	end
	if options["ce_sysadmins"] and params then
		params.ce_sysadmins = options["ce_sysadmins"].value
	end
	if options["ce_suspicious"] and params then
		params.ce_suspicious = options["ce_suspicious"].value
	end
	if options["ce_tell_buddy"] and params then
		params.ce_tell_buddy = options["ce_tell_buddy"].enabled
	end
	if options["ce_tips"] and options["ce_tips"].enabled then
		util.tmerge( STRINGS.LOADING_TIPS, STRINGS.MOD_CE.LOADING_TIPS )
	end
	if options["ce_sysadmin_response"] and params then
		if options["ce_sysadmin_response"].value > 3 then
			params.ce_sysadmin_response_type = "panicElevator"
			params.ce_sysadmin_response = options["ce_sysadmin_response"].value - 3
		else
			params.ce_sysadmin_response_type = "panicButton"
			params.ce_sysadmin_response = options["ce_sysadmin_response"].value
		end
	end
	
	for eventType, handlerFile in pairs( include( scriptPath.."/viz/viz" ) ) do
		modApi:addVizEvHandler( eventType, include( scriptPath.."/viz/"..handlerFile ))
	end
	
	modApi:addGuardDef( "ce_dummy_pather", {name="Dummy",traits={}} )
	local passives = include( scriptPath .. "/passive_abilities" )
	local passive_abilities = include( "sim/abilities/passive_abilities" )
	for name, passive in pairs(passives) do
		passive_abilities[name] = passive
	end
end

local function isElite( corp, name )
	if corp then
		return array.findIf( corp.THREAT_FIX or corp.THREAT, function(ar)return ar[1] == name end )
	end
end

local function updateItem( item, copy, hard, grenade )
	if hard then
		if item.traits.neptuneMpReduce then
			copy.traits.neptuneMpReduce = item.traits.neptuneMpReduce + 1
		end
		if item.traits.neptuneTagDuration then 
			copy.traits.neptuneTagDuration = item.traits.neptuneTagDuration + 1
		end
	end
	
	if grenade then
		if item.traits.explodes then 
			copy.traits.keepPathing = true
			copy.traits.throwUnit = copy.id
		end
	end
end

local function updateItems( guardId, unitdef, hard, grenade )
	local itemdefs = include("sim/unitdefs/itemdefs")
	
	for i, item in ipairs(unitdef.children or {}) do
		if type(item) == "string" then
			item = itemdefs[item]
		end
		if item.id == nil then
			for k, v in pairs(itemdefs) do
				v.id = k
			end
		end
		
		local newName = item.id .. "_ce_hard"
		
		if itemdefs[newName] then
			if not guarddefsDefaultChildren[guardId] then
				guarddefsDefaultChildren[guardId] = util.tdupe( unitdef.children )
			end
			unitdef.children[i] = itemdefs[newName]
		else
			local copy = {
				traits = {},
				id = newName
			}
			
			updateItem( item, copy, hard, grenade )
			
			if next(copy.traits) then
				if not guarddefsDefaultChildren[guardId] then
					guarddefsDefaultChildren[guardId] = util.tdupe( unitdef.children )
				end
			
				itemdefs[newName] = util.extend( itemdefs[item.id] )( copy )
				unitdef.children[i] = itemdefs[newName]
			end
		end
	end
end

local function lateLoad( modApi, options )
	local scriptPath = modApi:getScriptPath()
	local guarddefs = include( "sim/unitdefs/guarddefs" )
	local array = include("modules/array")
	local worldgen = include( "sim/worldgen" )
	local itemdefs = include("sim/unitdefs/itemdefs")
	
	for i, k in pairs(guarddefs) do
		if i ~= "spider_mod" then
			local unitdef = guarddefs[i]
			
			if not guarddefsDefaultTraits[i] then
				guarddefsDefaultTraits[i] = unitdef.traits
			end
			
			unitdef.traits = util.tcopy(guarddefsDefaultTraits[i])
			if guarddefsDefaultChildren[i] then
				unitdef.children = util.tdupe(guarddefsDefaultChildren[i])
			end
			
			local traits = unitdef.traits
			local commondefs = include( scriptPath .. "/commondefs" )
			unitdef.onWorldTooltip = commondefs.onGuardTooltip
			if (i == "pulse_drone") then
				traits.LOSarc = math.pi * 2
			end
			
			if options["ce_hardmode"] and options["ce_hardmode"].enabled then
				local eliteGuard = false
				if isElite( worldgen.KO, i ) or isElite( worldgen.SANKAKU, i ) or isElite( worldgen.PLASTECH, i ) or isElite( worldgen.FTM, i ) or isElite( worldgen.NEPTUNE, i ) then
					eliteGuard = true
				end
				
				--BASIC
				traits.hardmode = true
				traits.mpMax = (traits.mpMax or 0) + 2
				traits.mp =(traits.mp or 0) + 2
				
				--FIELD OF VIEW
				if traits.LOSperipheralRange then traits.LOSperipheralRange = (traits.LOSperipheralRange or 0)* 1.5 end
				traits.LOSrange = math.floor((traits.LOSrange or 0) * 3 + 1) / 2
				if (traits.LOSperipheralArc or 0) > 0 then traits.LOSperipheralArc = traits.LOSperipheralArc + math.pi/4 end
				if (traits.LOSarc or 0) > 0 then traits.LOSarc = traits.LOSarc+ math.pi/4 end
				if traits.lookaroundRange then
					traits.lookaroundRange = traits.lookaroundRange+2
				end
				traits.lookaroundArc = (traits.lookaroundArc or (math.pi/2 + math.pi/8)) + math.pi/4
				traits.lookaroundOffset = (traits.lookaroundOffset or (math.pi/4 - math.pi/16)) + math.pi/4
				
				--SELF BUFF
				traits.neurostim = (traits.neurostim or 1) * 2
				if traits.selfFirewallBuff then traits.selfFirewallBuff = traits.selfFirewallBuff + 1 end
				if traits.buffArmorOnKO then traits.buffArmorOnKO = traits.buffArmorOnKO * 2 end
				
				--RESISTANCES
				if traits.resistKO and traits.resistKO > 0 then traits.resistKO = traits.resistKO + 1 end
				traits.armor = math.floor((traits.armor or 0) *1.4)
				traits.empKO = math.max((traits.empKO or 0) -1,2)
				if traits.remove_noRaiseIfIceZero_inHard then
					traits.noRaiseIfIceZero = nil
				end
				
				--FIREWALL
				traits.bonus_ice =(traits.bonus_ice or 0) + 1
				if traits.max_bonus_ice then traits.max_bonus_ice = traits.max_bonus_ice + 1 end
				if (traits.mainframe_ice or 0) > 0 then traits.mainframe_ice = math.ceil(traits.mainframe_ice *1.2) end
				if (traits.mainframe_iceMax or 0) > 0 then traits.mainframe_iceMax = math.ceil(traits.mainframe_iceMax *1.2) end
				
				--AOE PROTECTION
				traits.aoe_shield = (traits.aoe_shield or 0) + 1
				traits.aoe_shield_range = (traits.aoe_shield_range or 0) + 2
				traits.electricalInterference =(traits.electricalInterference or 0) + 1
				traits.electricalInterferenceRange =(traits.electricalInterferenceRange or 0) + 2
				
				--AOE MAINFRAME
				if traits.mainframeRecapture then traits.mainframeRecapture = traits.mainframeRecapture + 2 end
				if traits.firewallSupportCount then traits.firewallSupportCount = traits.firewallSupportCount * 2 end
				if traits.firewallSupportRange then traits.firewallSupportRange = traits.firewallSupportRange + 2 end
				if traits.mainframe_suppress_rangeMax then traits.mainframe_suppress_rangeMax =traits.mainframe_suppress_rangeMax + 1 end
				if traits.mainframe_suppress_range then traits.mainframe_suppress_range=traits.mainframe_suppress_range + 1 end
				traits.range =(traits.range or 0) + 2
				
				--DRONE SUPPORT
				if traits.drone_repair then traits.drone_repair = traits.drone_repair + 1 end
				if traits.drone_boost then traits.drone_boost = math.floor(traits.drone_boost * 1.5) end
				if traits.drone_boost_range then traits.drone_boost_range = traits.drone_boost_range + 2 end
				
				--NEPTUNE
				if traits.frostZone then traits.frostZone = traits.frostZone + 1 end
				if traits.frostZoneRange then traits.frostZoneRange = traits.frostZoneRange + 1 end
				
				--UNUSED
				if traits.AOEFirewallsBuffRange then traits.AOEFirewallsBuffRange=traits.AOEFirewallsBuffRange + 1 end
				if traits.AOEFirewallsBuff then traits.AOEFirewallsBuff =traits.AOEFirewallsBuff*2 end
				traits.reverseIonField = (traits.reverseIonField or 0) + 2
				traits.keibi_bonus = (traits.keibi_bonus or 1) * 2
				
				--ELITE
				if eliteGuard and traits.armor == 1 or ((traits.armor or 0) >= 3 and not (traits.resistKO and traits.resistKO > 0))then
					traits.resistKO = (traits.resistKO or 0) + 1
					traits.LOSarc = traits.LOSarc + math.pi/8
					traits.mp = traits.mp+1
					traits.mpMax = traits.mpMax+1
				end
			end
			
			local grenades = options["ce_grenades"] or options["ce_hardmode"]
			updateItems( i, unitdef, options["ce_hardmode"] and options["ce_hardmode"].enabled, grenades and grenades.enabled )
		end
	end
end

local function lateUnload()
	local guarddefs = include( "sim/unitdefs/guarddefs" )
	
	for i, k in pairs(guarddefs) do
		if guarddefsDefaultTraits[i] then
			guarddefs[i].traits = util.tcopy(guarddefsDefaultTraits[i])
		end
		if guarddefsDefaultChildren[i] then
			guarddefs[i].children = util.tdupe(guarddefsDefaultChildren[i])
		end
	end
end


-- gets called before localization occurs and before content is loaded
local function initStrings( modApi )
    local dataPath = modApi:getDataPath()
    local scriptPath = modApi:getScriptPath()
	
	include( scriptPath.."/triggers" )
	
    local CE_STRINGS = include( scriptPath .. "/strings" )
    modApi:addStrings( dataPath, "MOD_CE", CE_STRINGS )  
end

return {
    init = init,
	lateInit = lateInit,
	unload = unload,
    load = load,
	lateLoad = lateLoad,
	lateUnload = lateUnload,
    initStrings = initStrings,
	earlyInit = earlyInit,
}
