local array = include( "modules/array" )
local serverdefs = include( "modules/serverdefs" )
local mathutil = include( "modules/mathutil" )
local array = include( "modules/array" )
local util = include( "modules/util" )
local modifiers = include( "sim/modifiers" )
local simdefs = include("sim/simdefs")
local simquery = include("sim/simquery")
local simplayer = include("sim/simplayer")
local unitdefs = include( "sim/unitdefs" )
local worldgen = include( "sim/worldgen" )
local simfactory = include( "sim/simfactory" )

local EPSILON = 0.0001

-------------------------------------------------------------------------------
-- These are passive abilities (no executeAbility function)

local function formatToolTip( header, body )
	return string.format( "<ttheader>%s\n<ttbody>%s</>", util.toupper(header), body )
end

local DEFAULT_PASSIVE =
{
	getName = function( self, sim, unit )
		return self.name
	end,
		
	createToolTip = function( self,sim,unit,targetUnit)
		return formatToolTip( self.name, string.format("PASSIVE\n%s", self.desc ) )
	end,

	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	
	ghostable = true,

	executeAbility = nil, -- Passives by definition have no execute.
}

local DEFAULT_DEBUFF =
{
	debuffAbility = true, 

	getName = function( self, sim, unit )
		return self.name
	end,
		
	createToolTip = function( self,sim,unit,targetUnit)
		return formatToolTip( self.name, string.format("DEBUFF\n%s", self.desc ) )
	end,

	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	
	ghostable = true,

	executeAbility = nil, -- Passives by definition have no execute.
}

local DEFAULT_BUFF =
{
	buffAbility = true, 

	getName = function( self, sim, unit )
		return self.name
	end,
		
	createToolTip = function( self,sim,unit,targetUnit)
		return formatToolTip( self.name, string.format("BUFF\n%s", self.desc ) )
	end,

	canUseAbility = function( self, sim, unit )
		return false -- Passives are never 'used'
	end,
	
	ghostable = true,

	executeAbility = nil, -- Passives by definition have no execute.
}

local function isOK(unit)
	return unit and unit:getLocation() and unit:isValid() and not unit:isDown()
end

local function isBrainy(unit)
	return isOK(unit) and unit:getPlayerOwner():isNPC() and unit:getBrain()
end

local passive_abilities =
{
	--Override of default mainframe attunement
	mainframe_attunement_passive = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.RESEARCH.MAINFRAME_ATTUNEMENT.NAME, 
		buffDesc = STRINGS.RESEARCH.MAINFRAME_ATTUNEMENT.UNIT_DESC, 

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self, unit )
			self.prevIce = 0
			unit:addMPMax(-2)
			self:onTrigger( sim, nil, nil, unit ) 		
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			local ice = userUnit:getTraits().mainframe_ice or 0
			if ice < 0 then ice = 0 end
			
			if self.prevIce ~= ice then
				userUnit:addMPMax(ice - self.prevIce)
				if userUnit:getTraits().mp > userUnit:getTraits().mpMax then 
					userUnit:getTraits().mp = userUnit:getTraits().mpMax 
				end
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
				self.prevIce = ice
			end
		end, 
	},

	ce_mainframe_booster = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.MAINFRAME_BOOST.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.MAINFRAME_BOOST.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_EMP, self, unit )
			sim:addTrigger( simdefs.TRG_END_TURN, self, unit )
			
			local mods = unit:getModifiers()
			
			self.modifier_LOSarc, self.LOSarcI = mods:add("LOSarc","mainframeBoost",modifiers.ADD,0)
			--self.modifier_mpMax, self.mpMaxI = mods:add("mpMax","mainframeBoost",modifiers.ADD,0)
			self.prevIce = 0
			
			self:onTrigger( sim, nil, nil, unit )  
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_EMP, self )
			sim:removeTrigger( simdefs.TRG_END_TURN, self )
			
			unit:getModifiers():remove("mainframeBoost")
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )  
			local ice = userUnit:getTraits().mainframe_ice or 0
			if ice < 0 then ice = 0 end
			if userUnit:getTraits().mainframe_status == "off" then ice = 0 end
			
			if self.prevIce ~= ice then
				--Removed, it makes it stronger in hard mode but change to use modifiers instead of fixed values already does that
				--ice = ice + (userUnit:getTraits().bonus_ice or 0)
				
				--Unrelated to the above code despite the name, it's simply a cap for the overall effect
				if userUnit:getTraits().max_bonus_ice and ice > userUnit:getTraits().max_bonus_ice then
					ice = userUnit:getTraits().max_bonus_ice
				end
				
				--[[local ap = 5 + ice*2 
				userUnit:getTraits().LOSarc = math.min(math.pi/4 + ice*3*math.pi/18,math.pi*2)
				userUnit:getTraits().lookaroundArc = math.min(math.pi/4 + ice*3*math.pi/18,math.pi*2)
				userUnit:getTraits().mpMax = ap]]
				
				userUnit:getModifiers():change("LOSarc",self.modifier_LOSarc,nil,ice * userUnit:getTraits().iceLOS)
				userUnit:addMPMax((ice - self.prevIce) * userUnit:getTraits().iceMp)
				--userUnit:getModifiers():change("mpMax",self.modifier_mpMax,nil,ice * userUnit:getTraits().iceMp)
				
				
				--To match the standard for other guards' lookaround values
				--Not sure why it's like this, but it's possibly to avoid the admittedly kinda ugly wide swinging motions you see in hard mode
				--userUnit:getTraits().lookaroundArc = userUnit:getTraits().LOSarc + math.pi/8
				if userUnit:getTraits().mp > userUnit:getTraits().mpMax then 
					userUnit:getTraits().mp = userUnit:getTraits().mpMax 
				end 
				
				sim:refreshUnitLOS( userUnit )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
				self.prevIce = ice
			end
		end, 
	},
	
	ce_firewallSupport = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.MAINFRAME_SUPPORT.NAME, 
		getDesc = function(traits)return string.format(STRINGS.MOD_CE.RESEARCH.MAINFRAME_SUPPORT.UNIT_DESC,traits.firewallSupportCount,(traits.firewallSupportCount > 1 and "devices") or "device",traits.firewallSupportRange)end,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )	
			sim:getNPC().checkForFireWallSupport = function() end
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN and evData:isPC() and not userUnit:isKO() and not userUnit:isDead() then
				local mainframeItems = {}
				
				local x0,y0 = userUnit:getLocation()
				for i,unit in pairs(sim:getAllUnits())do
					if unit:getTraits().mainframe_item and unit:getTraits().mainframe_ice and not unit:isPC() and unit:getTraits().mainframe_ice - unit:getTraits().mainframe_iceMax < userUnit:getTraits().firewallSupportStrength and not (unit:getTraits().noRaiseIfIceZero and unit:getTraits().mainframe_ice <= 0) then
						local x1,y1 = unit:getLocation()			
						local distance = mathutil.dist2d( x0, y0, x1, y1 )
						if distance <= userUnit:getTraits().firewallSupportRange then
							table.insert(mainframeItems,unit)
						end
					end
				end
				table.sort(mainframeItems,function(a,b)return a:getTraits().mainframe_ice < b:getTraits().mainframe_ice end)
				for i=1, math.min(#mainframeItems,userUnit:getTraits().firewallSupportCount) do
					local unit = mainframeItems[i]
					local delta = math.min(userUnit:getTraits().firewallSupportStrength,unit:getTraits().mainframe_iceMax + userUnit:getTraits().firewallSupportStrength - unit:getTraits().mainframe_ice)
					
					unit:getTraits().mainframe_ice = unit:getTraits().mainframe_ice + delta
					sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = unit, ice = unit:getTraits().mainframe_ice, delta = delta } )
					sim:triggerEvent( simdefs.TRG_ICE_BROKEN, { unit = mainframeItems[i], delta = delta } )
					
					--sim:dispatchEvent( simdefs.EV_SHOW_WARNING, {txt=STRINGS.UI.ROUTER_ALERT, color=cdefs.COLOR_CORP_WARNING, sound = "SpySociety_DLC001/HUD/gameplay/DLCrouter_firewall_increase",icon=nil} )
					sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = unit, kanim = "gui/hud_fx", symbol = "wireless_console_takeover", anim="idle", above=true, params={} } )
					sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = userUnit, kanim = "gui/hud_fx", symbol = "wireless_console_takeover", anim="idle", above=true, params={} } )
				end
			end
		end,
	},
	
	ce_randomizer_passive  = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.RANDOMIZER.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.RANDOMIZER.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN and evData:isPC() and userUnit:getBrain() and userUnit:getBrain():getSituation().ClassType == simdefs.SITUATION_IDLE then
				local idle = sim:getNPC():getIdleSituation()
				if sim:nextRand(1,3)==1 then
					if sim:nextRand(1,3)==1 then
						local x0, y0 = userUnit:getLocation()
						local closestAgent = simquery.findClosestUnit( sim:getPC():getUnits(), x0, y0, simquery.isAgent )
						idle:generatePatrolPath( userUnit, closestAgent:getLocation(), true )
					else
						idle:generatePatrolPath( userUnit, nil, nil, true )
					end
				end
			end
		end, 
	},
	ce_electricalInterference = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.INTERFERENCE.NAME, 
		getDesc = function(traits)
			return string.format(STRINGS.MOD_CE.RESEARCH.INTERFERENCE.UNIT_DESC,traits.electricalInterference,(traits.electricalInterference > 1 and"turns") or "turn",traits.electricalInterferenceRange)
		end,

		onSpawnAbility = function( self, sim, unit )
			unit:getTraits().interferedItems = {}
			sim:addTrigger( simdefs.TRG_OVERWATCH, self, unit )	
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )			
			sim:addTrigger( "agentGotItem", self, unit )				
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_OVERWATCH, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			sim:removeTrigger( "agentGotItem", self )
			for i, item in pairs(unit:getTraits().interferedItems) do
				item:getTraits().cooldownMax = item:getTraits().cooldownMax - userUnit:getTraits().electricalInterference
				item:getTraits().interference = item:getTraits().interference - userUnit:getTraits().electricalInterference
				if item:getTraits().cooldownMax == 0 then
					item:getTraits().cooldownMax = nil
				end
			end
			unit:getTraits().interferedItems = {}
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			local pcplayer = sim:getPC()
			for i, item in pairs(userUnit:getTraits().interferedItems) do
				item:getTraits().cooldownMax = item:getTraits().cooldownMax - userUnit:getTraits().electricalInterference
				item:getTraits().interference = item:getTraits().interference - userUnit:getTraits().electricalInterference
				if item:getTraits().cooldownMax == 0 then
					item:getTraits().cooldownMax = nil
				end
			end
			
			userUnit:getTraits().interferedItems = {}
			local x0, y0 = userUnit:getLocation()
			
			if isBrainy(userUnit) then
				for i, unit in pairs( pcplayer:getUnits() ) do
					local x1, y1 = unit:getLocation()
					if x1 and y1 and mathutil.distSqr2d(x0,y0,x1,y1) <= userUnit:getTraits().electricalInterferenceRange * userUnit:getTraits().electricalInterferenceRange then
						for i, item in pairs( unit:getChildren() ) do
							if not item:getTraits().disposable and not item:getTraits().augment and not item:getTraits().debugItem and #item:getAbilities() > 1 then
								table.insert(userUnit:getTraits().interferedItems,item)
								item:getTraits().cooldownMax = (item:getTraits().cooldownMax or 0) + userUnit:getTraits().electricalInterference
								item:getTraits().interference = (item:getTraits().interference or 0) + userUnit:getTraits().electricalInterference
							end
						end
					end
				end
			end
		end, 
	},
	ce_koBomb = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.KO_BOMB.NAME, 
		--buffDesc = STRINGS.MOD_CE.RESEARCH.KO_BOMB.UNIT_DESC,
		getDesc = function(traits)
			return string.format(STRINGS.MOD_CE.RESEARCH.KO_BOMB.UNIT_DESC,traits.koBomb or 2)
		end,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self, unit )			
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			if not unit:isDead() then
				sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
			end
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			local x0, y0
			if evType == simdefs.TRG_UNIT_KO and evData.ticks and evData.unit == userUnit then 
				x0, y0 = userUnit:getLocation()
			elseif evType == simdefs.TRG_UNIT_KILLED and evData.unit == userUnit then 
				x0, y0 = evData.corpse:getLocation()
				sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
			end
			
			if x0 then
				local range = userUnit:getTraits().koBomb or 2
				sim:emitSound( simdefs.SOUND_OVERLOAD, x0, y0, userUnit )
				sim:dispatchEvent( simdefs.EV_OVERLOAD_VIZ, {x = x0, y = y0, range = range } )
				local wasGroupKO = sim._daemonQueue ~= nil
				sim:dispatchEvent( simdefs.EV_KO_GROUP, true )
				local cells = simquery.rasterCircle( sim, x0, y0, range, 0)
				
				for i=1,#cells-1,2 do
					local x, y = cells[i], cells[i + 1]
					local raycastX, raycastY = sim:getLOS():raycast(x0, y0, x, y)
					if raycastX == x and raycastY == y then
						local cell = sim:getCell(x, y)
						for i, cellUnit in ipairs( cell.units ) do
							if simquery.isEnemyAgent(userUnit:getPlayerOwner(), cellUnit) and not cellUnit:isKO() then
								if cellUnit:getTraits().canKO then
									cellUnit:setKO( sim, userUnit:getTraits().koDamage or 2 )
								end
							end
						end
					end
				end
				userUnit:getTraits().koTimer = math.max(userUnit:getTraits().koTimer or 0,(userUnit:getTraits().koDamage or 2) + 1)
				if not wasGroupKO then
					sim:dispatchEvent( simdefs.EV_KO_GROUP, false )
				end
			end
		end, 
	},
	ce_aoe_shield = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.AOE_SHIELD.NAME, 
		getDesc = function(traits)
			return string.format(STRINGS.MOD_CE.RESEARCH.AOE_SHIELD.UNIT_DESC,traits.aoe_shield_range,traits.aoe_shield)
		end,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_ALERTED, self, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
			sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if (evType == simdefs.TRG_UNIT_NEWTARGET and evData.unit == userUnit) or (evType == simdefs.TRG_UNIT_ALERTED and evData.unit == userUnit) then
				if isOK(userUnit) and not userUnit:getTraits().aoe_shield_cooldown then
					local x0, y0 = userUnit:getLocation()
					userUnit:getTraits().aoe_shield_cooldown = true
					sim:dispatchEvent(simdefs.EV_PLAY_SOUND,"SpySociety/Actions/EMP_explo")
					sim:dispatchEvent("EV_CE_HILITE_RADIUS",{x=x0,y=y0,range=userUnit:getTraits().aoe_shield_range,cells = simquery.fillCircle( sim, x0, y0, userUnit:getTraits().aoe_shield_range, 0 ), color = { 1, 0, 0, 0.5 }})
					for i, unit in pairs(sim:getNPC():getUnits()) do
						local x1, y1 = unit:getLocation()
						if mathutil.distSqr2d(x0,y0,x1,y1) <= userUnit:getTraits().aoe_shield_range * userUnit:getTraits().aoe_shield_range then
							unit:getTraits().temporaryArmor = (unit:getTraits().temporaryArmor or 0) + userUnit:getTraits().aoe_shield * 2
							sim:dispatchEvent(simdefs.EV_UNIT_REFRESH,{unit = unit})
						end
					end
				end
			elseif evType == simdefs.TRG_START_TURN then
				userUnit:getTraits().aoe_shield_cooldown = nil
			end
		end
	},
	ce_alarm_drain = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.ALARM_DRAIN.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.ALARM_DRAIN.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self, unit )
			unit:addMPMax( 4 )
			unit:addMP( 4 )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ALARM_STATE_CHANGE then 
				local x0, y0 = userUnit:getLocation()
				userUnit:addMPMax( -1 )
				userUnit:addMP( -1 )
				sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = userUnit } )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )			
			end 
		end, 
	},
	ce_daemon_core = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.DAEMON_CORE.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.DAEMON_CORE.UNIT_DESC,
		icon = "gui/icons/item_icons/items_icon_small/icon-item_heart_monitor_small.png",

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
			if not unit:getTraits().usedDaemonCore then
				unit:getTraits().usedDaemonCore = true
				self.buffAbility = nil
				local programList = sim:getIcePrograms()
				sim:getNPC():addMainframeAbility( sim, programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))	 )
			end
		end,
		
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evData.unit == userUnit then
				if not userUnit:getTraits().usedDaemonCore then
					userUnit:getTraits().usedDaemonCore = true
					self.buffAbility = nil
					local programList = sim:getIcePrograms()
					sim:getNPC():addMainframeAbility( sim, programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))	 )
				end
			end
		end
	},
	ce_regenerative_nanocells_passive = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.REGENERATIVE_NANOCELLS.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.REGENERATIVE_NANOCELLS.UNIT_DESC,
		onSpawnAbility = function( self, sim, unit )
			unit:getTraits().regenerative_nanocells = true 
			unit.oldkillUnit = unit.killUnit
			unit.killUnit = function( self, sim )
				if self:getTraits().regenerative_nanocells then	
					self:setKO( self._sim, 2 )
					local x0, y0 = self:getLocation()
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.REGENERATIVE_NANOCELLS,x=x0,y=y0,color={r=245/255,g=245/255,b=245/255,a=1 }} )
					sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = self, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params={color ={{symbol="wall",r=1,g=0,b=0,a=1}} }} )
					self:getTraits().wounds = 0
				else
					self:oldkillUnit( sim )
				end
			end
			unit.onDamage = function( self, damage )
				self:getTraits().wounds = self:getTraits().wounds + damage

				if self:getTraits().wounds >= self:getTraits().woundsMax then
					if self:getTraits().regenerative_nanocells then	
						self:setKO( self._sim, 2 )
						local x0, y0 = self:getLocation()
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.REGENERATIVE_NANOCELLS,x=x0,y=y0,color={r=245/255,g=245/255,b=245/255,a=1 }} )
						sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = self, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params={color ={{symbol="wall",r=1,g=0,b=0,a=1}} }} )
						sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound = "SpySociety_DLC001/Actions/flackguard_armourincrease",x=x0,y=y0} )
						self:getTraits().wounds = 0
					else
						self:getTraits().dead = true
					end
				end

				if self:isDead() then
					if self:getTraits().canBeCritical then					
						self:setKO( self._sim, 3 )
						self._sim:dispatchEvent( simdefs.EV_SHOW_MODAL_REWIND )
						if self._sim:getTags().rewindsLeft > 0 and not self._sim:getPC():isNeutralized( self._sim ) then		
							if self._sim:getCurrentPlayer():isPC() then
								self._sim:dispatchEvent( simdefs.EV_BLINK_REWIND )
							else
								self._sim:getTags().blinkAtStartTurn = true
							end				
						end
					elseif self:getTraits().regenerative_nanocells then 
						self:setKO( self._sim, 1 )
					else 
						self:killUnit( self._sim )	
					end
				end
			end	
		end, 
	},
	
	ce_magnetic_rifle = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.MAGNETIC_RIFLE.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.MAGNETIC_RIFLE.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
			local ice = unit:getTraits().mainframe_ice or 0
			local gun = simquery.getEquippedGun( unit )
			
			if ice <= 0 then
				ice = 0
				unit:getTraits().genericPiercing = -99
				unit:getTraits().noAttack = true
				unit:setAiming(false)
				gun:getTraits().armorPiercing = nil
			else
				gun:getTraits().armorPiercing = ice
				unit:getTraits().noAttack = nil
				unit:getTraits().genericPiercing = nil
			end
			gun:getTraits().shots = math.floor(2*ice-ice*math.min(0.9,ice*0.05))
			gun:getTraits().baseDamage = ice
			gun:getTraits().damage = ice
			
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
		end,
		
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,
		
		onTrigger = function( self, sim, evType, evData, userUnit )
			local ice = userUnit:getTraits().mainframe_ice or 0
			local gun = simquery.getEquippedGun( userUnit ) or userUnit
			
			if ice <= 0 then
				ice = 0
				userUnit:getTraits().genericPiercing = -99
				userUnit:getTraits().noAttack = true
				userUnit:setAiming(false)
				gun:getTraits().armorPiercing = nil
			else
				gun:getTraits().armorPiercing = ice
				userUnit:getTraits().noAttack = nil
				userUnit:getTraits().genericPiercing = nil
			end
			gun:getTraits().shots = math.floor(2.5*ice-ice*math.min(0.9,ice*0.05)-0.5)
			gun:getTraits().baseDamage = ice
			gun:getTraits().damage = ice
			
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
		end,
	},
	
	ce_mainframe_refraction = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.AMBIENCE_REFRACTION.NAME,
		buffDesc = STRINGS.MOD_CE.RESEARCH.AMBIENCE_REFRACTION.UNIT_DESC,
		
		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self, unit )
			unit:getTraits().refracturedIce = unit:getTraits().refracturedIce or 0
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ICE_BROKEN and evData.delta and evData.delta < 0 and not userUnit:isKO() and not userUnit:isDead() then
				local delta = evData.delta
				if evData.unit and evData.unit:getTraits().mainframe_ice and evData.unit:getTraits().mainframe_ice < 0 then
					delta = delta - evData.unit:getTraits().mainframe_ice
				end
				delta = -delta
				if delta <= 0 then
					return
				end
				if evData.unit == userUnit then
					local units = {}
					local x0, y0 = userUnit:getLocation()
					for i, unit in pairs(sim:getAllUnits()) do
						if unit ~= userUnit and unit:getTraits().mainframe_item and unit:getTraits().mainframe_ice and not (unit:getTraits().noRaiseIfIceZero and unit:getTraits().mainframe_ice <= 0) then
							local x1, y1 = unit:getLocation()
							if mathutil.distSqr2d(x0,y0,x1,y1) <= userUnit:getTraits().refracture_from_range * userUnit:getTraits().refracture_from_range then
								table.insert(units,unit)
							end
						end
					end
					if #units == 0 then
						return
					end
					userUnit:getTraits().refracturedIce = userUnit:getTraits().refracturedIce + userUnit:getTraits().refracture_from * delta
					while userUnit:getTraits().refracturedIce >= 1 do
						local unit = units[sim:nextRand(#units)]
						unit:increaseIce(sim,1)
						userUnit:getTraits().refracturedIce = userUnit:getTraits().refracturedIce - 1
					end
				else
					local x0, y0 = userUnit:getLocation()
					local x1, y1 = evData.unit:getLocation()
					if mathutil.distSqr2d(x0,y0,x1,y1) <= userUnit:getTraits().refracture_to_range * userUnit:getTraits().refracture_to_range then
						userUnit:getTraits().refracturedIce = userUnit:getTraits().refracturedIce + userUnit:getTraits().refracture_to * delta
						while userUnit:getTraits().refracturedIce >= 1 do
							userUnit:increaseIce(sim,1)
							userUnit:getTraits().refracturedIce = userUnit:getTraits().refracturedIce - 1
						end
					end
				end
			end
		end, 
	},
	
	ce_long_range_sensors_passive  = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.LONG_RANGE_SENSORS.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.LONG_RANGE_SENSORS.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
			unit:getTraits().range = -2 + ((unit:getTraits().hardmode and 2) or 0)
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN and evData:isPC() then
				if userUnit:isDead() or userUnit:isKO() or not userUnit:getBrain() then
					userUnit:getTraits().range = -2 + ((userUnit:getTraits().hardmode and 2) or 0)
					return
				end
				userUnit:getTraits().range = userUnit:getTraits().range+2
				local x0, y0 = userUnit:getLocation()
				local closestAgent = simquery.findClosestUnit( sim:getPC():getUnits(), x0, y0, function(unit) return simquery.isAgent(unit) and not unit:isDown() end )
				if closestAgent then
					local x1, y1 = closestAgent:getLocation()
					local dist = math.sqrt(mathutil.distSqr2d( x0, y0, x1, y1 ))
					if dist <= userUnit:getTraits().range then
						sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/Engineer/wireless_emitter" )
						sim:dispatchEvent( simdefs.EV_SCANRING_VIS, {x=x0,y=y0,range=userUnit:getTraits().range} )
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.LONG_RANGE_SENSORS,x=x0,y=y0,color={r=204/255,g=204/255,b=204/255,a=1 }} )
						userUnit:getBrain():spawnInterest(x1, y1, simdefs.SENSE_SIGHT, simdefs.REASON_SCANNED, closestAgent)
						
						userUnit:getTraits().range = -2 + ((userUnit:getTraits().hardmode and 2) or 0)
					end
				end
			end
		end, 
	},
	ce_sprint_pads_silent_passive = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.SPRINT_PADS.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.SPRINT_PADS.UNIT_DESC, 
		onSpawnAbility = function( self, sim, unit )
			unit:addMPMax(3)
			unit:getTraits().dashSoundRange = 0
			unit:getTraits().always_patrol = true
			unit:getTraits().sneaking = true
		end,
	},

	ce_sprint_pads_silent_passive_2 = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.SPRINT_PADS_2.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.SPRINT_PADS_2.UNIT_DESC, 
		onSpawnAbility = function( self, sim, unit )
			unit:addMPMax(5)
			unit:getTraits().dashSoundRange = 0
			unit:getTraits().always_patrol = true
			unit:getTraits().sneaking = true
		end, 
	},
	
	ce_psi_support = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.PSI_SUPPORT.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.PSI_SUPPORT.UNIT_DESC,
		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWINTEREST, self, unit )
			unit:getTraits().psiSupport = true
		end, 
		
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
			sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self, unit )
			if unit:getTraits().psiLink then
				unit:getTraits().psiLink:getTraits().psiLink = nil
				if (unit:isDead()) then
					unit:getTraits().psiLink:setKO(sim,1)
				end
			end
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			--Linked unit
			local link = userUnit:getTraits().psiLink
			if evType == simdefs.TRG_START_TURN and isOK(userUnit) then
				if not isOK(link) then
					--Remove old link
					if link then
						link:getTraits().psiLink = nil
						userUnit:getTraits().psiLink = nil
						sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = link } )
						link = nil
					end
					
					--Find new potential link
					local aiPlayer = sim:getNPC()
					local guards = {}
					for i, guard in ipairs( aiPlayer:getUnits() ) do
						if isBrainy(guard) and not guard:getTraits().isDrone and not guard:getTraits().psiSupport and not guard:getTraits().psiLink then
							table.insert( guards, guard )
						end
					end
					
					--Update link
					if #guards > 0 then
						local rand = sim:nextRand( 1, #guards )
						local guard = guards[rand]
						guard:getTraits().psiLink = userUnit
						userUnit:getTraits().psiLink = guard
						link = guard
					end
				end
			elseif evType == simdefs.TRG_UNIT_KO then
				if evData.unit == userUnit then 
					--We were ko'd, alert our partner
					userUnit:getTraits().psiLink = nil
					if link then
						link:getTraits().psiLink = nil
						local brain = isBrainy(link)
						if brain and userUnit:getLocation() then
							local x1, y1 = userUnit:getLocation()
							link:setAlerted(true)
							brain:getSenses():addInterest(x1, y1, simdefs.SENSE_RADIO, "REASON_PSI", userUnit)
						end
					end
				elseif evData.unit == link and link then
					--Our partner was ko'd
					link:getTraits().psiLink = nil
					userUnit:getTraits().psiLink = nil
					local brain = isBrainy(userUnit)
					
					if brain and link:getLocation() then
						local x1, y1 = link:getLocation()
						userUnit:setAlerted(true)
						brain:getSenses():addInterest(x1, y1, simdefs.SENSE_RADIO, "REASON_PSI", link)
					end
				end
			elseif evType == simdefs.TRG_UNIT_KILLED then
				--Our partner died! (Us dying is handled in onDespawnAbility!)
				if link and evData.unit == link and isOK(userUnit) then
					userUnit:getTraits().psiLink = nil
					userUnit:setKO(sim,1)
				end
			elseif link then
				if evType == simdefs.TRG_UNIT_NEWTARGET then
					if evData.unit == userUnit then
						--We spotted an intruder
						local brain = isBrainy(link)
						if brain then
							local x1, y1 = evData.target:getLocation()
							link:setAlerted(true)
							brain:getSenses():addInterest(x1, y1, simdefs.SENSE_RADIO, "REASON_PSI", userUnit)
						end
					elseif evData.unit == link then
						--Our partner spotted an intruder
						local brain = isBrainy(userUnit)
						if brain then
							local x1, y1 = evData.target:getLocation()
							userUnit:setAlerted(true)
							brain:getSenses():addInterest(x1, y1, simdefs.SENSE_RADIO, "REASON_PSI", link)
						end
					end
				elseif evType == simdefs.TRG_UNIT_NEWINTEREST and evData.interest.reason ~= "REASON_PSI" and evData.interest.alerts and evData.interest.reason ~= simdefs.REASON_HUNTING then
					if evData.unit == link then
						--Our partner noticed something alerting
						local brain = isBrainy(userUnit)
						if brain then
							userUnit:setAlerted(true)
							brain:getSenses():addInterest(evData.interest.x, evData.interest.y, simdefs.SENSE_RADIO, "REASON_PSI", evData.interest.sourceUnit)
						end
					elseif evData.unit == userUnit then
						--We noticed something alerting
						local brain = isBrainy(link)
						if brain then
							link:setAlerted(true)
							brain:getSenses():addInterest(evData.interest.x, evData.interest.y, simdefs.SENSE_RADIO, "REASON_PSI", evData.interest.sourceUnit)
						end
					end
				end
			end
			
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
			if link then
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = link } )
			end
		end,
	},
	
	ce_neurostim_passive = util.extend( DEFAULT_BUFF ){
		name = STRINGS.MOD_CE.RESEARCH.NEUROSTIM.NAME, 
		--getDesc = function(traits) return string.format(STRINGS.MOD_CE.RESEARCH.NEUROSTIM.UNIT_DESC)end,
		buffDesc = STRINGS.MOD_CE.RESEARCH.NEUROSTIM.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_UNIT_ALERTED, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			local x0, y0 = userUnit:getLocation()
			local boost
			userUnit:getTraits().neurostimFalloff = (userUnit:getTraits().neurostimFalloff or 1)
			if evType == simdefs.TRG_UNIT_ALERTED and evData.unit == userUnit then 
				boost = 1.8
			elseif evType == simdefs.TRG_UNIT_KO and evData.unit == userUnit and evData.ticks then 
				boost = 0.8
				boost = boost / userUnit:getTraits().neurostimFalloff
				userUnit:getTraits().neurostimFalloff = userUnit:getTraits().neurostimFalloff + boost
			elseif evType == simdefs.TRG_UNIT_NEWTARGET and evData.unit == userUnit then 
				boost = 1.2
				boost = boost / userUnit:getTraits().neurostimFalloff
				userUnit:getTraits().neurostimFalloff = userUnit:getTraits().neurostimFalloff + boost
			end
			if boost and boost > 0 then
				userUnit:addMP(boost * (userUnit:getTraits().neurostim or 1))
				userUnit:addMPMax(boost * (userUnit:getTraits().neurostim or 1))
				--sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = userUnit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params={color ={{symbol="wall",r=1,g=1,b=0,a=1}} }} )
				sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = userUnit } )
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.NEUROSTIM,x=x0,y=y0,color={r=255/255,g=255/255,b=51/255,a=1 }} )
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound = "SpySociety/Actions/heal",x=x0,y=y0} )
			end
		end,
	},
	
	ce_spawnDrone = 
	{
		canUseAbility = function( self, sim, unit )
			return false
		end,
		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			if not sim.petTiers then
				sim.petsToAdd = -1
				sim.petTiers = {{},{},{}}
				local OMNI_HUMAN = util.tdupe(worldgen.OMNI_GUARD_FIX)
				table.insert(OMNI_HUMAN,{"ce_sankaku_engineer_LV2",400})
				local tiers = {worldgen.SANKAKU.GUARD_FIX,worldgen.SANKAKU.HUMAN_GUARD_FIX,worldgen.SANKAKU.THREAT_FIX,worldgen.SANKAKU.HUMAN_THREAT_FIX,worldgen.SANKAKU.LVL2,OMNI_HUMAN}
				for i = 1, #tiers, 2 do
					for j, v in pairs(tiers[i]) do
						local ok = true
						for k, w in pairs(tiers[i+1]) do
							if v[1] == w[1] then
								ok = false
								break
							end
						end
						if ok then
							table.insert(sim.petTiers[(i+1)/2],v[1])
						end
					end
				end
			end
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self, unit )
		end,
		
		canPutDrone = function( self, sim, cell )
			if not cell then
				return false
			end
			
			if not simquery.canPath( sim, nil, nil, cell ) then
				return false
			end
			if simquery.cellHasTag(sim, cell, "noguard") or simquery.cellHasTag(sim, cell, "entry") then
				return false
			end
			
			if cell.procgenRoom.tags["noguard"] or cell.procgenRoom.tags["entry"] then
				return false
			end
			
			return true
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evData.unit == userUnit then
				if not sim.petDrone then
					local drone
					if sim.pettier and sim.pettier ~= 0 then
						drone = simfactory.createUnit( unitdefs.lookupTemplate( sim.petTiers[sim.pettier][sim:nextRand(#sim.petTiers[sim.pettier])] ), sim )
					else
						drone = simfactory.createUnit( unitdefs.lookupTemplate( "camera_drone" ), sim )
						
						--drone = simfactory.createUnit( unitdefs.lookupTemplate( worldgen.CAMERA_DRONE_FIX[sim:nextRand(#worldgen.CAMERA_DRONE_FIX)][2] ), sim )
						
						sim.pettier = 0
					end
					drone:setPlayerOwner( sim:getNPC() )	
					drone:setPather(sim:getNPC().pather)
					sim:spawnUnit( drone )
					sim:getNPC():returnToIdleSituation( drone )
					sim.petDrone = drone
				end
			
				if not userUnit:getTraits().hasSpawnedDrone and sim.petDrone:isValid() and userUnit:getLocation() and sim.pettier < #sim.petTiers then
					userUnit:getTraits().hasSpawnedDrone = true
					local petsToAdd = sim.petsToAdd + (userUnit:getTraits().pettier or 1)
					sim.petsToAdd = petsToAdd
					for i = 1, math.ceil(petsToAdd) do
						if sim.pettier < #sim.petTiers then
							sim.pettier = sim.pettier + 1
							sim.petsToAdd = sim.petsToAdd - 1
							
							local drone = simfactory.createUnit( unitdefs.lookupTemplate( sim.petTiers[sim.pettier][sim:nextRand(#sim.petTiers[sim.pettier])] ), sim )
							drone:setPlayerOwner( sim:getNPC() )	
							drone:setPather(sim:getNPC().pather)
							sim:spawnUnit( drone )
							sim:getNPC():returnToIdleSituation( drone )
							
							local x,y = userUnit:getLocation()
							local possibleCells = {}
							for dx = -1,1 do
								for dy = -1,1 do
									local testCell = sim:getCell( x + dx, y + dy )
									if self:canPutDrone(sim,testCell) then
										table.insert(possibleCells,testCell)
									end
								end
							end
							if #possibleCells == 0 then
								if sim.petDrone:getLocation() then
									sim:warpUnit(drone,sim:getCell(sim.petDrone:getLocation()))
									if drone:getBrain():getSituation() == sim:getNPC():getIdleSituation() then
										sim:getNPC():getIdleSituation():generatePatrolPath( drone, drone:getLocation() )
									end
									sim:warpUnit(sim.petDrone,nil)
									sim.petDrone:destroyTab()
									sim:despawnUnit(sim.petDrone)
									sim.petDrone = drone
								else
									drone:destroyTab()
									sim:despawnUnit(drone)
									sim.petDrone:destroyTab()
									sim:despawnUnit(sim.petDrone)
									sim.petDrone = nil
									break
								end
							else
								sim:warpUnit(drone, possibleCells[sim:nextRand(1,#possibleCells)])
								if drone:getBrain():getSituation() == sim:getNPC():getIdleSituation() then
									sim:getNPC():getIdleSituation():generatePatrolPath( drone, drone:getLocation() )
								end
								local x,y = sim.petDrone:getLocation()
								if sim.petDrone:getLocation() then
									sim:warpUnit(sim.petDrone,nil)
								end
								local x,y = sim.petDrone:getLocation()
								sim.petDrone:destroyTab()
								sim:despawnUnit(sim.petDrone)
								sim.petDrone = drone
							end
						else
							break
						end
					end
				end
				
				if userUnit:getLocation() and sim.petDrone and sim.petDrone:isValid() and not sim.petDrone:getLocation() then
					local x,y = userUnit:getLocation()
					local possibleCells = {}
					for dx = -1,1 do
						for dy = -1,1 do
							local testCell = sim:getCell( x + dx, y + dy )
							if self:canPutDrone(sim,testCell) then
								table.insert(possibleCells,testCell)
							end
						end
					end
					if #possibleCells == 0 then
						sim.petDrone:destroyTab()
						sim:despawnUnit(sim.petDrone)
						sim.petDrone = nil
					else
						sim:warpUnit(sim.petDrone, possibleCells[sim:nextRand(1,#possibleCells)])
						if sim.petDrone:getBrain():getSituation() == sim:getNPC():getIdleSituation() then
							sim:getNPC():getIdleSituation():generatePatrolPath( sim.petDrone, sim.petDrone:getLocation() )
						end
					end
				end
			end
		end,
	},
	ce_droneMonitor = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.DRONE_MONITOR.NAME, 
		getDesc = function(traits) return string.format(STRINGS.MOD_CE.RESEARCH.DRONE_MONITOR.UNIT_DESC,traits.droneMonitor)end,
	
		
		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_APPEARED, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			sim:addTrigger( "TRG_UNIT_HACKED", self, unit )
			unit:getTraits().spottedWrecks = {}
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_APPEARED, self )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )
			sim:removeTrigger( "TRG_UNIT_HACKED", self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			local brain = isBrainy(userUnit)
			if brain then
				local x0, y0 = userUnit:getLocation()
				
				local sense
				local maxdist = math.huge
				local fx = false
				local bestTarget
				local prevTarget
				local prevx, prevy
				local interest = brain:getInterest()
				if interest and interest.reason == simdefs.REASON_FOUNDDRONE then
					prevTarget = interest.sourceUnit
					prevx, prevy = prevTarget:getLocation()
				end
				
				--Add EPSILON to range as comparison is not inclusive
				local rangeSqr = userUnit:getTraits().droneMonitor * userUnit:getTraits().droneMonitor + EPSILON
				
				--Already found a drone? If it's the closest then we should ignore the others
				--Also check that it has not gotten back up and whether it has moved (then we need to update by scanning)
				if prevTarget and (prevTarget:isDown() or prevTarget:getTraits().takenDrone) and evData.unit ~= prevTarget and not (evType == simdefs.TRG_START_TURN and prevx and (prevx ~= interest.x or prevy ~= interest.y) and mathutil.distSqr2d( x0, y0, prevx, prevy ) < rangeSqr) then
					maxdist = mathutil.distSqr2d( x0, y0, interest.x, interest.y )
				end
				
				if evType == simdefs.TRG_START_TURN and evData:isPC() then
					--Scan for drones in a radius
					maxdist = math.min(rangeSqr,maxdist)
					for i, unit in pairs(sim:getAllUnits()) do
						local x1, y1 = unit:getLocation()
						if x1 then
							local dist = mathutil.distSqr2d( x0, y0, x1, y1 )
							if dist < maxdist and not array.find( userUnit:getSeenUnits(), unit ) then
								if (unit:getTraits().wasDrone and not userUnit:getTraits().spottedWrecks[unit]) or (unit:getTraits().isDrone and (unit:isKO() or unit:getTraits().takenDrone)) then
									sense = simdefs.SENSE_RADIO
									bestTarget = unit
									maxdist = dist
									fx = true
								end
							end
						end
					end
				elseif (evType == simdefs.TRG_UNIT_APPEARED and evData.seerID == userUnit:getID()) or ((evType == simdefs.TRG_UNIT_WARP or evType == "TRG_UNIT_HACKED") and array.find( userUnit:getSeenUnits(), evData.unit )) then
					--We need to update if A: drone just appeared, B: was hacked while in sight, or C: already hacked then moved within sight.
					if evData.unit:getTraits().isDrone and evData.unit:getTraits().takenDrone then
						local x1, y1 = evData.unit:getLocation()
						local dist = mathutil.distSqr2d( x0, y0, x1, y1 )
						if dist < maxdist then
							sense = simdefs.SENSE_SIGHT
							bestTarget = evData.unit
							maxdist = dist
							fx = false
						end
					elseif evData.unit:getTraits().wasDrone then
						userUnit:getTraits().spottedWrecks[evData.unit] = true
					end
				end
				
				if bestTarget then
					local x,y = bestTarget:getLocation()
					local interrupt = false
					if not interest or interest.x ~= x or interest.y ~= y then
						--Check this before adding interest since that may just modify our existing interest
						interrupt = true
					end
					brain:getSenses():addInterest(x, y, sense, simdefs.REASON_FOUNDDRONE, bestTarget)
					if interrupt then
						--Either a new or moved interest, worth interrupting for (the drone can leave and reenter sight while we're moving so we can't just always interrupt)
						userUnit:interruptMove( sim )
						
						if userUnit:getPather() then
							userUnit:getPather():invalidatePath(userUnit)
						end
						
						brain:reset()
						if evType ~= simdefs.TRG_UNIT_WARP then
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt= STRINGS.MOD_CE.UI.DRONE_MONITOR,x=x,y=y,color={r=190/255,g=255/255,b=200/255,a=1 }} )
						end
					end
					sim:processReactions()
					if fx then
						sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/Engineer/wireless_emitter" )
						sim:dispatchEvent( simdefs.EV_SCANRING_VIS, {x=x0,y=y0,range=userUnit:getTraits().droneMonitor} )
					end
				end
			end
		end,
	},
	ce_drone_boost = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.DRONE_BOOST.NAME, 
		getDesc = function(traits) return string.format(STRINGS.MOD_CE.RESEARCH.DRONE_BOOST.UNIT_DESC,traits.drone_boost_range,traits.drone_boost) end, 

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )	
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KILLED, self, unit )
			unit:getTraits().buffedDrones = {}
		end,

		onDespawnAbility = function( self, sim, unit )
			for i, unt in pairs(unit:getTraits().buffedDrones) do
				unt:addMPMax( -unit:getTraits().drone_boost )
				if sim:getCurrentPlayer():isPC() then
					unt:getTraits().mp = math.min(unt:getTraits().mp,unt:getTraits().mpMax)
				end
			end
			unit:getTraits().buffedDrones = {}
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self )	
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KILLED, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			for id, unit in pairs(userUnit:getTraits().buffedDrones) do
				unit:addMPMax( -userUnit:getTraits().drone_boost )
			end
			userUnit:getTraits().buffedDrones = {}
			
			if isBrainy(userUnit) then 				
				local playSound = false
				for i, unit in pairs(sim:getAllUnits()) do
					local x0, y0 = unit:getLocation()
					local rangeSqr = userUnit:getTraits().drone_boost_range * userUnit:getTraits().drone_boost_range
					if unit:isValid() and x0 and y0 and unit:getTraits().isDrone and unit:getPlayerOwner() == sim:getNPC() and mathutil.distSqr2d( x0, y0,userUnit:getLocation() ) <= rangeSqr then
						table.insert(userUnit:getTraits().buffedDrones,unit)
						unit:addMPMax( userUnit:getTraits().drone_boost )
						if evType == simdefs.TRG_START_TURN and evData:isNPC() and not unit:isDown() then
							playSound = true
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt= STRINGS.MOD_CE.UI.DRONE_BOOST,x=x0,y=y0,color={r=1,g=1,b=1,a=1}} ) 
							local params = {color ={{symbol="wall",r=0,g=1,b=0,a=1}} }
							sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = unit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params=params} )
						end
					end
				end
				local x0, y0 = userUnit:getLocation()
				if playSound then
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound = "SpySociety_DLC001/Actions/flackguard_armourincrease",x=x0,y=y0} )
				end
			end
			if sim:getCurrentPlayer():isPC() then
				for i, unit in pairs(sim:getAllUnits()) do
					if unit:getTraits().isDrone and unit:getPlayerOwner() == sim:getNPC() then
						unit:getTraits().mp = math.min(unit:getTraits().mp,unit:getTraits().mpMax)
					end
				end
			end
		end, 
	},
	ce_ultraviolet_spectrometer_passive = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.ULTRAVIOLET_SPECTROMETER.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.ULTRAVIOLET_SPECTROMETER.UNIT_DESC, 
		onSpawnAbility = function( self, sim, unit )
			unit:getTraits().detect_cloak = true 
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_USEDOOR, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_USEDOOR, self, unit )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_UNIT_WARP or evType == simdefs.TRG_UNIT_USEDOOR then
				if isBrainy( userUnit ) then
					if evData.unit and (evData.unit == userUnit or evData.unit:isPC()) then
						for i, unit in pairs(sim:getPC():getAgents()) do
							if unit:getTraits().disguiseOn then
								local x1,y1 = unit:getLocation()
								local canSee, canSense = sim:canUnitSeeUnit(userUnit,unit)
								unit:getTraits().hidesInCover = true
								if canSee or canSense then 
									unit:setDisguise(false)	
									unit:interruptMove( sim )
								end
								unit:getTraits().hidesInCover = not unit:getTraits().disguiseOn
							end
						end
					end
				end
			end
		end, 
	},
	ce_search_tools = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.SEARCH_TOOLS.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.SEARCH_TOOLS.UNIT_DESC, 
		onSpawnAbility = function( self, sim, unit )
			unit:getTraits().detect_cloak = true 
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_USEDOOR, self, unit )
			unit:getTraits().seesHidden = true
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_USEDOOR, self, unit )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_UNIT_WARP or evType == simdefs.TRG_UNIT_USEDOOR then
				if isBrainy( userUnit ) then
					if evData.unit and (evData.unit == userUnit or evData.unit:isPC()) then
						for i, unit in pairs(sim:getPC():getAgents()) do
							if unit:getTraits().disguiseOn then
								local x1,y1 = unit:getLocation()
								local canSee, canSense = sim:canUnitSeeUnit(userUnit,unit)
								unit:getTraits().hidesInCover = true
								if canSee or canSense then 
									unit:setDisguise(false)	
									unit:interruptMove( sim )
								end
								unit:getTraits().hidesInCover = not unit:getTraits().disguiseOn
							end
						end
					end
				end
			end
		end, 
	},
	
	ce_ultrasonic_echolocation_passive = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.ULTRASONIC_ECHOLOCATION.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.ULTRASONIC_ECHOLOCATION.UNIT_DESC, 
		onSpawnAbility = function( self, sim, unit )
			unit:getTraits().ultrasonic_echolocation = true
			sim:addTrigger( simdefs.TRG_UNIT_WARP, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_WARP, self, unit )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_UNIT_WARP and simquery.isEnemyAgent(userUnit:getPlayerOwner(), evData.unit) and userUnit:getBrain() then
				local to_cell = evData.to_cell or evData.from_cell
				userUnit:getTraits().seesHidden = true
				local canSee, canSense = sim:canUnitSeeUnit(userUnit, evData.unit)
				local targetX, targetY

				if not canSee and canSense then
					targetX, targetY = to_cell.x, to_cell.y
				elseif not canSee and not canSense and evData.from_cell then
						-- Did unit warp OUT of peripheral vision? (NOTE: warping to nil still counts as leaving peripheral)
						-- NOTE: we really want to query here whether evData.unit *was* visible in peripheral (at evData.from_cell),
						-- based on invisibility, cover, etc. but there exists no such query, since evData.unit has already moved
						-- to a new cell by this time.  The closest we can do is call simquery.couldUnitSee, but it will base cover
						-- visibility on evData.unit's current cell -- leaving this as a small quirk, since the complete fix is quite complicated.
					if simquery.couldUnitSee(sim, userUnit, evData.unit, false, evData.from_cell ) then
						canSee, canSense = sim:canUnitSee(userUnit, evData.from_cell.x, evData.from_cell.y)
						if not canSee and canSense then
							targetX, targetY = evData.from_cell.x, evData.from_cell.y
						end
					end
				end
				userUnit:getTraits().seesHidden = false

				if targetX and targetY then 
				-- Unit either IS, or WAS in peripheral vision.  Either is noticeable.
					if userUnit:getBrain():getSenses():hasLostTarget(evData.unit) then
						userUnit:turnToFace( targetX, targetY , STRINGS.UI.TRACKED )
					else
						if evData.unit:isKO() or evData.unit:isDead() then
							if not evData.unit:hasBeenInvestigated() then
								userUnit:getBrain():getSenses():addInterest( targetX, targetY, simdefs.SENSE_PERIPHERAL, simdefs.REASON_FOUNDCORPSE, evData.unit)
							end
						else
							userUnit:getBrain():getSenses():addInterest( targetX, targetY, simdefs.SENSE_PERIPHERAL, simdefs.REASON_SENSEDTARGET, evData.unit)
						end
					end
				end		
			end
		end, 
	},
	
	ce_overtuned_reflexes_passive = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.OVERTUNED_REFLEXES.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.OVERTUNED_REFLEXES.UNIT_DESC, 
		onSpawnAbility = function( self, sim, unit )
			unit:getTraits().skipOverwatch = true 
			unit:getTraits().removeAttackAfterUse = true
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self, unit )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if (userUnit:getTraits().hasSkippedAttack or 0) > 0 then
				userUnit:getTraits().hasSkippedAttack = userUnit:getTraits().hasSkippedAttack - 1
				if (userUnit:getTraits().hasSkippedAttack <= 0) then
					userUnit:getTraits().hasSkippedAttack = nil
					userUnit:getTraits().skipOverwatch = true
					userUnit:getTraits().noAttack = nil
				end
			end
		end
	},
	ce_panicbutton = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.PANICBUTTON.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.PANICBUTTON.UNIT_DESC,
		onSpawnAbility = function( self, sim, unit )
			if sim:getParams().difficultyOptions.ce_sysadmin_response == 0 then
				array.removeElement( unit._abilities, self )
				return
			end
		
			if sim:getParams().difficultyOptions.ce_sysadmin_response_type == "panicElevator" then
				sim:addTrigger( "vip_escaped", self, unit )
				self.name = STRINGS.MOD_CE.RESEARCH.ELEVATORBUTTON.NAME
				self.buffDesc = string.format(STRINGS.MOD_CE.RESEARCH.ELEVATORBUTTON.UNIT_DESC,sim:getParams().difficultyOptions.ce_sysadmin_response or 2)
				self.alertText = STRINGS.MOD_CE.RESEARCH.ELEVATORBUTTON.ALERT
			else
				sim:addTrigger( simdefs.TRG_UNIT_ALERTED, self, unit )
				self.name = STRINGS.MOD_CE.RESEARCH.PANICBUTTON.NAME
				self.buffDesc = string.format(STRINGS.MOD_CE.RESEARCH.PANICBUTTON.UNIT_DESC,sim:getParams().difficultyOptions.ce_sysadmin_response or 2)
				self.alertText = STRINGS.MOD_CE.RESEARCH.PANICBUTTON.ALERT
			end
		end,

		onDespawnAbility = function( self, sim, unit )
			if sim:getParams().difficultyOptions.ce_sysadmin_response_type == "panicElevator" then
				sim:removeTrigger( "vip_escaped", self )
			else
				sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self )
			end
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evData.unit == userUnit and not (userUnit:isKO() or userUnit:isDead() or userUnit:getTraits().hasUsedPanicbutton or sim:getEnforcerWavesSpawned() > 0) then
				userUnit:getTraits().hasUsedPanicbutton = true
				local x0, y0 = userUnit:getLocation()
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=util.sformat( STRINGS.UI.ALARM_ADD, sim:getParams().difficultyOptions.ce_sysadmin_response or 2 ),x=x0,y=y0,color={r=255/255,g=10/255,b=10/255,a=1}} )
				sim:trackerAdvance( sim:getParams().difficultyOptions.ce_sysadmin_response or 2, self.alertText )
			end
		end, 
	},
	ce_piercing_scope_passive_new = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.PIERCING_SCOPE_NEW.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.PIERCING_SCOPE_NEW.UNIT_DESC,
		onSpawnAbility = function( self, sim, unit )
			unit:getTraits().seesHidden = true 
		end,
		
		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_SET_OVERWATCH, self, unit )
			sim:addTrigger( simdefs.TRG_OVERWATCH, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
			unit:getTraits().noReactionShots = true
		end, 
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_SET_OVERWATCH, self )
			sim:removeTrigger( simdefs.TRG_OVERWATCH, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
		end, 
		onTrigger = function( self, sim, evType, evData, userUnit )
			local aiming = userUnit:isValid() and userUnit:isAiming() and not userUnit:isDown()
			if aiming ~= userUnit:getTraits().seesHidden then
				userUnit:getTraits().seesHidden = aiming
				userUnit:getTraits().ignoresWalls = aiming
				
				if not userUnit:getTraits().refreshingLOS then
					sim:refreshUnitLOS( userUnit )
				end
			end
		end,
	},
	ce_piercing_scope_passive = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.PIERCING_SCOPE.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.PIERCING_SCOPE.UNIT_DESC,
		onSpawnAbility = function( self, sim, unit )
			unit:getTraits().seesHidden = true 
		end, 
	},
	ce_lockdown_scope_passive = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.LOCKDOWN_SCOPE.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.LOCKDOWN_SCOPE.UNIT_DESC,
		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_SET_OVERWATCH, self, unit )
			sim:addTrigger( simdefs.TRG_OVERWATCH, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_APPEARED, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_DISAPPEARED, self, unit )
			unit:getTraits().lockedDownUnits = {}
		end, 
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_SET_OVERWATCH, self )
			sim:removeTrigger( simdefs.TRG_OVERWATCH, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			sim:removeTrigger( simdefs.TRG_UNIT_APPEARED, self )
			sim:removeTrigger( simdefs.TRG_UNIT_DISAPPEARED, self )
			for unit in pairs(unit:getTraits().lockedDownUnits) do
				unit:getTraits().lockDownCount = unit:getTraits().lockDownCount - 1
			end
		end, 
		onTrigger = function( self, sim, evType, evData, userUnit )
			if userUnit:isValid() and userUnit:isAiming() and not userUnit:isDown() then
				if not userUnit:getTraits().usedToAim then
					userUnit:getTraits().usedToAim = true
					--userUnit:getTraits()._LOSperipheralRange = userUnit:getTraits().LOSperipheralRange
					--userUnit:getTraits().LOSperipheralRange = 0
				end
				local pcplayer = sim:getPC()
				for i, unit in pairs( pcplayer:getUnits() ) do 
					if sim:canUnitSeeUnit(userUnit,unit) then
						if not userUnit:getTraits().lockedDownUnits[unit] then
							local x0, y0 = unit:getLocation()
							unit:getTraits().lockDownCount = (unit:getTraits().lockDownCount or 0) + 1
							local r,g,b,a = userUnit:getTraits().lockdown_color.r, userUnit:getTraits().lockdown_color.g, userUnit:getTraits().lockdown_color.b, userUnit:getTraits().lockdown_color.a
							sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.LOCKDOWN,x=x0,y=y0,color={r=r,g=g,b=b,a=a}} )
							sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = unit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params={color ={{symbol="wall",r=r,g=g,b=b,a=a},{symbol="outline_side",r=r,g=g,b=b,a=a} }}} )
						end
						userUnit:getTraits().lockedDownUnits[unit] = true
					else
						if userUnit:getTraits().lockedDownUnits[unit] then
							userUnit:getTraits().lockedDownUnits[unit] = nil
							unit:getTraits().lockDownCount = unit:getTraits().lockDownCount - 1
						end
					end
				end
			else
				if userUnit:getTraits().usedToAim then
					--userUnit:getTraits().LOSperipheralRange = userUnit:getTraits().LOSperipheralRange + userUnit:getTraits()._LOSperipheralRange
					userUnit:getTraits().usedToAim = nil
					--userUnit:getTraits()._LOSperipheralRange = nil
				end
				for unit in pairs(userUnit:getTraits().lockedDownUnits) do
					unit:getTraits().lockDownCount = unit:getTraits().lockDownCount - 1
				end
				userUnit:getTraits().lockedDownUnits = {}
			end
			--sim:refreshUnitLOS( userUnit )
			sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
		end,
	},
	
	ce_holographic_pinpointing = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.HOLO_PINPOINT.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.HOLO_PINPOINT.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWINTEREST, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
		end,
		
		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWINTEREST, self )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
			if unit:getTraits().hasHoloTarget then
				local unit = unit:getTraits().hasHoloTarget
				unit:getTraits().holoTarget = unit:getTraits().holoTarget - 1
				if unit:getTraits().holoTarget == 0 then
					unit:getTraits().holoTarget = nil
				end
			end
		end,
		
		onTrigger = function( self, sim, evType, evData, userUnit )
			local oldHoloTarget = userUnit:getTraits().hasHoloTarget
			if evType == simdefs.TRG_START_TURN and evData:isPC() and isOK(oldHoloTarget) and not sim:canUnitSeeUnit(userUnit,oldHoloTarget) then
				--At the start of each turn notify us about our holo target
				local brain = isBrainy(userUnit)
				if brain then
					local x,y = oldHoloTarget:getLocation()
					local x0,y0 = userUnit:getLocation()
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.HOLOTARGET,x=x,y=y,color={r=101/255,g=232/255,b=248/255,a=1}} )
					sim:dispatchEvent( simdefs.EV_UNIT_WIRELESS_SCAN, { unitID = oldHoloTarget:getID() } )	
					brain:getSenses():addInterest( x, y, simdefs.SENSE_SIGHT, simdefs.REASON_SCANNED, oldHoloTarget, true)
				end
				
				--Remove holoTarget proprty from agent
				oldHoloTarget:getTraits().holoTarget = oldHoloTarget:getTraits().holoTarget - 1
				if oldHoloTarget:getTraits().holoTarget == 0 then
					oldHoloTarget:getTraits().holoTarget = nil
				end
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
				userUnit:getTraits().hasHoloTarget = nil
			elseif (evType == simdefs.TRG_UNIT_NEWINTEREST and evData.interest.sense == simdefs.SENSE_PERIPHERAL and evData.reason ~= simdefs.REASON_DOOR and evData.unit == userUnit)
				or (evType == simdefs.TRG_UNIT_NEWTARGET and evData.unit == userUnit) then
				--Agent entered our vision, mark them
				local unit = evData.target or evData.interest.sourceUnit
				if isOK(unit) and not userUnit:getTraits().hasHoloTarget then
					userUnit:getTraits().hasHoloTarget = unit
					local x0,y0 = unit:getLocation()
					unit:getTraits().holoTarget = (unit:getTraits().holoTarget or 0) + 1
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_wisp_reveal" )
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.HOLOTARGET,x=x0,y=y0,color={r=101/255,g=232/255,b=248/255,a=1}} )
					sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = unit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params={color ={{symbol="wall",r=101/255,g=232/255,b=248/255,a=1},{symbol="outline_side",r=101/255,g=232/255,b=248/255,a=0.75} }}} )
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = unit } )
				end
			end
		end
	},
	
	ce_patrol_generator = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.ALARM_PATROLS.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.ALARM_PATROLS.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self, unit )	
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ALARM_STATE_CHANGE then
				if userUnit:getBrain() and userUnit:getBrain():getSituation() == sim:getNPC():getIdleSituation() then
					userUnit:getBrain():getSituation():generatePatrolPath( userUnit, nil, nil, true )
					
					if userUnit:getTraits().patrolPath then
						local firstPoint = userUnit:getTraits().patrolPath[1]
						userUnit:getBrain():getSenses():addInterest(firstPoint.x, firstPoint.y, simdefs.SENSE_RADIO, simdefs.REASON_PATROLCHANGED, userUnit)
					end
			
					sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )	
				end
			end 
		end
	},
	
	ce_reverseIonField = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.REVERSE_IONFIELD.NAME, 
		getDesc = function(traits)
			return string.format(STRINGS.MOD_CE.RESEARCH.REVERSE_IONFIELD.UNIT_DESC,traits.reverseIonField)
		end,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_OVERWATCH, self, unit )	
			sim:addTrigger( simdefs.TRG_ICE_BROKEN, self, unit )				
			sim:addTrigger( simdefs.TRG_UNIT_KO, self, unit )		
			unit:getTraits().ionProtectedUnits = {}
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_OVERWATCH, self )
			sim:removeTrigger( simdefs.TRG_ICE_BROKEN, self )
			sim:removeTrigger( simdefs.TRG_UNIT_KO, self )
			for i, unit in pairs(unit:getTraits().ionProtectedUnits) do
				unit:getTraits().ionLayers = unit:getTraits().ionLayers - 1
				if unit:getTraits().ionLayers <= 0 then
					unit:getTraits().ionLayers = nil
					unit:getTraits().magnetic_reinforcement = unit:getTraits().magnetic_reinforcement_default
				end
			end
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			for i, unit in pairs(userUnit:getTraits().ionProtectedUnits) do
				unit:getTraits().ionLayers = unit:getTraits().ionLayers - 1
				if unit:getTraits().ionLayers <= 0 then
					unit:getTraits().ionLayers = nil
					unit:getTraits().magnetic_reinforcement = unit:getTraits().magnetic_reinforcement_default
				end
			end
			userUnit:getTraits().ionProtectedUnits = {}
			
			local brain = isBrainy(userUnit)
			if brain then
				local x0, y0 = userUnit:getLocation()
				for i, unit in pairs(sim:getAllUnits()) do
					local x1, y1 = unit:getLocation()
					if unit ~= userUnit and x1 and y1 and unit:getTraits().mainframe_iceMax and mathutil.distSqr2d(x0,y0,x1,y1) <= userUnit:getTraits().reverseIonField * userUnit:getTraits().reverseIonField then
						if unit:getTraits().magnetic_reinforcement_default == nil then
							unit:getTraits().magnetic_reinforcement_default = (unit:getTraits().magnetic_reinforcement ~= nil and unit:getTraits().magnetic_reinforcement ~= false)
						end
						unit:getTraits().magnetic_reinforcement = true
						unit:getTraits().ionLayers = (unit:getTraits().ionLayers or 0) + 1
						table.insert(userUnit:getTraits().ionProtectedUnits, unit)
					end
				end
			end 
		end, 
	},
	
	ce_drone_communications = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.DRONE_COMM.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.DRONE_COMM.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
		end,
		
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_UNIT_NEWTARGET then
				local x1, y1 = evData.target:getLocation()
				if evData.unit:getTraits().isDrone and userUnit:isValid() and userUnit:getPlayerOwner():isNPC() and not userUnit:isDown() then
					if evData.unit == userUnit then
						for i, unit in pairs(sim:getNPC():getUnits()) do
							if unit:getTraits().isDrone and unit ~= userUnit and unit:isValid() and unit:getPlayerOwner():isNPC() and not unit:isDown() then
								unit:getBrain():getSenses():addInterest(x1, y1, simdefs.SENSE_RADIO, "REASON_DRONE_COMM", evData.target)
							end
						end
					else
						userUnit:getBrain():getSenses():addInterest(x1, y1, simdefs.SENSE_RADIO, "REASON_DRONE_COMM", evData.target)
					end
				end
			end
		end,
	},
	
	ce_mp_pulse = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.MP_SIPHON.NAME, 
		getDesc = function(traits)
			return string.format(STRINGS.MOD_CE.RESEARCH.MP_SIPHON.UNIT_DESC,traits.mp_siphon,traits.mp_pulse)
		end,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_START_TURN, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_START_TURN, self )
		end,
		
		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_START_TURN and evData:isNPC() and userUnit:getLocation() and userUnit:getPlayerOwner():isNPC() and not userUnit:isDown() then
				local x,y = userUnit:getLocation()
				local currentCell = sim:getCell(x,y)
				local cells = {currentCell}
				local fillCells = simquery.rasterCircle(sim,x,y,userUnit:getTraits().mp_pulse,0)
				
				
				for i=1,#fillCells-1,2 do
					local cellX, cellY = fillCells[i], fillCells[i + 1]
					if x ~= cellX and y ~= cellY then
						local raycastX, raycastY = sim:getLOS():raycast(x, y, cellX, cellY)
						if raycastX == cellX and raycastY == cellY then
							table.insert(cells, sim:getCell(cellX, cellY))
						end
					end
				end
				sim:dispatchEvent("EV_CE_HILITE_RADIUS",{x=x,y=y,range=userUnit:getTraits().mp_pulse,cells = cells, color = { 0.5, 1, 1, 0.5 },rate = {8, 30, 0}})
				local mpBonus = 0
				local drained_units = {}
				for i, cell in pairs(cells) do
					for _, unit in pairs(cell.units) do
						if unit:getTraits().isAgent and unit:getPlayerOwner():isPC() and not unit:getTraits().isDrone and unit:getTraits().mp and unit:getTraits().mp > 0 then
							mpBonus = mpBonus + math.min(userUnit:getTraits().mp_siphon,unit:getTraits().mp)
							drained_units[unit] = math.min(userUnit:getTraits().mp_siphon,unit:getTraits().mp)
							unit:addMP(-userUnit:getTraits().mp_siphon)
						end
					end
				end
				sim:dispatchEvent("EV_CE_WAIT",60)
				if mpBonus > 0 then
					userUnit:addMPMax(mpBonus)
					sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=string.format(STRINGS.MOD_CE.UI.SIPHONED_ENERGY,math.floor(mpBonus * 2)/2),x=x,y=y,color={r=0.5,g=1,b=1,a=1 }} )
					sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = userUnit } )
					sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound = "SpySociety_DLC001/Actions/flackguard_armourincrease",x=x,y=y} )
					for unit, mp in pairs(drained_units) do
						local x0, y0 = unit:getLocation()
						sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=string.format(STRINGS.MOD_CE.UI.LOST_ENERGY,math.floor(mp * 2)/2),x=x0,y=y0,color={r=1,g=0,b=0,a=1 }} )
						sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = unit } )
					end
				end
			end
		end
	},
	
	ce_ko_weak = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.KO_WEAK.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.KO_WEAK.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			self.setKO = unit.setKO
			unit.setKO = function( userUnit, sim, ticks, fx )
				return self.setKO( userUnit, sim, ticks and ticks + 2, fx )
			end
		end,

		onDespawnAbility = function( self, sim, unit )
		end,
	},
	
	ce_keibi_grid_uplink = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.KEIBI_GRID_LINK.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.KEIBI_GRID_LINK.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_DAEMON_INSTALL, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_NEWTARGET, self, unit )
			sim:addTrigger( simdefs.TRG_UNIT_ALERTED, self, unit )
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_DAEMON_INSTALL, self )
			sim:removeTrigger( simdefs.TRG_UNIT_NEWTARGET, self )
			sim:removeTrigger( simdefs.TRG_UNIT_ALERTED, self )
		end,
		
		useCPUs = function() end,
		setCooldown = function() end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_DAEMON_INSTALL then
				userUnit:addMPMax(userUnit:getTraits().keibi_bonus)
				userUnit:addMP(userUnit:getTraits().keibi_bonus)
				
				local x0, y0 = userUnit:getLocation()
				sim:dispatchEvent( simdefs.EV_GAIN_AP, { unit = userUnit } )
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.KEIBI_BONUS,x=x0,y=y0,color={r=255/255,g=255/255,b=51/255,a=1 }} )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )
			elseif (evType == simdefs.TRG_UNIT_NEWTARGET and evData.unit == userUnit) or (evType == simdefs.TRG_UNIT_ALERTED and evData.unit == userUnit) then
				if userUnit:isValid() and not userUnit:isDown() then
					local possibleUnits = {}
					for _, unit in pairs( sim:getAllUnits() ) do
						if unit:getTraits().mainframe_item and unit:getTraits().mainframe_ice and unit:getTraits().mainframe_ice > 0 and unit:getPlayerOwner() ~= sim:getPC() and not unit:getTraits().mainframe_program then
							table.insert( possibleUnits, unit )		
						end
					end
					
					if #possibleUnits > 0 then
						local x0, y0 = userUnit:getLocation()
						local unit = simquery.findClosestUnit( possibleUnits, x0, y0 )
						
						local programList = sim:getIcePrograms()
						unit:getTraits().mainframe_program = programList:getChoice( sim:nextRand( 1, programList:getTotalWeight() ))

						local x, y = unit:getLocation()
						sim:dispatchEvent( simdefs.EV_CAM_PAN, { x, y } )

						sim:getPC():glimpseUnit( sim, unit:getID() )
						sim:dispatchEvent( simdefs.EV_UNIT_MAINFRAME_UPDATE, {units = {unit.unitID}, reveal=true} )
						sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = unit, ice = unit:getTraits().mainframe_ice, delta = 0, refreshAll = true} )
						sim:dispatchEvent(simdefs.EV_MAINFRAME_INSTALL_NEW_DAEMON, {target=unit})
						sim:dispatchEvent( simdefs.EV_PLAY_SOUND, "SpySociety/Actions/mainframe_daemonmove")	
					end
				end
			end
		end
	},
	
	ce_alarm_sync = util.extend( DEFAULT_BUFF )
	{
		name = STRINGS.MOD_CE.RESEARCH.ALARM_SYNC.NAME, 
		buffDesc = STRINGS.MOD_CE.RESEARCH.ALARM_SYNC.UNIT_DESC,

		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self, unit )	
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_ALARM_STATE_CHANGE, self )
		end,

		onTrigger = function( self, sim, evType, evData, userUnit )
			if evType == simdefs.TRG_ALARM_STATE_CHANGE then 
				local x0, y0 = userUnit:getLocation()
				userUnit:addMPMax( 1 )
				userUnit:addMP( 1 )
				
				sim:dispatchEvent( simdefs.EV_UNIT_FLOAT_TXT, {txt=STRINGS.MOD_CE.UI.ALARM_SYNC,x=x0,y=y0,color={r=255/255,g=255/255,b=255/255,a=1 }} )
				sim:dispatchEvent( simdefs.EV_UNIT_ADD_FX, { unit = userUnit, kanim = "fx/firewall_buff_fx_2", symbol = "character", anim = "in", above=true, params={color ={{symbol="wall",r=1,g=0,b=170/255,a=1}} }} )
				sim:dispatchEvent( simdefs.EV_PLAY_SOUND, {sound = "SpySociety_DLC001/Actions/flackguard_armourincrease",x=x0,y=y0} )
				sim:dispatchEvent( simdefs.EV_UNIT_REFRESH, { unit = userUnit } )			
			end 
		end, 
	},
	
	ce_gun_calm = util.extend( DEFAULT_BUFF )
	{
		onSpawnAbility = function( self, sim, unit )
			sim:addTrigger( simdefs.TRG_UNIT_SHOT, self, unit )		
		end,

		onDespawnAbility = function( self, sim, unit )
			sim:removeTrigger( simdefs.TRG_UNIT_SHOT, self )
		end,

		onTrigger = function( self, sim, evType, evData, equipped )
			if evType == simdefs.TRG_UNIT_SHOT and simquery.getEquippedGun(evData.sourceUnit) == equipped then 
				evData.targetUnit:getTraits().ce_suspicious = -math.huge
				evData.targetUnit:getTraits().ce_calmed = true
			end 
		end, 
	}
}
return passive_abilities