local Pather = include("sim/pather")
local util = include("modules/util")
local simdefs = include("sim/simdefs")
local guarddefs = include( "sim/unitdefs/guarddefs" )

local oldInit = Pather.init

function Pather:init(...)
	oldInit(self,...)
	
	self._cellReserves = {}
end

function Pather:reserveNode(node, prevNode, path)
	local reservation = {node=prevNode, path=path}
	local x, y = node.location.x, node.location.y
	self._pathReserves[ node.lid ] = reservation
	
	local cellLid = self:getNodeLid( x, y, 0 )
	if not self._cellReserves[cellLid] then
		self._cellReserves[cellLid] = {}
	end
	
	local cellReserves = self._cellReserves[cellLid]
	for i = 1, #cellReserves + 1 do
		if i > #cellReserves or node.t < cellReserves[i] then
			table.insert(cellReserves, i, node.t)
			break
		end
	end
	
	local stationaryLid = self:getNodeLid( x, y, -2 )
	
	if self._pathReserves[stationaryLid] and self._pathReserves[stationaryLid].path ~= path then
		self:invalidatePath( self._pathReserves[stationaryLid].path.unit )
	end
end

function Pather:unreserveNode(path, node)
	if self._pathReserves[ node.lid ] and self._pathReserves[ node.lid ].path == path then
		local x, y = node.location.x, node.location.y
		self._pathReserves[ node.lid ] = nil
		
		local endLid = self:getNodeLid( x, y, -1 )
		local stationaryLid = self:getNodeLid( x, y, -2 )
		if self._pathReserves[endLid] and self._pathReserves[endLid].path == path then
			self._pathReserves[endLid] = nil
		end
		if self._pathReserves[stationaryLid] and self._pathReserves[stationaryLid].path == path then
			self._pathReserves[stationaryLid] = nil
		end
		
		local cellLid = self:getNodeLid( x, y, 0 )
		local cellReserves = self._cellReserves[cellLid]
		for i = 1, #cellReserves do
			if node.t == cellReserves[i] then
				table.remove(cellReserves, i)
				break
			end
		end
	end
end

function Pather:reservePath(path)
	local prevNode = path.path:getStartNode()
	-- Reserve starting location (t = 0) so that other units needing to path through this location at (t = 1) know
	-- that they must wait for this unit to execute its path first.
	self:reserveNode(prevNode, prevNode, path)
	
	local isStationary = false
	
	if guarddefs["ce_dummy_pather"] then
		isStationary = true
		for i, node in ipairs(path.path:getNodes()) do
			if node.location.x ~= prevNode.location.x or node.location.y ~= prevNode.location.y then
				isStationary = false
				break
			end
		end
	end
	
	if isStationary then
		self._pathReserves[self:getNodeLid( prevNode.location.x, prevNode.location.y, -2 )] = {path=path,node=prevNode}
	else
		for i, node in ipairs(path.path:getNodes()) do
			assert( self._pathReserves[ node.lid ] == nil )
			self:reserveNode(node, prevNode, path)
			prevNode = node
		end
		
		if prevNode then
			path.endNode = prevNode
			self._pathReserves[self:getNodeLid( prevNode.location.x, prevNode.location.y, -1 )] = {path=path,node=prevNode}
		end
	end
end

function Pather:calculatePathPriority(path)
	local priority = 0
	if path.path then
		if path.result == simdefs.CANMOVE_INVALIDATED then
			priority = priority + 1000
		end
		
		local sx = path.path.startNode.location.x
		local sy = path.path.startNode.location.y
		for i, node in ipairs(path.path:getNodes()) do
			local pathStart = self:checkPathStart(node.location.x, node.location.y)
			if pathStart and pathStart.path.unit ~= path.unit then
				priority = priority + 100
			end
			local pathEnd = self:checkPathEnd(node.location.x, node.location.y)
			if pathEnd and pathEnd.path.unit ~= path.unit then
				priority = priority - 100
			end
			for i = 0, node.t-1 do
				local waitPoint = self:checkPathReservation(node.location.x, node.location.y, i)
				if waitPoint and waitPoint.path.unit ~= path.unit then
					priority = priority + 1
				end
			end
			
			if sx ~= node.location.x or sy ~= node.location.y then
				sx = -1
				sy = -1
			end
		end
		
		if guarddefs["ce_dummy_pather"] then
			if sx ~= -1 then
				priority = priority + 50
			end
		end
	end
	return priority
end