local shootOverwatch = include("sim/abilities/shootOverwatch")
local simquery = include("sim/simquery")
local simdefs = include("sim/simdefs")

local oldOnTrigger = shootOverwatch.onTrigger
local oldExecuteAbility = shootOverwatch.executeAbility

shootOverwatch.onTrigger = function( self, sim, evType, evData, userUnit, ... )
	if userUnit:isAiming() and userUnit:getTraits().skipOverwatch and not userUnit.modifiedSetAiming then
		local oldSetAiming = userUnit.setAiming
		userUnit.modifiedSetAiming = true
		userUnit.setAiming = function()
			userUnit.modifiedSetAiming = nil
			userUnit.setAiming = oldSetAiming
		end
	end
	
	local shootingDartAtPedler = false
	if userUnit:getTraits().dartgun and evData and not evData:getTraits().canKO and not evData:getTraits().isDrone and not userUnit:getTraits().hasFailedToKOPedler then
		local weapon = simquery.getEquippedGun( userUnit )
		if weapon then
			weapon:getTraits().canSleep = false
			shootingDartAtPedler = true
			userUnit:getTraits().shootingDartAtPedler = true
		end
	end
	
	oldOnTrigger( self, sim, evType, evData, userUnit, ... )
	
	if shootingDartAtPedler then
		local weapon = simquery.getEquippedGun( userUnit )
		weapon:getTraits().canSleep = true
	end
end

shootOverwatch.executeAbility = function( self, sim, unit, userUnit, targetUnit, ... )
	if userUnit:getTraits().removeAttackAfterUse then
		userUnit:getTraits().hasSkippedAttack = 2
		userUnit:getTraits().skipOverwatch = false
		userUnit:getTraits().noAttack = true
	end
	if userUnit:getTraits().shootingDartAtPedler then
		local weapon = simquery.getEquippedGun( userUnit )
		weapon:getTraits().canSleep = true
		userUnit:getTraits().hasFailedToKOPedler = {
			onTrigger = function( listener, sim, evType, evData )
				if evData:isNPC() then
					sim:removeTrigger( simdefs.TRG_START_TURN, listener )
					userUnit:getTraits().hasFailedToKOPedler = nil
				end
			end
		}
		
		sim:addTrigger( simdefs.TRG_START_TURN, userUnit:getTraits().hasFailedToKOPedler )
	end

	oldExecuteAbility( self, sim, unit, userUnit, targetUnit, ... )
end