local simquery = include( "sim/simquery" )
local simdefs = include( "sim/simdefs" )
local unitdefs = include( "sim/unitdefs" )
local mathutil = include( "modules/mathutil" )

simquery.isCellWatched = function( sim, player, cellx, celly )
	local isWatched, isNoticed ,isHidden = false, false, false

	if sim:canPlayerSee( player, cellx, celly ) then
		local seers = sim:getLOS():getSeers( cellx, celly )
		for _, seerID in ipairs(seers) do
			local seerUnit
			if seerID >= simdefs.SEERID_PERIPHERAL then
				seerUnit = sim:getUnit( seerID - simdefs.SEERID_PERIPHERAL )
			else
				seerUnit = sim:getUnit( seerID )
			end

			if seerUnit and seerUnit:getPlayerOwner() ~= player and simquery.canSeeLOS( sim, player, seerUnit ) then
				if simquery.checkCover( sim, seerUnit, cellx, celly ) and not seerUnit:getTraits().seesHidden and not seerUnit:getTraits().ultrasonic_echolocation then
					isHidden = true
				elseif seerID >= simdefs.SEERID_PERIPHERAL or (simquery.checkCover( sim, seerUnit, cellx, celly ) and not seerUnit:getTraits().seesHidden) then
					isNoticed = true
				else
					isWatched = true
					break
				end
			end
		end
	end

	if isWatched then
		return simdefs.CELL_WATCHED
	elseif isNoticed then
		return simdefs.CELL_NOTICED
	elseif isHidden then
		return simdefs.CELL_HIDDEN
	end
end

simquery.couldUnitSee = function( sim, unit, targetUnit, ignoreCover, targetCell )
	if unit:getTraits().globalVision then
		return true
	end
	if not unit:getTraits().hasSight then
		return false
	end
    if unit:isKO() then
        return false
    end
    if unit:getTraits().grappler then
    	return false
	end
	
	if targetCell and unit:getTraits().noPeripheralDetection then
		return false
	end

	if targetUnit and targetUnit ~= unit then

		if not targetUnit:getLocation() then
			return false
		end

		if targetUnit:getTraits().invisible and targetUnit:getPlayerOwner() ~= unit:getPlayerOwner() and not unit:getTraits().detect_cloak then
			return false
		end
        
        ignoreCover = ignoreCover or unit:isPC()
        if not ignoreCover then
        	if not targetCell then
        		targetCell = sim:getCell(targetUnit:getLocation() )
    		end
		    if not unit:getTraits().seesHidden and targetUnit:canHide() and
		     simquery.checkIfCellNextToCover(sim, targetCell) and simquery.checkCover( sim, unit, targetCell.x, targetCell.y ) then
			    return false
		    end
        end
	end

	return true
end

local function getKeyReason( exit )
	if exit.keybits == simdefs.DOOR_KEYS.ELEVATOR then
		return false, STRINGS.UI.DOORS.ELEVATOR
	elseif exit.keybits == simdefs.DOOR_KEYS.OFFICE then
		return false, STRINGS.UI.DOORS.OFFICE
	elseif exit.keybits == simdefs.DOOR_KEYS.SECURITY then
		return false, STRINGS.UI.DOORS.SECURITY, STRINGS.UI.DOORS.SECURITY_LONGDESC
	elseif exit.keybits == simdefs.DOOR_KEYS.VAULT then
		return false, STRINGS.UI.DOORS.VAULT, STRINGS.UI.DOORS.VAULT_LONGDESC
	elseif exit.keybits == simdefs.DOOR_KEYS.SPECIAL_EXIT then
		return false, STRINGS.UI.DOORS.SPECIAL_EXIT, STRINGS.UI.DOORS.SPECIAL_EXIT_LONGDESC
	elseif exit.keybits == simdefs.DOOR_KEYS.ELEVATOR_INUSE then
		return false, STRINGS.UI.DOORS.INUSE		
	elseif exit.keybits == simdefs.DOOR_KEYS.GUARD then
		return false, STRINGS.UI.DOORS.GUARD
    elseif exit.keybits == simdefs.DOOR_KEYS.DISABLED then
        return false, STRINGS.UI.DOORS.DISABLED
	end
end

simquery.canModifyExit = function( unit, exitOp, cell, dir )
	local exit = cell.exits[ dir ]
	assert( exit and exit.door )

	if unit:getTraits().canUseDoor == false then
		return false, STRINGS.UI.DOORS.CANT_USE
	end
	
	if unit:getTraits().lockDownCount and unit:getTraits().lockDownCount > 0 then
		return false, STRINGS.MOD_CE.UI.CANT_USE_LOCKDOWN
	end

	if unit:getTraits().movingBody then
		return false, STRINGS.UI.DOORS.DROP_BODY
	end
	
	if not unit:canAct() then
		return false
	end

	if (exit.keybits == simdefs.DOOR_KEYS.SECURITY) and unit:isNPC() and unitdefs.lookupTemplate("ce_dummy_doorkeys") then
		return true
	end
	if exit.keybits == simdefs.DOOR_KEYS.GUARD and unit:isPC() then
		return false, STRINGS.UI.DOORS.GUARD
 	end
	if exit.keybits == simdefs.DOOR_KEYS.ELEVATOR_INUSE or exit.keybits == simdefs.DOOR_KEYS.ELEVATOR then		
		return false
	end

	if exitOp == simdefs.EXITOP_OPEN then
		if not exit.closed then
			return false, STRINGS.UI.DOORS.NOT_CLOSED
		elseif exit.locked then
			return getKeyReason(exit)
		end

	elseif exitOp == simdefs.EXITOP_CLOSE then
		if exit.closed then
			return false, STRINGS.UI.DOORS.NOT_OPEN
		elseif exit.locked then
			return false, STRINGS.UI.DOORS.BROKEN
		end

	elseif exitOp == simdefs.EXITOP_TOGGLE_DOOR then
		if exit.locked then
			return false, STRINGS.UI.DOORS.SECURITY
		end

	elseif exitOp == simdefs.EXITOP_BREAK_DOOR then
		if not exit.closed and exit.locked then
			return false, STRINGS.UI.DOORS.BROKEN
		elseif not exit.closed then
			return false, STRINGS.UI.DOORS.NOT_CLOSED
		elseif exit.keybits == simdefs.DOOR_KEYS.GUARD then
			return false, STRINGS.UI.DOORS.CANT_BREAK
		elseif exit.locked then
			return getKeyReason(exit)
		end

	elseif exitOp == simdefs.EXITOP_LOCK then
		if not exit.closed then
			return false, STRINGS.UI.DOORS.NOT_CLOSED
		elseif exit.locked then
			return false, STRINGS.UI.DOORS.ALREADY_LOCKED
		elseif exit.keybits == nil then
			return false, STRINGS.UI.DOORS.NOT_LOCKED
		elseif not simquery.hasKey( unit, exit.keybits ) then
			return getKeyReason(exit)
		end

	elseif exitOp == simdefs.EXITOP_UNLOCK then
		if not exit.closed then
			return false, STRINGS.UI.DOORS.NOT_CLOSED
		elseif not exit.locked then
			return false, STRINGS.UI.DOORS.NOT_LOCKED
		elseif not simquery.hasKey( unit, exit.keybits ) then
			return getKeyReason(exit)
		end

	elseif exitOp == simdefs.EXITOP_TOGGLE_LOCK then
		if not exit.closed then
			return false, STRINGS.UI.DOORS.NOT_CLOSED
		elseif exit.keybits == nil then
			return false, STRINGS.UI.DOORS.NOT_LOCKED
		elseif not simquery.hasKey( unit, exit.keybits ) then
			return getKeyReason(exit)
		end

	elseif exitOp == simdefs.EXIT_DISARM then	
		if unit:getMP() < 3 then
			return false, "NOT ENOUGH AP"
		end
	end

	return true
end

simquery.canPathBetween = function( cellquery, unit, startcell, endcell )
	if endcell == nil then
		return false
	end

	local dx, dy = endcell.x - startcell.x, endcell.y - startcell.y
	if (startcell.x + 1 == endcell.x or startcell.x - 1 == endcell.x) and (startcell.y + 1 == endcell.y or startcell.y - 1 == endcell.y) then
		-- diagonal.  both exits to the diagonal must be open.
		local cell3 = cellquery:getCell( startcell.x, endcell.y )
		local cell4 = cellquery:getCell( endcell.x, startcell.y )
		if cell3 == nil or cell4 == nil then
			return false
		end
		local d1, d2 = simquery.getDirectionFromDelta( cell3.x - startcell.x, cell3.y - startcell.y ), simquery.getDirectionFromDelta( endcell.x - cell3.x, endcell.y - cell3.y )
		local d3, d4 = simquery.getDirectionFromDelta( cell4.x - startcell.x, cell4.y - startcell.y ), simquery.getDirectionFromDelta( endcell.x - cell4.x, endcell.y - cell4.y )

		if not (simquery.isOpenExit( startcell.exits[ d1 ] ) and simquery.isOpenExit( cell3.exits[ d2 ] ) and
				simquery.isOpenExit( startcell.exits[ d3 ] ) and simquery.isOpenExit( cell4.exits[ d4 ] )) then
			return false
		end
	
	else
		local dir = simquery.getDirectionFromDelta( dx, dy )
		local exit = startcell.exits[ dir ]
		local found = exit and exit.cell.x == endcell.x and exit.cell.y == endcell.y
		if unit then
			found = found and (not (exit.closed and exit.locked) or simquery.canModifyExit( unit, simdefs.EXITOP_UNLOCK, startcell, dir ))
			found = found and (not exit.closed or simquery.canModifyExit( unit, simdefs.EXITOP_OPEN, startcell, dir ))
		end
		if not found then
			return false
		end
	end
	
	return true
end

function simquery.canSeeLOS( sim, player, seer )
	if player == seer:getPlayerOwner() then
        return true
    end

	if seer:getTraits().invisible then
		--return false
	end

    if sim:getParams().difficultyOptions.dangerZones then
        return true
    end

    if player ~= nil and sim:canPlayerSeeUnit( player, seer ) then
        return true
    end

    return false
end

simquery.findNearestEmptyReachableCell = function( sim, x, y, unit )

	local cell = sim:getCell(x, y)

	local resultCell = nil
	for dx = -1,1 do
	for dy = -1,1 do
		local testCell = sim:getCell( cell.x + dx, cell.y + dy )
		if testCell and simquery.canReach(sim, x, y, testCell.x, testCell.y) and simquery.canPath( sim, unit, nil, testCell ) then
			if not resultCell then
				resultCell = testCell
			elseif unit then
				local unitX,unitY = unit:getLocation()
				if unitX == x and unitY == y then
					--pick a cell close to the current unit facing
					local cell1Dir = simquery.getDirectionFromDelta(testCell.x - unitX, testCell.y - unitY)
					local cell2Dir = simquery.getDirectionFromDelta(resultCell.x - unitX, resultCell.y - unitY)
					if math.abs(simquery.dirDiff(cell1Dir, unit:getFacing() ) ) < math.abs(simquery.dirDiff(cell2Dir, unit:getFacing() ) ) then
						resultCell = testCell
					end
				else
					--pick the closest cell
					local cell1Dist = mathutil.dist2d( testCell.x, testCell.y, unitX, unitY )
					local cell2Dist = mathutil.dist2d( resultCell.x, resultCell.y, unitX, unitY )
					if cell1Dist < cell2Dist then
						resultCell = testCell
					end
				end
			end
		end
	end
	end

	return resultCell
end

local old_isEnemyTarget = simquery.isEnemyTarget

function simquery.isEnemyTarget( player, unit, ignoreDisguise )
	if not old_isEnemyTarget( player, unit, ignoreDisguise ) then
		return false
	end
	if player and player:isPC() and unit:getTraits().invisible then
		return false
	end
	return true
end

function simquery.isEnemyAgent( player, unit, ignoreDisguise)

	if not unit:hasTrait("isAgent") then
		return false
	end
	
	return old_isEnemyTarget( player, unit, ignoreDisguise )
end

local old_findpath = simquery.findPath

function simquery.findPath( sim, unit, startcell, endcell, maxMP, goalFn )
	return old_findpath( sim, unit, startcell, endcell, maxMP, false )
end

function simquery.canPath( cellquery, unit, startcell, endcell )
	-- Check for dynamic impass.  Note that even if dynamicImpass exists, if the unit owner doesn't know it's there, this won't fail.
    if unit and unit:getTraits().lockDownCount and unit:getTraits().lockDownCount > 0 then
		return false, "LOCKED DOWN"
	end
	
	if unit == nil or unit:getTraits().dynamicImpass then
	    for i,cellUnit in ipairs(endcell.units) do
		    if cellUnit:getTraits().dynamicImpass and not (unit and not unit:getTraits().movePath and startcell and unit:isPC() and cellUnit:isNPC() and cellUnit:getTraits().invisible) then
                -- Owned emitters will turn themselves off, so they are not considered impassable.
                if cellUnit:getTraits().emitterID == nil or cellUnit:isGhost() or not unit or not cellUnit:canControl( unit ) then
				    return false, simdefs.CANMOVE_DYNAMIC_IMPASS
			    end
		    end
	    end
    end
	
	return simquery.canStaticPath( cellquery, unit, startcell, endcell )
end

