local npc_abilities = include("sim/abilities/npc_abilities")
local simunit = include("sim/simunit")
local simdrone = include( "sim/units/simdrone" )
local simdefs = include( "sim/simdefs" )
local unitdefs = include( "sim/unitdefs" )

local oldRecapture = simunit.recaptureMainframeItems
local oldprocessEMP = simunit.processEMP
local oldprocessEMPdrone = simdrone.processEMP
local oldProcessKOresist = simunit.processKOresist
local oldSetKO = simunit.setKO

simunit.recaptureMainframeItems = function(self,sim,range)
	oldRecapture(self,sim,range)
	if self:getTraits().hardmode then
		--oldRecapture(self,sim,range)
	end
end

function simunit:processEMP( bootTime, ... )
	if bootTime and self:getTraits().temporaryArmor then
		self:getTraits().temporaryArmor = self:getTraits().temporaryArmor - 2 * bootTime
		if self:getTraits().temporaryArmor <= 0 then
			self:getTraits().temporaryArmor = nil
		end
	end
	return oldprocessEMP( self, bootTime, ... )
end
function simdrone:processEMP( bootTime, ... )
	if bootTime and self:getTraits().temporaryArmor then
		self:getTraits().temporaryArmor = self:getTraits().temporaryArmor - 2 * bootTime
		if self:getTraits().temporaryArmor <= 0 then
			self:getTraits().temporaryArmor = nil
		end
	end
	return oldprocessEMPdrone( self, bootTime, ... )
end

function simunit:getArmor( )
	local armor = 0
	
	if self:getTraits().armor then
		armor = armor + self:getTraits().armor
	end
	
	if self:getTraits().temporaryArmor then
		armor = armor + math.ceil(self:getTraits().temporaryArmor * 0.5)
	end

	if self:getPlayerOwner() and self:getPlayerOwner():getTraits().boostArmor then
		armor = armor + self:getPlayerOwner():getTraits().boostArmor
	end
	
	if self:getTraits().firewallArmor then
		if unitdefs.lookupTemplate("ce_sk_drone_barrier") then
			local iceArmor = self:getTraits().mainframe_ice - (self:getTraits().weakBarrier or 0)
		
			if iceArmor > 0 and self:getTraits().mainframe_status ~= "off" and not self:isKO() then
				armor = armor + iceArmor
			end
		else
	
			if self:getTraits().mainframe_ice > 0 and self:getTraits().mainframe_ice > armor and self:getTraits().mainframe_status ~= "off" then
				armor = self:getTraits().mainframe_ice
			end
		end
	end

	if self:getPlayerOwner() then 
		if self:getPlayerOwner():isNPC() then
			if self:getSim():getPC():getTraits().taggedArmorMod and self:getTraits().tagged then
				armor = math.max( armor + self:getSim():getPC():getTraits().taggedArmorMod, 0)
			end
			if self:getPlayerOwner():getTraits().armorBuff then
				armor = armor + self:getPlayerOwner():getTraits().armorBuff 
			end
			if npc_abilities["transistordaemonred"] and self:getPlayerOwner():hasMainframeAbility("transistordaemonred") then
				armor = math.floor(armor * 0.5)
			end
		end
	end

	return armor
end

function simunit:increaseIce(sim,iceInc)
	if self:getTraits().mainframe_iceMax then
		sim:dispatchEvent( simdefs.EV_UNIT_UPDATE_ICE, { unit = self, ice = self:getTraits().mainframe_ice, delta = iceInc } )

		local raiseIce = true
		if self:getTraits().noRaiseIfIceZero and self:getTraits().mainframe_ice <= 0 then
			raiseIce = false
		end

		if self:getTraits().mainframe_iceMax and not self:getTraits().mainframe_console and ( not self:getPlayerOwner() or not self:getPlayerOwner():isPC() ) then

			if raiseIce then
				self:getTraits().mainframe_ice = self:getTraits().mainframe_ice+iceInc
				sim:triggerEvent( simdefs.TRG_ICE_BROKEN, { unit = self, delta = iceInc } )
			end

			self:getTraits().mainframe_iceMax = self:getTraits().mainframe_iceMax+iceInc					
		end
	end
end

function simunit:processKOresist( ... )
	if self:getTraits().canKO then
		return oldProcessKOresist( self, ... )
	else
		return 0
	end
end

function simunit:setKO( sim, ticks, ... )
	if ticks == 0 then
		return
	end
	
	return oldSetKO( self, sim, ticks, ... )
end