local cdefs = include("client_defs")
local unitrig = include( "gameplay/unitrig" )
local util = include( "modules/util" )
local mathutil = include( "modules/mathutil" )

local melee_state = class( unitrig.base_state )

function melee_state:init( rig )	
	unitrig.base_state.init( self, rig, "ce_melee" )		
end

function melee_state:onEnter( ev )
	unitrig.base_state.onEnter( self )

	local unit = self._rig:getUnit()
	local targetUnitID = ev.eventData.targetUnit:getID()
	local targetRig = self._rig._boardRig:getUnitRig(targetUnitID)

	self._rig:playSound("SpySociety/HitResponse/hitby_punch_flesh")
	self:waitForAnim( "melee" )

	local thread = ev.viz:spawnViz(
		function()
			self:waitForAnim( "melee_pst" )
			self:waitForAnim( "idle_pre" )
			self._rig:transitionUnitState( self._rig._idleState )
		end )
	thread:unblock()

end

local function melee( event, evType, eventData, boardRig, hud, vizThread )
	if eventData.finishMelee then
		boardRig._game:getCamera():restoreMemento( 0.5 )
		boardRig:wait( 60 )
	elseif eventData.unit:isValid() then
		boardRig:cameraLock( nil )

		local unitRig = boardRig:getUnitRig( eventData.unit:getID() )
		local targetRig = boardRig:getUnitRig( eventData.targetUnit:getID() )

	   	local x0,y0 = unitRig:getLocation()
	   	local x1,y1 = targetRig:getLocation()

	   	if x0 and y0 then
			boardRig:cameraCenterTwoPoints(x0,y0,x1,y1)
			MOAIFmodDesigner.playSound( "SpySociety/HUD/gameplay/markedForDeath_zoom")
		end	

		local blackout = include( "gameplay/viz_handlers/blackout" )
		local th = blackout( boardRig, event.viz, 180, targetRig, unitRig )
		th:unblock()
   		event.viz:addThread( th )

		boardRig:wait( 15 )	
		if unitRig then
			targetRig:transitionUnitState( targetRig._idleState )
			unitRig:refreshLocation()	
			unitRig._meleeState = unitRig._meleeState or melee_state(unitRig)
			unitRig:transitionUnitState( unitRig._meleeState, event )	
		end
	end
end

return melee