local worldgen = include( "sim/worldgen" )
local serverdefs = include( "modules/serverdefs" )
local util = include( "modules/util" )
local weighted_list = include( "modules/weighted_list" )
local unitdefs = include( "sim/unitdefs" )
local array = include( "modules/array" )
local simdefs = include( "sim/simdefs" )

local admins = {
	ftm = { SYS_ADMIN = "FTM_ADMIN", SYS_ADMIN_NEW = "FTM_ADMIN_NEW" },
	ko = { SYS_ADMIN = "KO_ADMIN", SYS_ADMIN_NEW = "KO_ADMIN_NEW" },
	sankaku = { SYS_ADMIN = "SANKAKU_ADMIN", SYS_ADMIN_NEW = "SANKAKU_ADMIN_NEW" },
	plastech = { SYS_ADMIN = "PLASTECH_ADMIN", SYS_ADMIN_NEW = "PLASTECH_ADMIN_NEW" },
}

function worldgen.generateThreats( cxt, spawnTable, spawnList )
	local omni = false
    if not spawnList then
        spawnList = simdefs.SPAWN_TABLE[ cxt.params.difficultyOptions.spawnTable ][ cxt.params.difficulty ]
	elseif spawnList == simdefs.OMNI_SPAWN_TABLE[ cxt.params.difficultyOptions.spawnTable ] and simdefs.OMNI_SPAWN_TABLE_SCALING[ cxt.params.difficultyOptions.spawnTable ] then
		if unitdefs.lookupTemplate("ce_dummy_omni") then
			spawnList = simdefs.OMNI_SPAWN_TABLE_SCALING[ cxt.params.difficultyOptions.spawnTable ][ cxt.params.difficulty ]
			omni = true
		elseif unitdefs.lookupTemplate("ce_omni_hacker") then
			spawnList = simdefs.OMNI_SPAWN_TABLE_SCALING[ cxt.params.difficultyOptions.spawnTable ][ 5 ]
			omni = true
		end
    end
	
	spawnTable = util.tdupe(spawnTable)
	spawnList = util.tdupe(spawnList)
	
	if admins[ cxt.params.world ] then
		spawnTable.SYS_ADMIN = worldgen[ admins[ cxt.params.world ].SYS_ADMIN ]
		spawnTable.SYS_ADMIN_NEW = worldgen[ admins[ cxt.params.world ].SYS_ADMIN_NEW ]
	end
	
	local adminType = "SYS_ADMIN"
	if spawnTable.SYS_ADMIN_NEW and (cxt.params.difficultyOptions.ce_sysadmins == 0 or (cxt.params.difficultyOptions.ce_sysadmins == 2 and cxt.rnd:nextInt(1,2) == 1)) then
		adminType = "SYS_ADMIN_NEW"
	end
	if cxt.params.difficultyOptions.ce_sysadmins ~= 3 and unitdefs.lookupTemplate("ce_ftm_sysadmin") and spawnTable[adminType] and next( spawnTable[adminType] ) then
		table.insert(spawnList, adminType)
		for i, v in ipairs(spawnList) do
			if v == "COMMON" then
				spawnList[i] = adminType
				spawnList[#spawnList] = nil
				break
			end
		end
	end
	
	if unitdefs.lookupTemplate("ce_sankaku_engineer_LV2") and spawnTable.OMNI_HUMAN then
		spawnTable.OMNI_HUMAN = util.tdupe(worldgen.OMNI_GUARD_FIX)
		table.insert(spawnTable.OMNI_HUMAN,{"ce_sankaku_engineer_LV2",400})
	end
	
	-- Insert OMNI drone
	if worldgen.CE_OMNI_DRONE and (omni or cxt.params.difficulty > 5) then
		local wt = util.weighted_list(worldgen.CE_OMNI_DRONE)
		if wt:getTotalWeight() > 0 then
			spawnTable.CE_OMNI_DRONE = worldgen.CE_OMNI_DRONE
			local droneCount = 0
			local mustSpawn = false
			if omni then
				table.insert(spawnList,"CAMERA_DRONE")
				mustSpawn = true
			end
			for i, v in ipairs(spawnList) do
				if (v == "CAMERA_DRONE") then
					droneCount = droneCount + 1
				end
			end
			
			if droneCount < 2 and not omni and cxt.rnd:nextInt(1,3) == 1 then
				if array.find(spawnList,"OMNI") and (cxt.rnd:nextInt(1,3) > 1 or not array.find(spawnList,"ELITE")) then
					array.removeElement( spawnList, "OMNI" )
					table.insert(spawnList,"CAMERA_DRONE")
					table.insert(spawnList,"CAMERA_DRONE")
					table.insert(spawnList,"ELITE")
					droneCount = droneCount + 2
					mustSpawn = true
				elseif array.find(spawnList,"ELITE") then
					array.removeElement( spawnList, "ELITE" )
					table.insert(spawnList,"CAMERA_DRONE")
					table.insert(spawnList,"COMMON")
					droneCount = droneCount + 1
					mustSpawn = true
				end
			end
			while droneCount >= 2 do
				local r = cxt.rnd:nextInt(1,3)
				if mustSpawn or r > 1 then
					mustSpawn = false
					array.removeElement( spawnList, "CAMERA_DRONE" )
					array.removeElement( spawnList, "CAMERA_DRONE" )
					droneCount = droneCount - 2
					table.insert(spawnList,"CE_OMNI_DRONE")
				else
					break
				end
			end
		end
	end
	
	local spawnedThreaths = {}
	local spawnIndex = 1
	local roomsLeft = #cxt.rooms
    local zoneThreats = {} -- map of zoneID to threats.
    for spawnIndex = 1, #spawnList do
		-- Find a spawn location
        local room, attempts = nil, 0
        local x, y = nil, nil
        while (x == nil or y == nil) and attempts < 20 do
            room = worldgen.findThreatRoom( cxt, zoneThreats )
            if room then
                x, y = worldgen.findGuardSpawn( cxt, zoneThreats[ room.zoneID ], room )
            end
            attempts = attempts + 1
        end
		
		if x and y then
			local unitType = spawnList[ spawnIndex ]
			local list = spawnTable[ unitType ]
			if list then
				list = util.tdupe( list )
				if (omni and unitType == "OMNI") or list._unique then
					-- Avoid duplicate guard types in OMNI
					-- Find the smallest available instance count
					local smallestCount = math.huge
					for i, v in pairs(list) do
						local templateName = type(i) == "string" and i or list[i][1]
						local spawnCount = spawnedThreaths[templateName] or 0
						if spawnCount < smallestCount then
							smallestCount = spawnCount
						end
					end
					-- Remove templates above that count
					for i, v in pairs(list) do
						local templateName = type(i) == "string" and i or list[i][1]
						local spawnCount = spawnedThreaths[templateName] or 0
						if spawnCount > smallestCount then
							list[i] = nil
						end
					end
					-- If an ordered list, remove any gaps
					if type(next(list)) == "number" then
						local newlist = {}
						for i, v in pairs(list) do
							newlist[#newlist + 1] = v
						end
						list = newlist
					end
				end
				
				-- Select template
				local spawns = util.weighted_list( list )
				if spawns:getTotalWeight() < 1 then
					log:write( "WARNING: temporary spawn list weight is %s %s", tostring( spawns:getTotalWeight() ), util.stringize( list, 2 ) )
					spawns = util.weighted_list( spawnTable[ unitType ] )
				end
				
				if spawns:getTotalWeight() > 0 then
					local templateName = spawns:getChoice( cxt.rnd:nextInt( 1, spawns:getTotalWeight() ))
					
					--Finalize unit
					local unit =
					{
						x = x,
						y = y,
						template = templateName,
						spawnTable = unitType
					}

					spawnedThreaths[templateName] = (spawnedThreaths[templateName] or 0) + 1
					
					if worldgen.isGuard( unit ) then
						unit = worldgen.finalizeGuard( cxt, unit )
					end
					if unit then
						if zoneThreats[ room.zoneID ] == nil then
							zoneThreats[ room.zoneID ] = {}
						end
						table.insert( zoneThreats[ room.zoneID ], unit )
						table.insert( cxt.units, unit )
					end
				else
					log:write( "ERROR: couldn't select a template, spawn list weight is %s %s", tostring( spawns:getTotalWeight() ), util.stringize( list, 2 ) )
				end
			else
				log:write( "ERROR: spawn table doesn't contain any unit templates in the category [%s] %s", tostring( unitType ), util.stringize( spawnTable, 3 ) )
			end
		else
			log:write( "ERROR: couldn't place unit anywhere in room %s", tostring(room and room.roomIndex) )
		end
    end
end

local function generateThreatsWrapper( ... )
	return worldgen.generateThreats( ... )
end

local upvName, upvVal
local upvIdx = 0
repeat
	upvIdx = upvIdx + 1
	upvName, upvVal = debug.getupvalue(worldgen.worlds.ftm.generateUnits, upvIdx)
until ( upvName == nil or upvName == "generateThreats" )

debug.setupvalue( worldgen.worlds.ftm.generateUnits, upvIdx, generateThreatsWrapper )